*CMZ :          24/03/96  14.13.11  by  Hartwig Albrecht
*CMZ :  2.01/01 20/12/95  14.56.42  by  Hartwig Albrecht
*-- Author :    Hartwig Albrecht   20/10/95

      SUBROUTINE ALMTRA

************************************************************************
*
* Description: fill MTRA and MVER etc from LUJETS
*
* Called from: APEVNT, AFEVNT
*
* External references:
*
* Argument in/out description: none
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "jets/LUJETS.fi"
#include "auto/MTRA.fi"
#include "auto/MTRX.fi"
#include "auto/MVER.fi"
#include "auto/MVEX.fi"
#include "auto/TOPP.fi"
#include "auto/MCME.fi"

      INTEGER I, J, IM, JM, TPLUAR

#include "arte/Arte.ff"
#include "auto/MTRA.ff"
#include "auto/MTRX.ff"
#include "auto/MVER.ff"
#include "auto/MVEX.ff"
#include "auto/TOPP.ff"
#include "auto/MCME.ff"

      IF (JMVER .EQ. 0)  CALL MMBOOK (JMVER, 0)
      IF (JMTRA .EQ. 0)  CALL MMBOOK (JMTRA, 0)
      J = NMTRA()

      DO 10 I = 1, N

C particles rejected in ALPYTH:

        IF (I_V(1,I,JTEMP) .LT. 0)  GO TO 10

        IF (K(I,1) .EQ. 0)  GO TO 10
        IF (K(I,1) .EQ. 13)  GO TO 10
        IF (K(I,1) .GE. 21)  GO TO 10

C check particle code

        IQ(L_A(2,I,JTEMP)) = TPLUAR (K(I,2))
        IF (IQ(L_A(2,I,JTEMP)) .EQ. 0)  THEN
          C78 = 'ALMTRA: particle with unknown Lund code'
          CALL CH78BI (K(I,2))
          CALL ME78WC (9763124)
          GO TO 10
        END IF

C this is the chance to skip unnecessary particles from LUJETS

C ... future: IF (rejection condition fulfilled)  GO TO 10

C lund particle accepted

        J = J + 1
        IQ(L_A(1,I,JTEMP)) = J

 10   CONTINUE

      CALL MMBOOK (JMTRA, J - NMTRA())

      DO 70 I = 1, N
        J = I_V(1,I,JTEMP)
        IF (J .LE. 0)  GO TO 70

C momentum / mass (NOT energy !)

        Q(QMTRA_PX(J)) = P(I,1)
        Q(QMTRA_PY(J)) = P(I,2)
        Q(QMTRA_PZ(J)) = P(I,3)
        Q(QMTRX_MASS(J)) = P(I,5)

C track length

        IF (V(I,5) .NE. 0.)  THEN
C length in cm:
          Q(QMTRX_LENG(J)) = V(I,5) * 0.1 *
     +      SQRT ((P(I,4) + P(I,5)) * (P(I,4) - P(I,5))) / P(I,5)
        ELSE IF (K(I,4) .NE. 0 .AND. K(I,1) .NE. 3 .AND.
     +    K(I,1) .NE. 13 .AND. K(I,1) .NE. 14)  THEN
          Q(QMTRX_LENG(J)) = 0.1 * SQRT ((V(I,1) - V(K(I,4),1))**2 +
     +      (V(I,2) - V(K(I,4),2))**2 + (V(I,3) - V(K(I,4),3))**2)
        END IF

C default length for undecayed particles

        IF (K(I,1) .EQ. 1 .AND. FMTRX_LENG(J) .EQ. 0.)
     +    Q(QMTRX_LENG(J)) = 9999.999
C qmtra_ml may be overwritten later on
        Q(QMTRA_ML(J)) = FMTRX_LENG(J)

C Arte particle code

        CALL MMBREL (JMTTO, I_V(2,I,JTEMP), J)

        IF (K(I,1) .EQ. 4)  THEN

C particle is expected to decay with a given decay length

          IQ(IQMTRA_FLAG(J)) = 1

        ELSE IF (K(I,1) .GE. 5)  THEN

C decayed particle: weak decay or resonance/em decay

          IF (FTOPP_WID(I_V(2,I,JTEMP)) .GT. 1E-9 .OR.
     +        FTOPP_WID(I_V(2,I,JTEMP)) .EQ. 0.)  THEN
            IQ(IQMTRA_FLAG(J)) = 2
            Q(QMTRX_LENG(J)) = 0.
            Q(QMTRA_ML(J)) = FMTRX_MASS(J)
          ELSE
            IQ(IQMTRA_FLAG(J)) = 3
          END IF

C other undecayed particles:  IMTRA_FLAG = 0

        END IF

C mother - daughter relation

        IM = K(I,3)
 20     IF (IM .EQ. I .OR. IM .EQ. 0)  GO TO 30
        JM = I_V(1,IM,JTEMP)
        IF (JM) 30,40,50

C primary vertex

 30     IF (OMCME_MVER(NMCME()) .EQ. 0)  THEN
          CALL MMBOOK (JMVER, 1)
          CALL MMBREL (JMVME, NMVER(), NMCME())
        END IF

        CALL MMBREL (JMVMT, OMCME_MVER(NMCME()), J)
        IQ(IQMTRA_ORIG(J)) = OMCME_MVER(NMCME())
        GO TO 70

C track IM discarded:

 40     IM = K(IM,3)
        GO TO 20

C decay

 50     CALL MMBREL (JMTMT, JM, J)

        IF (IMTRA_FLAG(JM) .EQ. 2)  THEN

C mother is a resonance - same vertex

          CALL MMBREL (JMVMT, OMTRX_BVER(JM), J)

C IMTRA_ORIG < 0: track points to a resonance (in MTRA) and not to MVER

          IQ(IQMTRA_ORIG(J)) = - JM

        ELSE IF (PMTRX_EVER(JM) .EQ. 0)  THEN

C new decay vertex

          CALL MMBOOK (JMVER, 1)
          Q(QMVER_X_L()) = V(I,1) * 0.1
          Q(QMVER_Y_L()) = V(I,2) * 0.1
          Q(QMVER_Z_L()) = V(I,3) * 0.1
          IQ(IQMVER_FLAG_L()) = 2

          IQ(IQMTRA_ORIG(J)) = NMVER()
          CALL MMBREL (JMVMT, NMVER(), J)
          CALL MMBREL (JMTMV, JM, NMVER())

        ELSE

C existing decay vertex

          IQ(IQMTRA_ORIG(J)) = MMTRX_EVER(PMTRX_EVER(JM))
          CALL MMBREL (JMVMT, MMTRX_EVER(PMTRX_EVER(JM)), J)

        END IF

 70   CONTINUE

      END

************************************************************************
* End of ALMTRA
************************************************************************
