*-- Author :    Jenny Ivarsson   11/09/96

      SUBROUTINE ALSPEC

************************************************************************
*
* Description: Treat the special decay (for example golden J/psi K-short)
*              Before PYEVNT the selected particle (for example B0) and
*              its antiparticle were not allowed to decay.
*              Now, tables are changed and the decays are pursued properly
*              (taking mixing into account). Tables are then changed back
*              to the default values
*
* Called from: ALPYTH
*
* External references:
*
* Argument in/out description
*
************************************************************************

      IMPLICIT NONE

#include "arte/ZDECAY.fi"

      INTEGER I,particle,channel,gKC, special,
     +        idc(maxfin), oMDCY(maxfin), LUCOMP, oMDME(maxfin,5)


#include "arte/Arte.fi"
#include "jets/LUDAT3.fi"
#include "jets/LUJETS.fi"
#include "arte/Arte.ff"
c----------------------------------------------------
C Find the selected particle and give it the flag "decayed",
C although it has not. (This is a legal trick to fool the program)

      special = 0

      DO I = 1, N
         IF(K(I,2).eq.gKF(abs(meson)).and.K(I,1).eq.1) THEN
	   K(I,1) = 11
	   special = I
	   goto 10		! Stop searching as soon as 1 meson is found
	 ENDIF
      ENDDO

10    continue

C Allow all particles to decay, including selected particles.
C Particles with flag "decayed" will of course not decay...

      gKC = LUCOMP(gKF(abs(meson)))
      MDCY(gKC,1) = 1

      CALL LUEXEC

C Put back the selected particle to its undecayed state

      IF(special.ne.0) THEN
         if(K(special,2).eq.gKF(abs(meson))) K(special,1) = 1
      ENDIF

C--> Change the decay table for the selected particle and its decay products

c Loop over particles for which special decay channels have been requested

      do particle = 1, maxfin

         if(gMDCY(particle).ne.0) then

    	   gKC = LUCOMP(gKF(particle))		! Save:
	   idc(particle)   = MDCY(gKC,2)	! Entry in the decay table
	   oMDCY(particle) = MDCY(gKC,3)	! Number of decay channels

c Loop over number of selected decay channels

           do channel = 1, gMDCY(particle)

c Save the old information on the channel which will become "special"

              oMDME(particle,channel) = MDME(idc(particle)-1+channel,1)

c Switch on the special channel

              MDME(idc(particle)-1+channel,1) = 1

           enddo

c Switch off all other channels

           do channel = gMDCY(particle),oMDCY(particle)-1
             MDME(idc(particle)+channel,1) = 0
           enddo

         endif

      enddo

c--> Execute LUEXEC for the selected particle only

      call LUEXEC


c--> Change back decay tables.

      do particle = 1, maxfin

         if(gMDCY(particle).ne.0) then

         do channel = 1, gMDCY(particle)
            MDME(idc(particle)-1+channel,1) = oMDME(particle,channel)
         enddo

         do channel = gMDCY(particle),oMDCY(particle)-1
            MDME(idc(particle)+channel,1) = 1
         enddo

         endif

      enddo

c
c      call LULIST(1)

      END

************************************************************************
* End of ALSPEC
************************************************************************
