//
// This file is part of the C++ interface for Arte
// the HERA-B reconstruction and analysis tool
//
// This is the definition of the ArtePointer class.
//
// ArtePointer allows access to the Arte data tables in
// a pointer-like way.
//
//
// .SS Usage // A Pointer to an GESL is declared as
//
// .ft B
//      ArtePointer<GESL> geslPoint;
//
//
// .SS See Also
//
// ArteStructures.hh
//
// Author: Christoph Borgmeier
//

#ifndef _ARTE_POINTER_HH_
#define _ARTE_POINTER_HH_

//#ifdef __GNUC__
//#pragma interface
//#endif

//! Outside Coomment

#include <string.h>
#include <arte/ArteProto.hh>

#include "arte/PointerBase.hh"

template <class T> class ArteRelationIterator;

template <class T>
class ArtePointer: public ArtePointerBase
{
//: Class for accessing entries in Arte tables.
// The ArtePointer imitates the behaviour of a normal C pointer to an object of type T.
public:
  inline ArtePointer( const ArtePointerBase& i );
  inline ArtePointer( const ArtePointer<T>& i );

  inline ArtePointer( unsigned int i);
  //: Create a Pointer to the ith objects in table "T".
  // example cout << ArtePointer<MTRA>(5)->pz << endl;
  // That's it. .bb xxxx .S xxx .

  //  inline ArtePointer( const ArteRelationPointer<T> rp );

  inline ArtePointer();

  inline T & operator*() const;
  //: Dereferences the pointer.
  // Detailed comment ( I don't understand it...).

  inline T * operator->() const;
  // Dereferences the pointer.

  //inline const T& operator=( const ArtePointer<T>& arg );

  inline static void removeAll();

  inline static ArtePointer<T> append( const T& obj );

  inline static unsigned int nrow();
  // Returns the number of entries in the corresponding Arte table.

  inline static ArteLink & getLink();

protected:
  inline static int relType();
  // Returns the Arte Link.

protected:
  inline static int ncol();
  // Return the width of the Arte table

  inline static int* top();
};

#include "arte/Pointer.icc"

#endif
