//                              -*- Mode: C++ -*-
//: CmdBase.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#include "care/CmdBase.hh"
#ifdef USEKUIP
  extern "C"  { void kugetr_(float *); }
  extern "C"  { void kugeti_(int *); }
#endif

/*
CLASS
  CmdBase
  Abstract base class to define an interface for the commands
  issued throu the Kuip interface or throu other ways (to be defined)
  The flag USEKUIP is used to select the KUIP interface. 


class CmdBase {
public:
  //-- Constructors
  CmdBase();
  //-- Destructor
  // Destructor: currently void
  virtual ~CmdBase();
  
  //-- Behavioural operations
  //   Reset stored data to default values
  virtual void defValues() = 0;
  //--
  //   Execute stored command
  virtual void execute() = 0;

# ifdef USEKUIP
  //--
  //   Read a command from external
  virtual void read() = 0;
  //--
  //   Read and execute
  virtual void readAndExe();
# endif
  //--
  //   Dump internal data
  virtual void dump() = 0;

protected:
# ifdef USEKUIP
  //--
  //  Read a float
  float readFloat() const;
  //--
  //  Read an integer
  int readInt() const;
# endif
  ///////////////////// Data Part /////////////////////////////////
  //--
  // No datum. The Class has the size of a function pointer.
};
*/

///////// CmdBase member function definitions //////////////////

CmdBase::CmdBase(){}
CmdBase::~CmdBase(){}

//--
//  Read a float
float CmdBase::readFloat() const{
  float x;
# ifdef USEKUIP
    kugetr_(&x);
# endif
  return x;
}

//--
//  Read an integer
int CmdBase::readInt() const{
  int i;
# ifdef USEKUIP
    kugeti_(&i);
# endif
  return i;
}

#ifdef USEKUIP
  //--
  //   Read a command from external
  void CmdBase::readAndExe(){
    read();
    execute();
  }
#endif




