//                              -*- Mode: C++ -*-
//: CmdCareCalib.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#include "care/CmdCareCalib.hh"
#include "care/ECMCcalib.hh"


/////////////// The function seen by kuip

extern "C" {
  void carecalib_(){
    CmdCareCalib calCmd;
    calCmd.readAndExe();
  }
}; // end of extern
//////////////////////////////////////////////////////////

/*
CLASS
  CmdCareCalib
  Class to deal with the care/calib kuip command.

class CmdCareCalib {
public:
  //-- Constructors
  CmdCareCalib();
  //-- Destructor
  // Destructor: currently void
  virtual ~CmdCareCalib();
  
  //-- Behavioural operations
  //   Reset stored data to default values
  virtual void defValues();
  //--
  //   Execute stored command
  virtual void execute();
  //--
  //   Read acommand from external
  virtual void read();
  //--
  //   Dump internal data
  virtual void dump();

private:
  ///////////////////// Data Part /////////////////////////////////
  //--
  Energy_t  inner,middle,outer;
};

*/

CmdCareCalib::CmdCareCalib() : inner(30.0), middle(6.7), outer(6.7) {}

CmdCareCalib::~CmdCareCalib(){}

void CmdCareCalib::defValues(){
  inner = 30.0;
  middle = 6.7;
  outer = 6.7;
}

//--
//   Execute stored command
void CmdCareCalib::execute(){
  ECMCcalib::setCalibration(inner,middle,outer);
}

# ifdef USEKUIP
//--
//   Read a command from external
void CmdCareCalib::read(){
  inner = readFloat();
  middle = readFloat();
  outer = readFloat();
}
# endif                          
//--
//   Printout Information for the current MinimumEnergy data stored
void CmdCareCalib::dump(){
  ECMCcalib::dumpCalib();
}






