//                              -*- Mode: C++ -*-
//: CmdCareMinims.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#include "care/CmdCareMinims.hh"
#include "care/ECHitCellArray.hh"
#include "auto/HITC.hh"
#include "auto/MVEX.hh"
#include "auto/MCME.hh"
#include "auto/MTMT.hh"

/////////////// The function seen by kuip

extern "C" {
  void careminims_(){
    CmdCareMinims minCmd;
    minCmd.readAndExe();
  }
}; // end of extern
//////////////////////////////////////////////////////////

/*
CLASS
  CmdCareMinims
  Base class to deal with the Minims kuip command

class CmdCareMinims {
public:
  //-- Constructors
  CmdCareMinims();
  //-- Destructor
  // Destructor: currently void
  virtual ~CmdCareMinims();
  
  //-- Behavioural operations
  //   Reset stored data to default values
  virtual void defValues();
  //--
  //   Execute stored command
  virtual void execute();
  //--
  //   Read acommand from external
  virtual void read();
  //--
  //   Dump internal data
  virtual void dump();

private:
  ///////////////////// Data Part /////////////////////////////////
  //--
  Energy_t  inner,middle,outer;
};

*/

CmdCareMinims::CmdCareMinims() : inner(0.15), middle(0.1), outer(0.05) {}

CmdCareMinims::~CmdCareMinims(){}

void CmdCareMinims::defValues(){
  inner = 0.15;
  middle = 0.1;
  outer = 0.05;
}

//--
//   Execute stored command
void CmdCareMinims::execute(){
  ECHitCellArray::setMinimumEnergy(inner,middle,outer);
}

# ifdef USEKUIP
//--
//   Read acommand from external
void CmdCareMinims::read(){
  inner = readFloat();
  middle = readFloat();
  outer = readFloat();
}
# endif
                          
//--
//   Printout Information for the current MinimumEnergy data stored
void CmdCareMinims::dump(){
   ECHitCellArray::dumpCutOff();
}

