//                              -*- Mode: C++ -*-
//:ECCellID.C
// 
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 9/4/97
// Last Update ........
//

#include "care/ECCellID.hh"



/*
CLASS
  ECCellID
  This class is used to store a Cell Identification Number.

OPERATOR
  The access to the Cell ID is given by (overloaded) function operators.

WARNING
  The three parameter constructor and function operator make use of 
  data defined by the LUTsManager. Be sure to use these members of the class
  only after the initialization of LUTs by the LUTsManager.


class ECCellID {
public:
  // Constructors
  //--
  // Default constructor
  inline ECCellID();
  //-
  // Constructor with three parameters: Module number, ix, iy.
  // Module number shold be in the range [1-number of modules] and
  // ix and iy in the range [1,ndim] where ndim is the block division.
  ECCellID(int ModNum, int ix, int iy);
  //--
  // Constructor with four parameters: Module number, ix, iy and cell dimension
  inline ECCellID(int ModNum, int ix, int iy, int ndim);
  //--
  //  function operator to get the cell id.
  inline int operator()() const;
  //--
  //  function operator to set a new cell ID and get it back
  inline int operator()(ECCellID cell);
  //-- 
  //  function operator to set a new cell ID and get it back.
  //  Input as the three parameter constructor
  int operator()(int ModNum, int ix, int iy);
  //-- 
  //  function operator to set a new cell ID and get it back.
  //  Input as the four parameter constructor
  inline int operator()(int ModNum, int ix, int iy, int ndim);
  //--
  //  function to get a Cell Index from current Cell ID
  CellIdx_t CellIdx() const;

private:
  //--
  // Identification number
  int  cell_id;
};
*/

//////////////////// Class ECCellID static data initialization  //////////

int ECCellID::firstPlane = 1;

//////////////////// Class ECCellID definition //////////////////////

//-
// Constructor with three parameters: Module number, ix, iy.
// Module number shold be in the range [1-number of modules] and
// ix and iy in the range [1,ndim] where ndim is the block division.

 
int ECCellID::operator()(ArtePointer<GCAL> modPointer, int ix, int iy) {
  int ndim = modPointer->ncl;
  return operator()(modPointer, ix, iy, ndim);
};

//-- 
//  static function to set the first module
void ECCellID::setFirstPlane(int firstPln){
  firstPlane = firstPln;
}
