//                              -*- Mode: C++ -*-
//:ECLastHit.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 6/5/97
// Last Update ........
//

#include "care/ECLastHit.hh"
#include "care/care.hh"

#include "care/caredata.hh"

#ifdef PERFORMANCES
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
Chrono * lastHitTime = 0;
#endif

#include "care/ECALDetector.hh"

#undef WRITE

#ifdef WRITE
#  include <iostream.h>
#endif

#ifndef CMATCH
// struct TrackInfo constructors
// default constructor
TrackInfo::TrackInfo() : track(0), zf(0.0), xf(0.0), yf(0.0),
  txf(0.0), tyf(0.0), pf(0.0), mctrack(0){}
// full constructor
TrackInfo::TrackInfo(ArtePointer<RTRA> tr, float z, float x, float y, 
		     float tx, float ty, float p, ArtePointer<MTRA> mc) :
   track(tr), zf(z), xf(x), yf(y),
   txf(tx), tyf(ty), pf(p), mctrack(mc){};
#endif
//

void eclasthit_(int * it, float * z, double * x, double * , int *imc){
  
# ifdef PERFORMANCES
  if(lastHitTime == 0){
    ChronoManager CM;
    lastHitTime = CM.createChrono("Last Hit ");
  }
  lastHitTime->start();
# endif

  if( ECalInfo.init ){
    
#ifndef CMATCH
    TrackInfo  curr;
    ArtePointer<RTRA>  track(*it);
    ArtePointer<MTRA>  mctrack(*imc);
    //  filling
    curr.track = track;
    curr.zf = *z;
    curr.xf = *x++;
    curr.yf = *x++;
    curr.txf = *x++;
    curr.tyf = *x++;
    curr.pf = *x++;
    curr.mctrack = mctrack;

    ECALDetector::storeTrack(curr);
#endif
# ifdef WRITE
    cout<<" Cached the last hit of track "<<*it<<" Z="<<*z
	<<"  X= "<<curr.xf<<"  Y= "<<curr.yf<<" MCid="<<mctrack<<endl;
#endif
    
# ifdef PERFORMANCES
    lastHitTime->stop();
# endif
  }
}

