//                              -*- Mode: C++ -*-
//:ECCellID.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 9/4/97
// Last Update ........
//
#ifndef ECCELLID_HH_
#define ECCELLID_HH_
#include "arte/Arte.hh"
#include "auto/GMAX.hh"
#include "auto/GSLX.hh"
#include "auto/GCAL.hh"
#include "auto/GCAX.hh"
#include "care/ECALtypes.hh"
#include "care/ELLUTsManager.hh"

/**
class ECCellID.
  This class is used to store a Cell Identification Number.

OPERATOR
  The access to the Cell ID is given by (overloaded) function operators.

NUMBERING
  Some functions of this class takes the parameters
  Module number, ix, iy. They are defined in this way:
  NUMBERING: Module
     Module number shold be in the range [1-number of modules] and is
     better represented as a ArtePointer<GCAL>.
  NUMBERING: ix and iy
     These integers should lie in the range [1,ndim] 
     where ndim is the block division (currently 1 for Outer, 2 for Middle 
     and or 5 for Inner calorimeter).


*/

class ECCellID {
public:
  // Constructors
  //--
  /// Default constructor
  inline ECCellID();
  //-
  /** Constructor with three parameters: Module number, ix, iy.
   Module number shold be in the range [1-number of modules] and
   ix and iy in the range [1,ndim] where ndim is the block division. */
  inline ECCellID(ArtePointer<GCAL> mod, int ix, int iy);
  //--
  /// Constructor with 4 parameters: Module number, ix, iy and cell dimension.
  inline ECCellID(ArtePointer<GCAL> mod, int ix, int iy, int ndim);
  //--
  ///  Constructor for GCAL numbering
  inline ECCellID(int modID);
  //--
  ///  function operator to get the cell id.
  inline int operator()() const;
  //--
  ///  function operator to set a new cell ID and get it back
  inline int operator()(ECCellID cell);
  //-- 
  /**  function operator to set a new cell ID and get it back.
     Input as the three parameter constructor */
  int operator()(ArtePointer<GCAL> modPointer, int ix, int iy);
  //-- 
  /**  function operator to set a new cell ID and get it back.
    Input as the four parameter constructor. */
  inline int operator()(ArtePointer<GCAL> modPointer, 
			int ix, int iy, int ndim);
  //--
  ///  Get module number.
  inline int module() const;
  //--
  ///  Get Module pointer.
  inline ArtePointer<GCAL> modulePointer() const;

  //--
  ///  Get index of the cell inside the module.
  inline void getModuleIndex(int & ix, int & iy, int & ndim) const;
  //--
  ///  function to get a Cell Index from current Cell ID
  inline CellIdx_t cellIdx() const; 

  //-- 
  ///  static function to set the first module
  static void setFirstPlane(int firstPln);

private:
  //--
  /// Identification number
  int  cell_id;
  //--
  /// First plane of the module
  static int firstPlane;
};

#include "care/ECCellID.icc"
#endif // ECCELLID_HH_
