//                              -*- Mode: C++ -*-
//:ELLUTsManager.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 9/4/97
// Last Update ........
//
#ifndef ELLUTSMANAGER_HH_
#define ELLUTSMANAGER_HH_
#include "arte/Arte.hh"
#include "auto/GCAL.hh"
#include "auto/GCAX.hh"
#include "auto/GMAX.hh"
#include "auto/GSLX.hh"
#include "care/ECALtypes.hh"
class ELCellGrid;
class ELNearNeiCellLUT;
class ECCellID;

/**
class  ELLUTsManager.
  Interface to operate with the LUTs of the calorimeter reconstruction.

KEYWORDS
  Look_Up_Table, Singleton

LUTS
  This class initialize and provide access to 4 Look Up Tables (LUTs):
  ELCellGrid, ELNearNeiCellLUT, ELFirstCellTable and ELCellsDescription.


PATTERN
  Singleton. The class MUST be initialized with a call to
  LUTsManager::instance() without any parameter and before use of LUTs.
  After the call all LUTs managed are correctly initialized. To access them
  create LUTsManager instances and use the const inline access functions
  cellsGrid(), neiboursTable(), cellsDescription() and firstCellTable(). 

OPTIMIZATION STATUS:
  High for accessing to information stored, low at initialization.

*/

class ELLUTsManager {
public:
 
  //--
  ///  Construtor
  inline ELLUTsManager();
  //--
  /// static member function to initialize the LUTsManager
  static void instance();
  
  //--
  ///  reference to CellDesc LUT
  inline GCCE &  cellsDesc(CellIdx_t cellnum) const ;
  //--
  ///  reference to CellDesc LUT
  inline const ELCellGrid  & cellsGrid() const ;
  //--
  ///  return references to initialized table FirstCellInMod
  inline CellIdx_t firstCell(ArtePointer<GCAL> gcal) const ;
  inline CellIdx_t firstCell(ArtePointer<GCAX> gcax) const ;

  //--
  ///  return references to initialized table NearNei
  inline ELNearNeiCellLUT & neiboursTable() const;

  //--
  ///  Rebuild the GCAX table 
  static void RebuildGCAX();

private:
  //--
  /// Flag to test if there
  static int initialized;

  /// The Look Up Tables
  static ELCellGrid  cells;
  static ELNearNeiCellLUT  nearNei;

};


#include "care/ELLUTsManager.icc"
#endif // ELLUTSMANAGER_HH_
