//                              -*- Mode: C++ -*-
//: EMClusterTree.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 1/10/97
// Last Update ........
//

#ifndef EMCLUSTERTREE_HH_
#define EMCLUSTERTREE_HH_

#ifdef _OLD_NAMES_
#  include <vector.h>
#else
#  include <vector>
#endif

#include "care/ECALtypes.hh"
#include "care/EMCluster.hh"
#include "care/EMTrackOnEcal.hh"
#include "care/EMCellFraction.hh"


/**
class EMClusterTree.
  Concrete derived class for the hierarchical cluster tree used in
  pattern recognition of ECAL. This class stores the full tree.

*/

class EMClusterTree : public EMCluster {
public:
  /**@name Constructors and Destructors */
  //@{
  // Constructor.
  ///  The default constructor builds an empty list. 
  EMClusterTree();
 
  /// Destructor:   A virtual destructor is needed.
  virtual ~EMClusterTree();
  //@}

  /**@name Beheavioral methods */
  //@{
  /// Reset all info in the cluster.
  void  reset();
  /// Start preparing to receive "size" cells
  void  start(unsigned int size);

  //--
  /// start a new cluster with a given cell. The passed cell is actually MOVED
  EMCluster * newCluster(EMClusterMember * clusmem);
  //--
  /// get a pointer to the next cell to clusterize
  EMHitCell*  newHitCell(CellIdx_t currIdx);
  //--
  /// Store parameters of a track
  void newTrack(TrackInfo & trackInfo);
  //--
  /// get the space for a new cell fraction 
  EMCellFraction* newCellFraction(EMHitCell* cell, Energy_t fraction);

  /**
    Analysis and traversing of the cluster tree is done by EAClusterAnalyzers
    that are object with definite analyzing properties. 
    EMClusterMembers MUST ignore these properties: in this way a loose coupling
    between EMClusterMembers and EAClusterAnalyzers is achieved and both
    can vary independently during program evolution.
    Examples of EAClusterAnalyzers are those that should return
    total energy of a cluster, the Center of Gravity, the Number of Cells. */
  virtual void analyzeMe(EAClusterAnalyzer & analyzer) const;

  /**
    Analysis and traversing of the cluster tree is done by EAClusterAnalyzers
   that are object with definite analyzing properties.
   EMClusterMembers MUST ignore these properties: in this way a loose coupling
   between EMClusterMembers and EAClusterAnalyzers is achieved and both
   can vary independently during program evolution.
   Examples of EAClusterAnalyzers are those that should return
   total energy of a cluster, the Center of Gravity, the Number of Cells. */
  virtual void modifyMe(EDClusterModifier & analyzer);
  
  //--
  ///  Dump for debugging.
  virtual void dump() const;

  //@}

  /**@name Status set/access functions */
  //@{
  //--
  /// return the Cell in which the point on ECAL is
  EMHitCell* whichHitCell(Coord_t x, Coord_t y) const;
  //--
  /// return the Cell pointer of a given CellIdx_t
  EMHitCell* whichHitCell(CellIdx_t cell) const{
    return whichHitCellArray[cell.index()];
  };

  vector<EMTrackOnEcal>::iterator beginTrackList(){
    return trackOnEcal.begin();
  };
   
  vector<EMTrackOnEcal>::iterator endTrackList(){
    return &trackOnEcal[nextTrackIdx];
  };
  //@}


private:
  //--
  /// arrays to store ClusterMembers statically
  vector<EMCluster> subCluster;
  vector<EMHitCell> clusCell;
  vector<EMTrackOnEcal> trackOnEcal;
  vector<EMCellFraction> cellFrac;

  ///  LookUp Table to get the cluster in which a cell is 
  vector<EMHitCell*> whichHitCellArray;

  /// indeces
  unsigned int nextClusIdx;
  unsigned int nextCellIdx;
  unsigned int nextTrackIdx;
  unsigned int nextFracIdx;
};


#endif // EMCLUSTERTREE_HH_




