//                              -*- Mode: C++ -*-
//: ESCorCOGPos.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 14/4/97
// Last Update ........
//

#ifndef ESCORCOGPOS_HH_
#define ESCORCOGPOS_HH_

#include "care/ESCOGPos.hh"

/**
class ESCorCOGPos.
  It finds better extimations of the starting point of an
  electromagnetic shower starting from a COG extimation.
*/

class ESCorCOGPos :  public ESCOGPos {
public:

  /// Constructor
  ESCorCOGPos();
  /// Destructor: currently void but needs to be virtual
  virtual ~ESCorCOGPos();
 
  /**@name Beheavioral methods */
  //@{ 
  ///   Evaluate the position for a cluster.
 
  virtual void evaluatePos(const EMCluster* clus);

  //--
  ///  Evaluate the correction on a given cog (by class Position)
  void correct(const ESPosition & cog);
  //--
  ///  Evaluate the correction on a given cog (by class COGPos)
  void correct(const ESCOGPos & cog);

  /**
    apply the correction to a point (x,y) evaluated by the COG algorithm
    and store the result in the body of the object. 
    In input the center and the number of divisions of the cell in which
    the COG is are required. */
  void applyTo(Coord_t xmean, Coord_t ymean,  // COG estimations
               Coord_t xc, Coord_t yc,        // center of the cell
               int ndv);
  /**
    apply the correction to a point (x,y) evaluated by the COG algorithm
    and store the result in the body of the object. 
    In input is required the pointer to the Hit cell in which
    the COG is. */
  void applyTo(Coord_t xmean, Coord_t ymean,  // COG estimations
               const EMHitCell* cell); 

  /**
    Apply the correction to a coordinate (x or y). 
    It does not modify object internal state. */
  Coord_t apply(Coord_t mean, Coord_t center, int ndv) const;
  //@}

  //--
  ///  Filling the static data for corrections (light)
  static void fill(SubCal subCal, float hardLam);
  //--
  ///  Filling the static data for corrections (full)
  static void fill(SubCal subCal, float hardLam, float hardFre,
                   float softLam, float softFre);
  //--
  ///  Set/reset flag
  static void setFlag(bool flag);
  //--
  ///  Print informations on static data
  static void dump();


  // accessor
  //-- 
  ///  return the new extimation of the x conversion point
  inline Coord_t x() const;
  //-- 
  ///  return the new extimation of the y conversion point
  inline Coord_t y() const;

private:
  //--
  /// Correct on internal data
  void correct(); 
  //--
  ///  new extimation of the x conversion point
  float xn;
  //--
  ///  new extimation of the y conversion point
  float yn;
  // 
  // static data
  //--
  /// flag to turn correction on/off
  static bool doit;
  //--
  /// data on shower lateral profile
  static float hardLamb[6];
  static float hardFreq[6];
  static float softLamb[6];
  static float softFreq[6];
  
};


#include "care/ESCorCOGPos.icc"
#endif // ESCORCOGPOS_HH_
