/*
*************************************************************************
**
** Description: "Cebra Memory Package Tables"
**
** Language:	C
**
** Called from: everywhere
**
** Created: 	29 Oct 97
** Updated:  
**
** Author:  	Sergey Moukhine
**          	HERA-B Collaboration
**          	JINR-Dubna
**          	Russia
**
*************************************************************************
*/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "cmtab.h"


#ifdef  __cplusplus
extern "C" {
#endif

/*---------------------------------------------------------------------*/

#define GET_VAL(IND) (*(int *)((char *)recordAdr+IND*tab->size))
#define RECORD_ADR							\
   if ((int)tab->record == 1)						\
      recordAdr = (void *)((char *)(&(tab->record))+sizeof(void *));	\
   else									\
      recordAdr = tab->record;

/*---------------------------------------------------------------------*/

/*
*************************************************************************
**		Tables handling
*************************************************************************
*/

/*---------------------------------------------------------------------*/

int CmTabInit(
              CmTableT	*tab,
              int     	recSize,	/*  record size		*/
              int     	init, 		/*  init n records	*/
              int     	ext, 		/*  extention		*/
              int     	order	 	/*  1 - ordered, 0 - not*/
              )
{
int maxRec;

maxRec = init;

tab->record   = NULL;
tab->nTotal   = 0;
tab->nExt     = ext;
tab->nUsed    = 0;
tab->size     = recSize;
tab->order    = order;

if (recSize <= 0 || ext < 0 || maxRec < 0)
   return (FAILURE);

if (maxRec == 0)
   return(OK);
tab->record   = malloc(recSize * maxRec);
if (tab->record)
   {
   tab->nTotal = maxRec;
   return(OK);
   }
else
   {
   tab->nTotal = 0;
   return(FAILURE);
   }
}

/*---------------------------------------------------------------------*/

int CmTabInitInPlace(
              CmTableT	*tab,
              int     	recSize,	/*  record size		*/
              int     	init, 		/*  max n records	*/
              int     	order	 	/*  1 - ordered, 0 - not*/
                    )
/*
**	Initialize table in which the records start just after
**	CmTableT structure.
*/
{
int maxRec;

maxRec = init;

tab->record   = NULL;
tab->nTotal   = 0;
tab->nExt     = 0;
tab->nUsed    = 0;
tab->size     = recSize;
tab->order    = order;

if (recSize <= 0 || maxRec < 0)
   return (FAILURE);

tab->nTotal = maxRec;
tab->record = (void *)(1);
return(OK);
}

/*---------------------------------------------------------------------*/

void CmTabPrint(CmTableT *tab)

{
void *recordAdr;
RECORD_ADR;
printf ("Table: size=%d used=%d total=%d ext=%d order=%d adr=%d\n",
	tab->size, tab->nUsed, tab->nTotal,
	tab->nExt, tab->order, (int)recordAdr);
}

/*---------------------------------------------------------------------*/

int CmTabFindIndex(CmTableT *tab, int key)
{
/* return index in the table where key is found
** or the place where it is needed to put a new key
*/

int  index;
int  ind1;
int  ind2;
int  v1;
int  v2;
int  v;
void *recordAdr;

if (tab->nTotal < 0)
   return(FAILURE);
if (tab->nUsed == 0)
   return(0);

RECORD_ADR;

/* if not ordered table
  ==========================
*/

if (tab->order != 1)
   {
   for (index=0; index<tab->nUsed; index++)
      {
      if (key == GET_VAL(index))
         return(index);
      }
   return(tab->nUsed);
   }

/* if ordered table
  ======================
*/

ind1 = 0;
ind2 = tab->nUsed-1;
v1 = GET_VAL(ind1);
v2 = GET_VAL(ind2);

while (ind2 - ind1 > 1)
   {

   /* already exists */

   if (v1 == key)
      return(ind1);
   if (v2 == key)
      return(ind2);

   /* add before ind1 */

   if (key < v1)
      return(ind1);

   /* add after ind2 */

   if (key > v2)
      return(ind2+1);

   /* divide interval */

   index = (ind1+ind2)/2;
   v = GET_VAL(index);
   if (key <= v)
      {
      ind2 = index;
      v2   = v;
      }
   else
      {
      ind1 = index;
      v1   = v;
      }
   }

/* interval is empty */

if (ind1 == ind2)
   {
   if (key <= v1)
      return(ind1);
   else
      return(ind1+1);
   }
else
   {
   if (key <= v1)
      return(ind1);
   else
   if (key <= v2)
      return(ind2);
   else
      return(ind2+1);
   }
}

/*---------------------------------------------------------------------*/

int CmTabAdd(CmTableT *tab, void *rec)
{
int  index;
int  key;
int  v;
int  pos;
void *recordAdr;

if (tab->nTotal < 0)
   return(FAILURE);

RECORD_ADR;

/* find record */

if (tab->order == 0)
   {
   index = tab->nUsed;
   }
else	/* ordered table */
   {
   if (tab->nUsed == 0)
      index = 0;
   else
      {
      key = *((int *)rec);
      index = CmTabFindIndex(tab,key);
      if (index == FAILURE)
         return(FAILURE);
      if (index < tab->nUsed)
         {
         v = GET_VAL(index);
         if (v == key)
            return(index);
         }
      }
   }

/* extend if not enought space */

if (tab->nUsed >= tab->nTotal)
   {
   if (tab->nExt <= 0)
      return(FAILURE);
   if ((int)tab->record == 1)
      return(FAILURE);

   tab->nTotal += tab->nExt;
   if (!tab->record)
      tab->record   = malloc(tab->nTotal * tab->size);
   else
      tab->record = realloc(tab->record, tab->nTotal * tab->size);
   if (!tab->record)
      {
      tab->nTotal = 0;
      tab->nUsed  = 0;
      return(FAILURE);
      }
   }

/* add to the table
  ==================
*/

RECORD_ADR;

pos = index * tab->size;
if (index != tab->nUsed)
   memmove((void *)((char *)recordAdr + pos + tab->size),
           (void *)((char *)recordAdr + pos),
           (tab->nUsed - index)*tab->size);
memmove((void *)((char *)recordAdr + pos), rec, tab->size);
tab->nUsed++;
return(index);
}

/*---------------------------------------------------------------------*/

void *CmTabFind(CmTableT *tab, int key)
/* Find data by key */
{
int  index;
int  v;
void *recordAdr;

if (tab->nTotal < 0)
   return(NULL);

RECORD_ADR;

index = CmTabFindIndex(tab,key);
if (index == FAILURE || index >= tab->nUsed)
   return(NULL);
v = GET_VAL(index);
if (v != key)
   return(NULL);

return((void *)((char *)recordAdr + index*tab->size));
}

/*---------------------------------------------------------------------*/

void *CmTabGet(CmTableT *tab, int index)
{
void *recordAdr;

if (!tab->record)
   return(NULL);
if (index < 0 || index >= tab->nUsed)
   return(NULL);

RECORD_ADR;
return((void *)((char *)recordAdr + index*tab->size));
}

/*---------------------------------------------------------------------*/

int CmTabN(CmTableT *tab)
{
return(tab->nUsed);
}

/*---------------------------------------------------------------------*/

void CmTabNull(CmTableT *tab)
{
void *recordAdr;

if (!tab->record)
   return;
RECORD_ADR;

memset(recordAdr,0,tab->nTotal*tab->size);
tab->nUsed = 0;
}

#ifdef  __cplusplus
      }
#endif
