#ifndef CMBOOK_H
#define CMBOOK_H

/*
*************************************************************************
**
** Description: header file for "Cebra Memory Package"
**
** Created: 	20 Sep 97
** Updated:	
**
** Author:  	Sergey Moukhine
**          	HERA-B Collaboration
**          	JINR-Dubna
**          	Russia
**
*************************************************************************
*************************************************************************
**
**	Functions (FORTRAN call sequence)
**	=================================
**
**	cminit(iq)
**				Initialization.
**				--------------- 
**
**		IQ    (in) 	Address which will be used by the user
**				for references.
**		     		For example IQ(ID+1) is the 1-st word in
**				the bank with identifier ID.
**
**	cmbook(id, nd)
**				Book a new bank.
**				---------------- 
**
**		ID    (out)	Bank identifier (return 0 if lack of memory).
**				Variable ID must reside in static memory.
**		ND    (in)	Number of integers to be booked
**
**	cmpush(id, nd)
**				Alter the size of a bank.
**				-------------------------
**
**		ID    (in,out)	Bank identifier (return 0 if lack of memory)
**				Variable ID must reside in static memory.
**		ND    (in)	N of additional data words, < 0 to decrease
**
**	cmfree(id)
**				Delete unused space in a bank
**				-----------------------------
**
**		ID    (in,out)	Bank identifier
**				Variable ID must reside in static memory.
**
*************************************************************************
*************************************************************************
*/

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct {
int	len;		/*  total length (in bytes)			*/
int	origin;		/*  address of origin (index)			*/
int	nd;		/*  number of data words			*/
int	status;		/*  status word	(is not used)			*/
} BankT;

/* ///////////////////////////////////////////////////////////////////
			Functions
/////////////////////////////////////////////////////////////////// */

void cmbook_(int *id, int *nd);
void cmpush_(int *id, int *nd);
void cmfree_(int *id);
void cminit_(int *iq);
void cmerror_(int *ier, char *mess);
void cmSetError(void (*errFun)(int *, char *));

void cmprint_(int *id);

#ifdef  __cplusplus
       }
#endif

#endif
