#ifndef CMTAB_H
#define CMTAB_H

/*
*************************************************************************
**
** Description: header file for "Cebra Memory Package Tables"
**
** Created: 	29 Oct 97
** Updated:  
**
** Author:  	Sergey Moukhine
**          	HERA-B Collaboration
**          	JINR-Dubna
**          	Russia
**
*************************************************************************
*/

#ifdef  __cplusplus
extern "C" {
#endif

#ifndef OK
#define		OK		0
#endif

#ifndef FAILURE
#define		FAILURE		-1
#endif

typedef struct {
int	nTotal;		/*  total n of records 				*/      
int	nExt;		/*  extention when no space (in records) 	*/
int	nUsed;		/*  n of used records        			*/
int	size;		/*  record size              			*/
int     order;		/*  1 - order; 0 - no order			*/
void	*record;	/*  array of records				*/
} CmTableT;


/* //////////////////////////////////////////////////////////////////
//			Functions
////////////////////////////////////////////////////////////////// */

extern int  CmTabInit(
              	CmTableT	*tab,
              	int     	recSize,	/*  record size		*/
              	int     	init, 		/*  init n records	*/
              	int     	ext, 		/*  extention (n rec)	*/
              	int     	order	 	/*  1 - ordered, 0 - not*/
		     );
	/*
	** Initialize table.
	** Return: OK or FAILURE
	*/

		     
int CmTabInitInPlace(
              CmTableT	*tab,
              int     	recSize,	/*  record size		*/
              int     	init, 		/*  max n records	*/
              int     	order	 	/*  1 - ordered, 0 - not*/
                    );
/*
**	Initialize table in which the records start just after
**	CmTableT structure.
** 	Return: OK or FAILURE
*/

extern int  CmTabAdd(CmTableT *tab, void* rec);
	/*
	** Add new record to the table.
	** If the table is "ordered" the first word
	** in the record must be a key.
	*/

extern void *CmTabGet(CmTableT *tab, int ind);
	/*
	** Find data by index (return record address or NULL)
	*/

extern void *CmTabFind(CmTableT *tab, int key);
	/*
	** Find data by key (return record address or NULL)
	*/

extern void CmTabPrint(CmTableT *tab);
	/*
	** Print table
	*/

extern int  CmTabN(CmTableT *tab);
	/*
	** Return number of records in the table
	*/

extern void CmTabNull(CmTableT *tab);
	/*
	** Null contents (all space available) of the table
	*/

extern int  CmTabFindIndex(CmTableT *tab, int key);
	/*
	** Return index in the table where key is found
	** or the place where it is needed to put a new key
	*/


#ifdef  __cplusplus
       }
#endif

#endif
