#include <stdio.h>
#include <strings.h>
#include <ctype.h>
#include "database/Lists.h"
#include "database/DbLists.h"
#include "arte/Arte.h"
#include "auto/MEMO.h"
#include "auto/SUBT.h"
#include "database/DbUtils.h"
#include "database/DbArteBridge.h"

int DbOpen(char *Csym, char *Name) {
  return (dbopen_( Csym, Name, strlen(Csym), strlen(Name) ) );
}

int dbopen_(char *Csym, char *Name, int len1, int len2) {

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Description: Open a MIZZI file/server and connect a symbolic name to it.
   *
   * Called from: anywhere.
   *
   * External references: OpenTables
   *
   * Argument in/out description
   *  Csym    in     symbolic name (GEOMIN, GEOMOUT, ...)
   *  Name    in     file/server name
   *  len1           lenght of Csym (needed for FORTRAN strings, forget it)
   *  len2           lenght of Name (needed for FORTRAN strings, forget it)
   *
   * Return value: 1 success, 0 error.
   *
   * Warnings: This function is thought for FORTRAN users.
   *           C users can use DbOpen. 
   *
   ************************************************************************/
 
  char mess[80]="DBOPEN: ", warn[80]="DBOPEN: ";
  List(DbFiles) F;
  int lun;

  ToUpper( Csym, len1 );

  strncpy( DbFiles->Csym, Csym, (size_t)len1 );
  DbFiles->Csym[len1] = '\0';
  strncpy( F.Name, Name, (size_t)len2 );
  F.Name[len2] = '\0';

  if( ListSearchF( DbFiles, "s", &DbFiles->Csym ) != 0 ) {

    /* 
     * Symbolic name already connected to a file.
     * Use dbclose before reconnecting to a new file.
     */

    strcat( mess, "symbolic name " );
    strcat( mess, DbFiles->Csym );
    strcat( mess, " already connected to a file/server" );
    meerrc_( "asd", mess, strlen(mess) );
    return 0;
  }

  /* For the moment we do so. See below. */

  iounit_( Csym, &lun, len1 );
  if( lun == 0 ) {
    strcat( mess, "symbolic name " );
    strcat( mess, DbFiles->Csym );
    strcat( mess, " unknown" );
    meerrc_( "fdg", mess, strlen(mess) );
    return 0;
  } 

  strcat( warn, "opening file/server " );
  strcat( warn, F.Name );
  strcat( warn, " on " );
  strcat( warn, DbFiles->Csym );
  mewarn_( warn, strlen(warn) );

  if( ( F.Id = OpenTables( F.Name , 0 ) ) == 0 ) {
    strcat( mess, "cannot open file/server " );
    strcat( mess, F.Name );
    meerrc_( "lkf", mess, strlen(mess) );
    return 0;
  }

  strcpy(F.Csym , DbFiles->Csym);
  if( ListAdd(DbFiles, &F) == 0 ) {
    strcat( mess, "cannot alloc memory" );
    meerrc_( "idg", mess, strlen(mess) );
    return 0;
  }
  return 1;
}

int DbClose(char *Csym) {
  return (dbclos_( Csym, strlen(Csym) ) );
}

int dbclos_(char *Csym, int len) {

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Description: Close a MIZZI file/server and disconnect the
   *              symbolic name from it.
   *
   * Called from: anywhere.
   *
   * External references: CloseTables
   *
   * Argument in/out description
   *  Csym    in     symbolic name (GEOMIN, GEOMOUT, ...)
   *  Name    in     file/server name
   *  len1           lenght of Csym (needed for FORTRAN strings, forget it)
   *  len2           lenght of Name (needed for FORTRAN strings, forget it)
   *
   * Return value: 1 success, 0 error.
   *
   * Warnings: This function is thought for FORTRAN users.
   *           C users can use DbClose. 
   *
   ************************************************************************/

  List(DbFiles) *F;
  char mess[80]="DBCLOS: ", warn[80]="DBCLOS: ";

  ToUpper( Csym, len );

  strncpy( DbFiles->Csym, Csym, (size_t)len );
  DbFiles->Csym[len] = '\0';

  if( (F = ListSearchF( DbFiles, "s", &DbFiles->Csym)) == 0 ) {
    strcat( mess, "symbolic name " );
    strcat( mess, DbFiles->Csym );
    strcat( mess, " not connected to a file/server" );
    meerrc_( "qwe", mess, strlen(mess) );
    return 0;
  }

  strcat( warn, "closing file/server " );
  strcat( warn, F->Name );
  strcat( warn, " on " );
  strcat( warn, F->Csym );
  mewarn_( warn, strlen(warn) );

  CloseTables( F->Id );
  
  ListRemove( DbFiles, F );

  return 1;
}

int DbWriteHeader(char *Csym, int *Mlink, int major, int minor) {
  return ( dbwrit_( Csym, Mlink, &major, &minor, strlen(Csym) ) );
}

int dbwrit_(char *Csym, int *Mlink, int *major, int *minor, int len) {

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Description: Write to a MIZZI file a header bank and its subs.
   *
   * Called from: anywhere.
   *
   * External references: dbwtab_
   *
   * Argument in/out description
   *  Csym    in     symbolic name (GEOMIN, GEOMOUT, ...)
   *  Mlink   in     header bank link
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  len            lenght of Csym (needed for FORTRAN strings, forget it)
   *
   * Return value: 1 success, 0 error.
   *
   * Warnings: This function is thought for FORTRAN users.
   *           C users can use DbWriteHeader. 
   *
   ************************************************************************/

  int n, m, nlink, *Link, lun, sub, retval=1;
  char mess[80]="DBWRIT: ", warn[80]="DBWRIT: ", Name[5];
  List(DbTables) *T;

  ToUpper (Csym, len);

  nlink = (int)Mlink / 4 - artelink_.larln1;

  /* Check if Mlink is a correct Zebra link. */

  if( nlink <= 0 || nlink >= artelink_.narlnk ) {
    strcat( mess, "link not inside link area" );
    meerrc_( "ytr", mess, strlen(mess) );
    return 0;
  }

  strncpy( Name, MEMO(nlink).NAME, (size_t)4 );
  Name[4] = '\0';

  /* Check if table is a header bank. */

  if( MEMO(nlink).SUP[0] != 0 ) {
    strcat( mess, "bank " );
    strcat( mess, Name );
    strcat( mess, " not a header one" );
    meerrc_( "poi", mess, strlen(mess) );
    return 0;
  }

  /* 
   * Memory address of the bank. Points directly to the columns number.
   */
  
  Link  =    gcbank_.iq + *Mlink;

  /* 
   * Check if the bank is marked for output. For the moment we utilize 
   * in a not proper manner the ARTE I/O handling.
   */

  iounit_( Csym, &lun , len);
  if( lun == 0 ) {
    strcat( mess, "symbolic name " );
    strncat( mess, Csym, (size_t)len );
    strcat( mess, " unknown" );
    meerrc_( "emb", mess, strlen(mess) );
    return 0;
  } 

  if( !(MEMO(nlink).IOD & 0x00000002) || 
      !(MEMO(nlink).IOD & ziocom_.iiobit[lun-1]) ) {
    strcat( mess, "header bank " );
    strcat( mess, Name );
    strcat( mess, " not marked for output on " );
    strncat( mess, Csym, (size_t)len );
    m = 30;
    mepric_( "juv", &m, mess, strlen(mess) );
    return 0;
  }

  if( !(MEMO(nlink).IOA & 0x00000002) || 
      !(MEMO(nlink).IOA & ziocom_.iiobit[lun-1]) ) return 0;
  
  /* Check if the bank is empty. */

  if( *Mlink == 0 || N_U(Link) == 0 ) {
    strcat( mess, "header bank " );
    strcat( mess, Name );
    strcat( mess, " is empty - nothing to save" );
    m = 30;
    mepric_( "vaf", &m, mess, strlen(mess) );
    return 0;
  }

  /* Write out header bank ... */

  strcat( warn, "writing table " );
  strcat( warn, Name );
  strcat( warn, " on " );
  strncat( warn, Csym, (size_t)len );
  mewarn_( warn, strlen(warn) );

  retval &= dbwtab_( Csym, Mlink, major, minor, len );

  /* ... and all subs. */

  for( n = MEMO(nlink).SUB1 ; n <= MEMO(nlink).SUB2 ; n++ ) {
    
    sub = SUBT(n).SUB;

    if( sub == nlink ) continue;


    if(!*(int *)((artelink_.larln1 + sub ) * 4) ||
    !N_U(gcbank_.iq+ *(int *)((artelink_.larln1 + sub ) * 4))) continue;

    if( !( MEMO( sub ).IOA & 0x00000002 ) ) continue; 
    if( !( MEMO( sub ).IOD & ziocom_.iiobit[lun-1] ) ) {
      strncpy( Name, MEMO( sub ).NAME, (size_t)4 );
      Name[4] = '\0';
      strcat( mess, "bank " );
      strcat( mess, Name );
      strcat( mess, " not marked for output on " );
      strncat( mess, Csym, (size_t)len );
      strcat( mess, " - table skipped" );
      m = 30;
      mepric_( "cuc", &m, mess, strlen(mess) );
      continue;
    }

    if( !( MEMO( sub ).IOA & ziocom_.iiobit[lun-1] ) ) continue;

    
    retval &= dbwtab_( Csym, (int *)((artelink_.larln1 + sub ) * 4),
		       major, minor, len );

  }
  return retval;

}

int DbWriteTable(char *Csym, int *Mlink, int major, int minor) {
  return ( dbwtab_( Csym, Mlink, &major, &minor, strlen(Csym) ) );
}

int dbwtab_(char *Csym, int *Mlink, int *major, int *minor, int len) {

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Description: Write to a MIZZI file a single bank.
   *
   * Called from: anywhere.
   *
   * External references: MIZZI functions.
   *
   * Argument in/out description
   *  Csym    in     symbolic name (GEOMIN, GEOMOUT, ...)
   *  Mlink   in     bank link
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  len            lenght of Csym (needed for FORTRAN strings, forget it)
   *
   * Return value: 1 success, 0 error.
   *
   * Warnings: This function is thought for FORTRAN users.
   *           C users can use DbWriteTable.
   *
   ************************************************************************/

  int n, m, j, nlink, *Link, Tab, FieldType, Nrows;
  char mess[80]="DBWTAB: ", Name[5], *warn, what;
  List(DbTables) *T;
  List(DbFiles) *F;
  Tables Id;

  ToUpper (Csym, len);

  nlink = (int)Mlink / 4 - artelink_.larln1;
  
  /* Check if Mlink is a correct Zebra link. */

  if( nlink <= 0 || nlink >= artelink_.narlnk ) {
    strcat( mess, "link not inside link area" );
    meerrc_( "asa", mess, strlen(mess) );
    return 0;
  }

  strncpy( Name, MEMO(nlink).NAME, (size_t)4 );
  Name[4] = '\0';

  /*
   * Look for Bank entry in DbTables list to get bank fields.
   */

  strcpy( DbTables->Name, Name );
  T    =    ListSearchF( DbTables, "s", &DbTables->Name );
  if( T == 0 ) {
    strcat( mess, "bank " );
    strcat( mess, Name );
    strcat( mess, " not declared in configuration kumac" );
    meerrc_( "das", mess, strlen(mess) );
    return 0;
  }

  /*
   * Memory address of the bank. Points directly to the columns number.
   */
  
  Link  =    gcbank_.iq + *Mlink;

  /* File id (given by OpenTables). */
  
  strncpy( DbFiles->Csym, Csym, (size_t)len );
  DbFiles->Csym[len] = '\0';
  F = ListSearchF( DbFiles, "s", &DbFiles->Csym );
  if( F == 0 ) {
    strcat( mess, "symbolic name " );
    strcat( mess, DbFiles->Csym );
    strcat( mess, " not connected to a file/server");
    meerrc_( "ctf", mess, strlen(mess) );
    return 0;
  }

  Id   =    F->Id;
      
  /* Book Table T->Name. */
    
  Tab  =    BookTable( Id, T->Name, T->Fmt, T->Comm );
    
  TableStamp( Id, *major , *minor );
    
  /* Check if Major and Minor revision numbers have already been used. */
    
  if( MatchTable( Id, Tab, 0 ) ) {
    strcat( mess, "table " );
    strcat( mess, Name );
    strcat( mess, " with major and minor numbers");
    sprintf( mess+strlen(mess), " %d  %d ", *major, *minor );
    strcat( mess, " already existing" );
    mewarn_( mess, strlen(mess) );
    warn = "Overwrite y/n ?";
    mewarn_( warn, strlen(warn) );
    do {
      scanf( "%c", &what );
      what = toupper ( what );
    } while ( what != 'Y' && what != 'N' );
    
    if( what == 'N' ) {
      UnbookTable( Id, Tab );
      return 0;
    }

    strcpy( mess, "DBWRIT: overwriting table " );
    strcat( mess, Name );
    mewarn_( mess, strlen(mess) );

  }

  /* Release unused rows. */

  mmfree_( Mlink );

  SkipTableField( Id, 1 );
  
  /* Put rows number into the first field. */
  
  Nrows = N_U(Link);
  PutTableField( Id, TABLEint, 1, &Nrows );
  
  /* Finally record the bank into the db file. */
  
  /* Cycle on the fields of the bank. */
  
  for( n = 0, j = 0 ; n < T->Entries ; n++ ) {

    switch( T->Type[n] ) {
      
    case 'I':
      FieldType = TABLEint;
      break;
      
    case 'F':
      FieldType = TABLEfloat;
      break;
      
    case 'B': case 'H':
      FieldType = TABLEuint;
      break;
      
    default:
      break;
    }

    
    /* Cycle on the dimension of each field. */
    
    for( m = 0 ; m < ( (T->Dim[n] == 0) ? 1 : T->Dim[n] ) ; m++ ) {
	
      PutTableStruct( Id, FieldType, MEMO(nlink).COL * 4, N_U(Link),
		      (void *)(Link + 2 + j + m));
    }
    j += ( (T->Dim[n] ==0 ) ? 1 : T->Dim[n] );
    
  }
  StoreTable( Id, Tab, 0 );
  UnbookTable( Id, Tab );
  return 1;
  
}

int DbReadHeader(char *Csym, int *Mlink, int major, int minor) {
  return ( dbread_( Csym, Mlink, &major, &minor, strlen(Csym) ) );
}

int dbread_(char *Csym, int *Mlink, int *major, int *minor, int len) {

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Description: Read from a MIZZI file a header bank and its subs.
   *
   * Called from: anywhere.
   *
   * External references: dbrtab_
   *
   * Argument in/out description
   *  Csym    in     symbolic name (GEOMIN, GEOMOUT, ...)
   *  Mlink   in     bank link
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  len            lenght of Csym (needed for FORTRAN strings, forget it)
   *
   * Return value: 1 success, 0 error.
   *
   * Warnings: This function is thought for FORTRAN users.
   *           C users can use DbReadHeader.
   *
   ************************************************************************/

  int n, m, nlink, Nrows, lun, sub, retval=1;
  char mess[80]="DBREAD: ",warn[80]="DBREAD: ", Name[5];
  List(DbTables) *T;
  Tables Id;

  ToUpper( Csym, len );

  nlink = (int)Mlink / 4 - artelink_.larln1;

  /* Check if Mlink is a correct Zebra link. */

  if( nlink <= 0 || nlink >= artelink_.narlnk ) {
    strcat( mess, "link not inside link area" );
    meerrc_( "&m^", mess, strlen(mess) );
    return 0;
  }

  strncpy( Name, MEMO(nlink).NAME, (size_t)4 );
  Name[4] = '\0';

  /* Check if the bank is a header one. */

  if( MEMO(nlink).SUP[0] != 0 ) {
    strcat( mess, "bank " );
    strcat( mess, Name );
    strcat( mess, " not a header one" );
    meerrc_( "123", mess, strlen(mess) );
    return 0;
  }

  /* 
   * Check if the bank is marked for input. For the moment we utilize 
   * in a not proper manner the ARTE I/O handling.
   */

  iounit_( Csym, &lun , len);
  if( lun == 0 ) {
    strcat( mess, "symbolic name " );
    strncat( mess, Csym, (size_t)len );
    strcat( mess, " unknown" );
    meerrc_( "656", mess, strlen(mess) );
    return 0;
  } 

  if( !(MEMO(nlink).IOA & 0x00000001) || 
      !(MEMO(nlink).IOA & ziocom_.iiobit[lun-1]) ) return 0;

  if( !(MEMO(nlink).IOA & ziocom_.iiobit[lun-1]) ) {
    strcat( mess, "header bank " );
    strcat( mess, Name );
    strcat( mess, " not marked for input on " );
    strncat( mess, Csym, (size_t)len );
    m = 30;
    mepric_( "W3w", &m, mess, strlen(mess) );
    return 0;
  }

  /* 
   * Drop existing tables.
   */

  if( *Mlink != 0 ) mmdrop_( Mlink );

  /* Start reading. */  

  strcat( warn, "reading table " );
  strcat( warn, Name );
  strcat( warn, " on " );
  strncat( warn, Csym, (size_t)len );
  mewarn_( warn, strlen(warn) );

  retval &= dbrtab_( Csym, Mlink, major, minor, len );

  for( n = MEMO(nlink).SUB1 ; n <= MEMO(nlink).SUB2 ; n++ ) {

    sub = SUBT(n).SUB;

    if( sub == nlink ) continue;

    if( !(MEMO( sub ).IOA & 0x00000001) || 
	!(MEMO( sub ).IOA & ziocom_.iiobit[lun-1]) ) continue;

    if( !(MEMO( sub ).IOD & 0x00000001) || 
	!(MEMO( sub ).IOD & ziocom_.iiobit[lun-1]) ) continue;


    retval &= dbrtab_( Csym, (int *)((artelink_.larln1 + sub) * 4),
		       major, minor, len );

  }

  /* Book parallel tables and relations. */

  m = 0;
  mmbook_( Mlink, &m );
  mmrupd_( &nlink );

  return retval;

}

int DbReadTable(char *Csym, int *Mlink, int major, int minor) {
  return ( dbrtab_( Csym, Mlink, &major, &minor, strlen(Csym) ) );
}

int dbrtab_(char *Csym, int *Mlink, int *major, int *minor, int len) {

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Description: Read from a MIZZI file a single bank.
   *
   * Called from: anywhere.
   *
   * External references: MIZZI functions.
   *
   * Argument in/out description
   *  Csym    in     symbolic name (GEOMIN, GEOMOUT, ...)
   *  Mlink   in     bank link
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  len            lenght of Csym (needed for FORTRAN strings, forget it)
   *
   * Return value: 1 success, 0 error.
   *
   * Warnings: This function is thought for FORTRAN users.
   *           C users can use DbReadTable.
   *
   ************************************************************************/

  int n, m, j, nlink, *Link, Tab, FieldType, Nrows;
  char mess[80]="DBRTAB: ", Name[5];
  List(DbTables) *T;
  List(DbFiles) *F;
  Tables Id;

  ToUpper( Csym, len );

  nlink = (int)Mlink / 4 - artelink_.larln1;

  /* Check if Mlink is a correct Zebra link. */

  if( nlink <= 0 || nlink >= artelink_.narlnk ) {
    strcat( mess, "link not inside link area" );
    meerro_( mess, strlen(mess) );
    return 0;
  }

  strncpy( Name, MEMO(nlink).NAME, (size_t)4 );
  Name[4] = '\0';

  strcpy( DbTables->Name, Name );
  T    =    ListSearchF( DbTables, "s", &DbTables->Name );
  if( T == 0 ) {
    strcat( mess, "bank " );
    strcat( mess, Name );
    strcat( mess, " not declared in configuration kumac - table skipped"
);
    mewarn_( mess, strlen(mess) );
    return 0;
  }

  /* File id (given by OpenTables). */

  strncpy( DbFiles->Csym, Csym, (size_t)len );
  DbFiles->Csym[len] = '\0';
  F = ListSearchF( DbFiles, "s", &DbFiles->Csym );
  if( F == 0 ) {
    strcat( mess, "symbolic name " );
    strcat( mess, DbFiles->Csym );
    strcat( mess, " not connected to a file/server");
    meerrc_( "ca@", mess, strlen(mess) );
    return 0;
  }

  Id   =    F->Id;

  /* Book Table T->Name for reading. */

  Tab  =    BookTable( Id, T->Name, 0, 0 );

  if( Tab == 0 ) {
    strcat( mess, "bank " );
    strcat( mess, Name );
    strcat( mess, " not booked in file/server " );
    strcat( mess, F->Name );
    strcat( mess, " - table skipped" );
    mewarn_( mess, strlen(mess) );
    return 0;
  }

  TableStamp( Id, *major, *minor );

  if( FetchTable( Id, Tab, TABLEeq ) == 0 ) {
    strcat( mess, "table " );
    strcat( mess, Name );
    strcat( mess, " with major and minor ");
    sprintf( mess+strlen(mess), "%d %d", *major, *minor );
    strcat( mess, " doesn't exist" );
    UnbookTable( Id, Tab );
    meerrc_( "tu*", mess, strlen(mess) );
    return 0;
  }

  SkipTableField( Id, 1 );

  /* Get rows number (first field). */

  GetTableField( Id, TABLEint, 1, &Nrows );

  /* Check if bank exists already. */

  if( *Mlink != 0 ) {
    strcat( mess, "existing bank " );
    strcat( mess, Name );
    strcat( mess, " will be overwritten" );
    mewarn_( mess, strlen(mess) );
  }
  mmdrop_( Mlink );

  /* Book space in memory. */

  mmbook_( Mlink, &Nrows );

  /*
   * Memory address of the bank. Points directly to the columns number.
   */
  
  Link  =    gcbank_.iq + *Mlink;

  /* Cycle on the fields of the bank. */

  for( n = 0, j = 0 ; n < T->Entries ; n++ ) {

    switch( T->Type[n] ) {
      
    case 'I':
      FieldType = TABLEint;
      break;
      
    case 'F':
      FieldType = TABLEfloat;
      break;
      
    case 'B': case 'H':
      FieldType = TABLEuint;
      break;
      
    default:
      break;
    }
    
    /* Cycle on the dimension of each field (scalar 0, vector n). */

    for( m = 0 ; m < ( ( T->Dim[n] == 0 ) ? 1 : T->Dim[n] ) ; m++ ) {
      
      GetTableStruct( Id, FieldType, MEMO(nlink).COL*4, Nrows,
		      (void *)(Link + 2 + j + m) );
    }
    j += ( ( T->Dim[n] == 0 ) ? 1 : T->Dim[n] );

  }
  UnbookTable( Id, Tab );

  /* Free unused rows. */

  mmfree_( Mlink );

  return 1;
}











