#include <stdio.h>
#include <ctype.h>
#include "database/Lists.h"
#include "database/DbLists.h"
#include "database/DbUtils.h"
#include "arte/Arte.h"
#include "auto/MEMO.h"

int DbListFile(char *Csym) {
  return (dblfil_(Csym, strlen(Csym) ) );
}

int dblfil_(char *Csym, int len) {

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Description: List the content of a MIZZI file.
   *
   * Called from: anywhere.
   *
   * External references: MIZZI functions.
   *
   * Argument in/out description
   *  Csym    in     symbolic name (GEOMIN, GEOMOUT, ...)
   *  len     in     lenght of Csym. Don't use it.
   *
   * Warnings: This function is thought for FORTRAN users.
   *           C users can use DbListFile.
   *
   ************************************************************************/

  char *tbname, mess[80]="DBLFIL: ";
  int xe, i, major, minor;
  int flag = 1;
  Tables x;
  List(DbFiles) *F;

  ToUpper( Csym, len );

  strncpy ( DbFiles->Csym, Csym, len );
  DbFiles->Csym[len] = '\0';

  F = ListSearchF( DbFiles, "s", &DbFiles->Csym );
  if( F == 0 ) {
    strcat( mess, "symbolic name " );
    strcat( mess, DbFiles->Csym );
    strcat( mess, " not connected to a file/server" );
    meerrc_( "y54", mess, strlen(mess) );
    return 0;
  }

  x = F->Id;

  printf( "\nListTables: Listing tables for file/server %s\n", F->Name );
  SearchTable(x,0,"","");
  while( tbname = SearchTable(x,1,0,0) ) {
    flag = 0;
    xe = BookTable( x, tbname, 0, 0 );

    printf( "\n===>Name    '%s'\n", tbname );
    printf(   "--->Title   '%s'\n", TableTitle(x,xe) );
    printf(   "--->Format  '%s'\n", TableDescription(x,xe) );

    TableStamp( x, 0, 0 );

    printf( "--->Entries\n" );

    for( TableStamp( x, 0, 0 ) ; MatchTable( x, xe, 1 ) ; 
	 TableStamp( x, major, minor ) ) {

      major = TableMajor(x);
      minor = TableMinor(x);

      printf( "    Major = %d          Minor = %d\n", major, minor );

    }
    UnbookTable( x, xe );
  }

  if( flag ) {
    strcat( mess, "file empty\n");
    mewarn_( mess, strlen(mess) );
  }
  return 1;
}

int DbListTable(char *Csym, int *Mlink, int major, int minor) {
  return (dbltab_(Csym, Mlink, &major, &minor, strlen(Csym) ) );
}

int dbltab_(char *Csym, int *Mlink, int *major, int *minor, int len) {

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Description: list the content of a ARTE bank into a MIZZI file.
   *
   * Called from: anywhere.
   *
   * External references: MIZZI functions.
   *
   * Argument in/out description
   *  Csym    in     symbolic name (GEOMIN, GEOMOUT, ...)
   *  Mlink   in     bank link
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  len     in     lenght of Csym. Don't use it.
   *
   * Warnings: This function is thought for FORTRAN users.
   *           C users can use DbListTable.
   *
   ************************************************************************/

  int n, m, j, r, nlink, *Link, *temp, Tab, FieldType, Nrows, c, lc;
  char mess[80]="DBLTAB: ", Name[5], field[32], *ptfmt; 
  char spaces[]="          ";
  List(DbTables) *T;
  List(DbFiles) *F;
  size_t size;
  Tables Id;

  ToUpper( Csym, len );

  nlink = (int)Mlink / 4 - artelink_.larln1;
  
  /* Check if Mlink is a correct Zebra link. */

  if( nlink <= 0 || nlink >= artelink_.narlnk ) {
    strcat( mess, "link not inside link area" );
    meerrc_( "5g5", mess, strlen(mess) );
    return 0;
  }

  strncpy( Name, MEMO(nlink).NAME, (size_t)4 );
  Name[4] = '\0';

  /*
   * Look for Bank entry in DbTables list.
   */

  strcpy( DbTables->Name, Name );
  T    =    ListSearchF( DbTables, "s", &DbTables->Name );
  if( T == 0 ) {
    strcat( mess, "bank " );
    strcat( mess, Name );
    strcat( mess, " not declared in configuration kumac" );
    meerrc_( "90t", mess, strlen(mess) );
    return 0;
  }
  /* File id (given by OpenTables). */
  
  strncpy( DbFiles->Csym, Csym, (size_t)len );
  DbFiles->Csym[len] = '\0';
  F = ListSearchF( DbFiles, "s", &DbFiles->Csym );
  if( F == 0 ) {
    strcat( mess, "symbolic name " );
    strcat( mess, DbFiles->Csym );
    strcat( mess, " not connected to a file/server");
    meerrc_( "m,.", mess, strlen(mess) );
    return 0;
  }

  Id   =    F->Id;

  Tab  =    BookTable( Id, T->Name, 0, 0 );
  if( !Tab ) {
    strcat( mess, "bank " );
    strcat( mess, Name );
    strcat( mess, " not booked in file/server " );
    strcat( mess, F->Name );
    meerrc_( "?><", mess, strlen(mess) );
    return 0;
  }
  
  Link =    gcbank_.iq + *((int *)((artelink_.larln1+nlink)*4));
  
  TableStamp( Id, *major, *minor );
  
  if( FetchTable( Id, Tab, TABLEeq ) == 0 ) {
    strcat( mess, "table " );
    strcat( mess, Name );
    strcat( mess, " with major and minor ");
    sprintf( mess+strlen(mess), "%d %d", *major, *minor );
    strcat( mess, " doesn't exist" );
    meerrc_( "::l", mess, strlen(mess) );
    UnbookTable( Id, Tab );
    return 0;
  }
  
  SkipTableField( Id, 1 );
  GetTableField( Id, TABLEint, 1, &Nrows );
  
  /* Alloc temporary memory. */

  temp = (int *) malloc ( (size_t) (MEMO(nlink).COL * 4 * Nrows) );
  if( temp == 0 ) {
    strcat( mess, "cannot allocate temporary memory" );
    meerrc_( "p[l", mess, strlen(mess) );
    UnbookTable( Id, Tab );
    return 0;
  }
  
  for( n = 0, j = 0 ; n < T->Entries ; n++ ) {
    for( m = 0 ; m < ( ( T->Dim[n] == 0 ) ? 1 : T->Dim[n] ) ; m++ ) {

      switch( T->Type[n] ) {
      case 'I':
	FieldType = TABLEint;
	break;
      case 'F':
	FieldType = TABLEfloat;
	break;
      case 'B': case 'H':
	FieldType = TABLEuint;
	break;
      default:
	  break;
      }
      
      GetTableStruct( Id, FieldType, MEMO(nlink).COL*4, Nrows,
		      (void *)(temp + j + m) );
    }
    j += ( ( T->Dim[n] == 0 ) ? 1 : T->Dim[n] );
  }
  UnbookTable( Id, Tab );
  
  printf("\nListing table '%s' in file %s\n\n", Name, F->Name );

  for( r = 0 , lc = 1 ; r < Nrows ; r++ ) {
    
    printf("===>Row number %d\n", r+1 );
    
    for( n = 0 , j = 0 , ptfmt = T->Fmt + 14; n < T->Entries ; n++ ) {
      for( m = 0 ; m < ( ( T->Dim[n] == 0 ) ? 1 : T->Dim[n] ) ; m++ ) {
	
	if( (lc++ % 20) == 0 ) {
	  printf("...More...\n");
	  fflush( stdin );
	  while( (c = fgetc( stdin )) == EOF );
	  if( c == 'q' ) {
	    n = T->Entries - 1;
	    r = Nrows - 1;
	    break;
	  }
	}
	
	sscanf( ptfmt , "%s" , field );
	ptfmt += strlen(field)+1;
	
	switch( T->Type[n] ) {
	case 'I':
	  strncat( field, spaces, 10 - strlen(field + 2) );
	  printf( "    %s   I      %d\n", field + 2, 
		  *(int *) (temp + r*MEMO(nlink).COL + j + m) ); 
	  break;
	case 'F':
	  strncat( field, spaces, 10 - strlen(field + 2) );
	  printf( "    %s   F      %g\n", field + 2,
		  *(float *) (temp + r*MEMO(nlink).COL + j + m) );
	  break;
	case 'B':
	  strncat( field, spaces, 10 - strlen(field + 3) );
	  printf( "    %s   B      %o\n", field + 3,
		  *(float *) (temp + r*MEMO(nlink).COL + j + m) );
	  break;
	case 'H':
	  strncat( field, spaces, 10 - strlen(field + 3) );
	  printf( "    %s   H      %c%c%c%c\n", field + 3,
		  *( (char *) (temp + r*MEMO(nlink).COL + j + m) ),
		  *( (char *) (temp + r*MEMO(nlink).COL + j + m) + 1 ),
		  *( (char *) (temp + r*MEMO(nlink).COL + j + m) + 2 ),
		  *( (char *) (temp + r*MEMO(nlink).COL + j + m) + 3 ) );
	  break;
	default:
	  break;
	}
      }
      j += ( ( T->Dim[n] == 0 ) ? 1 : T->Dim[n] );
    }
  }

  /* Release temporary memory. */

  free ( temp );
  
  return 1;
}

void ToUpper( char *Csym, int len ) {

  while ( len-- != 0 ) 
    *Csym++ = toupper(*Csym);

  return;

}





