//                              -*- Mode: C++ -*-
//: dgcal2.C
//
// Author: Benedetto Giacobbe
// ECAL group
// HERA-B experiment, Desy
// e-mail: giacobbe@bo.infn.it
//
// Created 21/8/97
// Last Update ........
//
// Digitalization routine for level 1
//    Level  Input  Output  Description
//      2    MCAL   DCAL    Fills the Dcal bank for all channels

#include "arte/ArteMessage.hh"
#include "care/ECALtypes.hh"
#include "care/ECMCcalib.hh"
#include "care/ECCellID.hh"
#include "care/dgcal.hh"
#include "care/ECDcalIO.hh"
#include "auto/HITC.hh"
#include "auto/MVEX.hh"
#include "auto/MCME.hh"
#include "auto/MTMT.hh"
#include "auto/ECA.hh"

#define NUMTOW 203
#define DIM_HEADER 5
#define DIM_TRAILER 2

float dgcal_enecalc(int ipl, int ix, int iy);

void dgcal2_(int *){

  int ix = 0, iy = 0, ndim = 0;
  ECMCcalib  mcCalib;          // Calibrator object
  
  ECDcalIO dcal;

  dcal.start();
  unsigned short sfed_id;
  for(ArteTableIterator<ECBO> ecbo; ecbo.valid(); ecbo.next()){
    // Header
    sfed_id =  (unsigned short) ecbo->fdid;
    dcal<< sfed_id << 0 << 0 << 0 << 0;
   
    if(ecbo->stat == 1){
      // int ind = ecbo->numb;
      for (int j=0; j<32; j++){
	int cellidx = ecbo->chan[j]; 
	if(cellidx != -1){
	  
	  CellIdx_t cell = cellidx;
	  ECCellID  cellid(cell->ceid);

	  cellid.getModuleIndex(ix, iy, ndim);
	  float ener = dgcal_enecalc(cellid.module(), ix, iy);
	  
	  ener *= mcCalib[ndim];  // MC Energy correction 

	  int adcCh = dgcalech(cellidx, ener);    // digitalized energy
	  dcal << (unsigned short) adcCh;  	  
	} else {
	  dcal << 0;
	}
      }
    } else {
      for(int j=0; j<32; j++){
	dcal << 0;
      }
    }
    // Trailer
    dcal << 0 << 0;
  }
}





