//                              -*- Mode: C++ -*-
//: dgcalinit.C
//
// Author: Benedetto Giacobbe & Mauro Villa
// ECAL group
// HERA-B experiment, Desy
// e-mail: giacobbe@bo.infn.it, villa@bo.infn.it
//
// Created 21/7/97
// Last Update ........
//
// Initialization of the Digitalization routine 


#include "arte/Arte.hh"
#include "auto/ECA.hh"
#include "auto/GCCE.hh"
#include "care/ECCellID.hh"
#include "care/ECHitCellArray.hh"
//#include "mizzi/htable.h"
#include "care/dgcal.hh"
#include "arte/ArteMessage.hh"
#define NUMBOARD 187
#define NUMCHANN 32

typedef unsigned short ushort;


extern "C"
{
  void dgcalinit_(){
    int ndim=0;
    ECCellID  cellid(0);
    float minEnergy[6];
    ECHitCellArray::getMinims(minEnergy[5],minEnergy[2],minEnergy[1]);
    /* Write and Read calibration constants */
    if( ArtePointer<ECBO>::getLink()()==0){
      // if the calibration tables are not present then
      // build them!
      ECBO board = {0,0,0,
		    {0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
		     0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0} };
      ECHC channelbase = { 1, // status of the channel
			   50.,50.,2., // info pedestal (ped1, ped2, wid)
			   {0., 1., 0., 0., 0., 0.}, // 6 calibration constants
			   {0., 1., 0., 0., 0., 0.}, // 6 calibration constants
			   1., // chi square
			   0.1, .10, // energy thresholds
			   10., 2., 1., // MIP data
			   -1, -1, -1}; // FED channel data
      ECHC channelInfo;
      //                 outer middle       inner
      float mip[6] = {0., 30.,   30., 0., 0., 15.};
      
      for(int i=0; i<NUMBOARD; i++)
	ArtePointer<ECBO>::append(board);
      
      ArteTableIterator<GCCE> gcce(1);  // looping index on cells
      
      for(ArteTableIterator<ECBO> ecbo; ecbo.valid(); ecbo.next()){
	ecbo->fdid = 0x0EC00+ ecbo.index();  // Front End Identifier
	ecbo->numb = ecbo.index()-1;             // number of board
	for(int j=0; j<NUMCHANN; j++, gcce.next()){
	  if( gcce.valid()){ // found a valid channel. Book an entry in ECHC
	    ecbo->chan[j] = gcce.index(); 
	    channelInfo = channelbase;
	    channelInfo.fdid = ecbo->fdid;
	    channelInfo.ecbo = ecbo.index(); 
	    channelInfo.chan = j;
	    cellid(gcce->ceid);
	    ndim = cellid.modulePointer()->ncl;
	    channelInfo.mip = mip[ndim];
	    float q_mips = channelInfo.pcal[0];
            float mipch_red = channelInfo.mip/4095.;
            float mipch_redpow = mipch_red;
            for(int i=1;i<6;i++){
    	      q_mips += channelInfo.pcal[i]*mipch_redpow;
    	      mipch_redpow *= mipch_red;
 	    }
 	    channelInfo.chmi = 0.3/q_mips;   

	    //
	    ArtePointer<ECHC>::append(channelInfo);  // ECHC booked
	  } else {
	    // channels no more valid. stop booking ECHC entries
	    ecbo->chan[j] = -1; 
	  }
	}
	if( ecbo->chan[0] == -1){
	  ecbo->stat = -1;  // board not used
	  //
	} else {
	  ecbo->stat = 1;  // board used 
	}
      }
      //   find the energy thresholds
      //
      float ener, wid;
      ArtePointer<ECHC> echc;
      for(ArteTableIterator<GCCE> gcce2; gcce2.valid(); gcce2.next()){
	cellid(gcce2->ceid);
	ndim = cellid.modulePointer()->ncl;
	ener = minEnergy[ndim];
	echc = gcce2.index();
	wid = echc->wid;
	echc->wid = 0.0;
	echc->thr1 = dgcalech(gcce2.index(), ener);  
	echc->thr2 = dgcalech(gcce2.index(), ener*2.0);
	echc->wid = wid;
      }
    }
  }
}
