#include "rich/PhotonDetection.h"
#include "rich/vessel.h"

// ==========================================================================
// 
//  interface
//
#include "auto/GEPT.hh"
#include "auto/HITB.hh"
#include "auto/DIGB.hh"
#include "auto/MIMP.hh"
#include "auto/MIMX.hh"
#include "auto/MIDI.hh"
#include "auto/HIDI.hh"
#include "auto/MIHI.hh"
#include "auto/MTRA.hh"

extern "C" {
extern F77NAME(ricini)();
extern F77NAME(mmbrel)(int*,int*,int*);

inline void mmbrel(int* rel, int row1, int row2) {
  F77NAME(mmbrel)(rel,&row1,&row2);
  }  

void F77NAME(dgrich)(int *gede) {
  if (!RICH_Detector) F77NAME(ricini)();
 
  int  daqid;
  DIGB digit;
  HITB hit;
  
  memset(&digit,0,sizeof(digit));
  memset(&hit  ,0,sizeof(hit));
  
  Photon p;

  int dig1=ArtePointer<DIGB>::nrow()+1;
  int hit1=ArtePointer<HITB>::nrow()+1;

  ArtePointer<GEPT> gept(*gede);
  int n0=gept->mim1;
  int n1=gept->miml+1;
  int i;

  for (i=n0;i<n1;i++) {
    ArtePointer<MIMP> mimp(i);
    if (mimp->gede!=*gede) {
      printf("dgrich: no photon %d %d\n",mimp->gede,*gede); getchar();
      continue;
      }
//
//  conversion MIMP to Photon:
//
    p.x()=Vec3(mimp->x*10,mimp->y*10,mimp->z*10);
    p.n()=Vec3(mimp->tx,mimp->ty,mimp->p);
    p.e()=norm(p.n());
    p.n()/=p.e();    
//    int ipol=int(mimp->time);     
//    p.polarisation()=
//        FromPolar(1,double(ipol/10000)/1000.,double(ipol%10000)/1000.); 
//        
//  raytracing & detection
//  
    if (!RICH_Vessel->raytrace(&p)) continue;
    if (!RICH_Detector->dodetect(&p,&daqid)) continue;
    if (RICH_Detector->ChannelExist(daqid)) { 
//
//  check if cell is hit twice
//
      ArtePointer<HITB> hitb(hit1);
      ArtePointer<DIGB> digb(dig1);
      ArteTableIterator<HITB> h(hit1);
      ArteTableIterator<DIGB> d(dig1);
      for (;d.valid();digb=d.next(),hitb=h.next()) {
        if (digb->id==daqid) break;
        }
//
//  append if id is not present in the table
//
      if (!d.valid()) { 
        digit.id  =daqid;
        digb=ArtePointer<DIGB>::append(digit);
//
// generate hit
//
        Vec3 pos=RICH_Detector->pos();
        hit.x=pos.x()/10.;
        hit.y=pos.y()/10.;
        hit.z=pos.z()/10.;
        hit.r=RICH_Detector->pm().type().colsize()/10.;
        hit.gede=mimp->gede;
        hitb=ArtePointer<HITB>::append(hit);
//
// relation: hit to digit: one to one relation (used: many digit to one hit)
//
        mmbrel(&ArtePointer<HITB_DIGB>::getLink().getRef(),
               hitb.index(),digb.index());
        }        
//
// relation: mimp to digit: many to many relation
//
      mmbrel(&ArtePointer<DIGB_MIMP>::getLink().getRef(),
             mimp.index(),digb.index());
                
//
// relation: mimp to hit:   many to many relation
//
      mmbrel(&ArtePointer<HITB_MIMP>::getLink().getRef(),
             mimp.index(),hitb.index());
      
      }
    }
  if (dig1!=ArtePointer<DIGB>::nrow()+1) {
    gept->dig1=dig1;
    gept->digl=ArtePointer<DIGB>::nrow();
    gept->hit1=hit1;
    gept->hitl=ArtePointer<HITB>::nrow();
    }
  }


} // end extern "C"
