/*
*************************************************************************
**
** Description: class EvdirSqlCompiler:
**	        Compiler errors)
**
** Language:	C++
**
** Created: 20 Aug 97
** Updated:  
**
** Author:  Sergey Moukhine
**          HERA-B Collaboration
**          JINR-Dubna
**          Russia
**
*************************************************************************
*/

#include <stdio.h>

#include <evdir/EvdTable.h>
#include <evdir/EvdCommon.h>
#include <evdir/EvdCompiler.h>


/****************************************************************
 * Error:		put error message, exit
 ****************************************************************/

void EvdirSqlCompiler::Error(
int ind		/*  case of error	*/
)

{
char *s = CompilerData.errorBuffer;

TranError();
sprintf((s+strlen(s)), GetErrorMess(ind));
EmergencyExit();
}

/****************************************************************
 * ErrorI:		put error message, exit
 ****************************************************************/

void EvdirSqlCompiler::ErrorI(
int ind,	/*  case of error	*/
int i		/*  data		*/
)
{
char *s = CompilerData.errorBuffer;

TranError();
sprintf((s+strlen(s)), GetErrorMess(ind), i);
EmergencyExit();
}

/****************************************************************
 * ErrorS:		put error message, exit
 ****************************************************************/

void EvdirSqlCompiler::ErrorS(
int  ind,	/*  case of error	*/
const char *str	/*  string		*/
)
{
char *s = CompilerData.errorBuffer;

TranError();
sprintf((s+strlen(s)), GetErrorMess(ind), str);
EmergencyExit();
}

/****************************************************************
 * ErrorIS:	put error message, exit
 ****************************************************************/

void EvdirSqlCompiler::ErrorIS(
int  ind,	/*  case of error	*/
int  i,		/*  data		*/
const char *str	/*  string 		*/
)
{
char *s = CompilerData.errorBuffer;

TranError();
sprintf((s+strlen(s)), GetErrorMess(ind), i, str);
EmergencyExit();
}

/****************************************************************
 * ErrorSS:	put error message, exit
 ****************************************************************/

void EvdirSqlCompiler::ErrorSS(
int  ind,	/*  case of error	*/
const char *s1,	/*  string 1		*/
const char *s2	/*  string 2		*/
)
{
char *s = CompilerData.errorBuffer;

TranError();
sprintf((s+strlen(s)), GetErrorMess(ind), s1, s2);
EmergencyExit();
}

/****************************************************************
 * GetErrorMess
 ****************************************************************/

char* EvdirSqlCompiler::GetErrorMess(
int ind		/*  case of error	*/
)
{
  static char text[500];

  switch (ind)
    {
      case 1:
        strcpy(text, "not enought memory");
        break;
	
      case 2:
        strcpy(text, "Compiler tables are broken?: call to %s");
        break;

      case 3:
        strcpy(text, "number is too long");
        break;

      case 4:
        strcpy(text, "wrong character: ASCII code \\0%o");
        break;

      case 5:
        strcpy(text, "unknown item or wrong construction");
        break;

      case 6:
        strcpy(text, "name is too long");
        break;

      case 7:
        strcpy(text, "internal error: %s");
        break;

      case 8:
        strcpy(text, "wrong DATE format, must be \'dd-MON-yyyy\'");
        break;

      case 9:
        strcpy(text, "wrong TIME");
        break;

      case 10:
        strcpy(text, "wrong format of number");
        break;

      case 11:
        strcpy(text, "%s");
        break;

      case 12:
        strcpy(text, "wrong field name: %s");
        break;

      case 13:
        strcpy(text, "types in Expression List must be the same");
        break;

      case 14:
        strcpy(text, "wrong type of operand: %s");
        break;

      case 15:
        strcpy(text, "wrong type of operand for operation %s : %s");
        break;
        
      case 16:
        strcpy(text, "bit number for value of type %s");
        break;
        
      case 17:
        strcpy(text, "types for BETWEEN x AND y must be the same");
        break;
        
      default:
        sprintf(text, "unknown EvdirSql compiler error: %d", ind);
    }
   return(text);
}

/****************************************************************
 * TranError:			*  Put Error prefix
 ****************************************************************/

void EvdirSqlCompiler::TranError()

{
char *s = CompilerData.errorBuffer;
int i;

if (lex.last < 0)
   {
   sprintf((s+strlen(s)), "EvdirSql Error: ");
   }
else
   {
   sprintf((s+strlen(s)), "EvdirSql error at the following line:\n");
   sprintf((s+strlen(s)), "  %s\n", lex.text);
   sprintf((s+strlen(s)), "--");
   for (i=0; i < lex.last -1; i++)
      sprintf((s+strlen(s)), "-");
   sprintf((s+strlen(s)), "^\n");
   }
}

void EvdirSqlCompiler::EmergencyExit()
{
longjmp(CompilerData.longJmpBuf,1);
}

