/*
*************************************************************************
**
** Description: header file for class EvdirSqlTable:
**	        Compiler Table manipulations
**
** Created: 20 Aug 97
** Updated:  
**
** Author:  Sergey Moukhine
**          HERA-B Collaboration
**          JINR-Dubna
**          Russia
**
*************************************************************************
*/
#ifndef		EVDIR_SQL_TABLE_H
#define		EVDIR_SQL_TABLE_H

class EvdirSqlTable {

private:

   int    nTotal;	/*  total n of records       */
   int    nExt;		/*  extention when no space  */
   int    nUsed;	/*  n of used records        */
   int    size;		/*  record size              */
   int    maxUsed;	/*  max (n_used): for STACK  */

public:

   void   *record;	/*  array of records         */

/*=======================	Table initialization		*/

int Init(int recSize, int maxRec, int nExt);

/*   Parameters:
 *		int     recSize;	record size
 *		int     maxRec;		total n of records
 *		int     nExt; 		extention
 *   Return:
 *		OK or FAILURE
 */

/*=======================	Add new record			*/

int Add(void *rec);

/*   Parameters:
 *		void    *rec;	record
 *   Return:
 *		index of new record, FAILURE if error
 */

/*=======================	Replace record			*/

int Replace(void *rec, int index);

/*   Parameters:
 *		void    *rec;	record
 *		int     index;	record index
 *   Return:
 *		OK or FAILURE
 */

/*=======================	Get address of record		*/

void *Get(int index);

/*   Parameters:
 *		int     index;	record index
 *   Return:
 *		address of record, NULL if error
 */

/*=======================	Get address of the last record.
				Remove the last record		*/

void *Pop(void);

/*  
 *   Return:
 *		address of record, NULL if error
 */

/*=======================	N of used records		*/

int Nused(void);

/*   Parameters:
 *   Return:
 *		n of used records
 */

/*=======================	make table empty 		*/

void Empty(void);

/*=======================	drop table 			*/

void Drop(void);

/*=======================	print table  			*/

void Print(void);


};

#endif
