%{
/*
*************************************************************************
**
** Description: class EvdirSqlCompiler:
**	        Compiler grammar
**
** Language:	C++, Bison
**
** Created: 20 Aug 97
** Updated:  
**
** Author:  Sergey Moukhine
**          HERA-B Collaboration
**          JINR-Dubna
**          Russia
**
*************************************************************************
*/
%}

%token	EQ NE LT LE GT GE NAME
%token	INT DOUBLE MONTH RUN
%token	COMPILE FILL_RUN_TABLE

/*	Keywords	*/

%token	AND OR NOT IN BETWEEN

%left	OR
%left	AND
%left	EQ NE
%left	LT LE GT GE
%left	'+' '-'
%left	'*' '/'
%right	NOT UMINUS UPLUS

%start	CompileOrFillRunTable

%{

#include <evdir/EvdTable.h>
#include <evdir/EvdCommon.h>
#include <evdir/EvdCompiler.h>
#include <evdir/EvdPrint.h>

#ifdef DEBUG
static EvdirSqlPrint prt;
#endif

static int nInList;

#define yyparse EvdirSqlCompiler::yyparse

#define YYPURE

%}

%%

CompileOrFillRunTable:
			COMPILE SelectClause
		|	FILL_RUN_TABLE FillRunTable
		;

/*=============		Select		==============*/

SelectClause:
			/* nothing */
		|	LogicalExpr
		;

FillRunTable:	
			/* nothing */
		|	RunSelection
			{
			RunOperation(OpPlus,$1,$1);
			memcpy(&rec->runTab,
			       GET_RECORD(TyRun,TbRun,$1),
			       sizeof(RunPairT));
			}

		|	RunSelectionTerm AND LogicalExpr
			{
			RunOperation(OpPlus,$1,$1);
			memcpy(&rec->runTab,
			       GET_RECORD(TyRun,TbRun,$1),
			       sizeof(RunPairT));
			}

		|	RunSelectionTerm OR LogicalExpr
			{
			TableDrop(TbRun);
			TableInit(TbRunTab);
			}

		|	LogicalExpr
			{
			TableDrop(TbRun);
			TableInit(TbRunTab);
			}
		;
		

/*=============		Primary		==============*/

Primary:
			Number
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->primary.kind = PrNumber;
			    rec->primary.link = $1;
			    $$ = ADD_RECORD(TbPrimary);
			    }
			 }
			 
		|	Date
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->primary.kind = PrDate;
			    rec->primary.link = $1;
			    $$ = ADD_RECORD(TbPrimary);
			    }
			 }
			 
		|	Time
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->primary.kind = PrTime;
			    rec->primary.link = $1;
			    $$ = ADD_RECORD(TbPrimary);
			    }
			 }
			 
		|	Field
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->primary.kind = PrField;
			    rec->primary.link = $1;
			    $$ = ADD_RECORD(TbPrimary);
			    }
			 }
			 
		|	RUN
		;	

Date:
			'\"' INT '-' MONTH '-' INT '\"'
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->date.day = GetInt($2);
			    rec->date.month = $4;
			    rec->date.year = GetInt($6);

			    if (rec->date.year < 1995 ||
			        rec->date.year > 2050)
			       Error(8);

			    if (rec->date.day < 1 ||
			        rec->date.day > 31)
			       Error(8);

			    switch (rec->date.month)
			       {
			       case 2:
			          if (rec->date.day > 29)
			             Error(8);
			          if (rec->date.year%4 != 0 && 
			              rec->date.day > 28)
			             Error(8);
			          break;
			       
			       case 4:
			       case 6:
			       case 9:
			       case 11:
			          if (rec->date.day > 30)
			             Error(8);
			          break;

			       default:;
			       }
			    $$ = ADD_RECORD(TbDate);
			    }
			 }
		;
		
Time:
			INT ':' INT
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->time.hours = GetInt($1);
			    rec->time.minutes = GetInt($3);
			    rec->time.seconds = 0.;
			 
			    if (rec->time.hours > 24)
			       Error(9);
			    if (rec->time.minutes > 59)
			       Error(9);
			    if (rec->time.hours == 24 && 
			        !(rec->time.minutes == 0 &&
			          rec->time.seconds == 0.))
			       Error(9);

			    $$ = ADD_RECORD(TbTime);
			    }
			 }

		|	INT ':' INT ':' Number
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->time.hours = GetInt($1);
			    rec->time.minutes = GetInt($3);
			    rec->time.seconds = GetDouble($5);
			 
			    if (rec->time.hours > 24)
			       Error(9);
			    if (rec->time.minutes > 59)
			       Error(9);
			    if (rec->time.hours == 24 && 
			        !(rec->time.minutes == 0 &&
			          rec->time.seconds == 0.))
			       Error(9);
			    if (rec->time.seconds >= 60)
			       Error(9);
			    
			    $$ = ADD_RECORD(TbTime);
			    }
			 }
		;

Field:
			NAME
			 {
			 if (lex.kind == LxCompile)
			    {
			    $$ = TableInd(TbField,$1);
			    if ($$ == FAILURE)
			       ErrorS(12,GET_NAME(TbName,$1));
			    }
			 }

		;

/*=============		Number		==============*/

Number:
			INT
		|	DOUBLE
		;

/*=============		Expr		==============*/

Expr:
			Primary
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpPrimary;
			    rec->expr.link = $1;
			    rec->expr.kind = TypeOfPrimary($1);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	'(' Expr ')'
			 {
			 $$ = $2;
			 }
			  
		|	Expr '+' Expr
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpPlus;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpPlus,$1,$3);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr '-' Expr
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpMinus;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpMinus,$1,$3);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr '*' Expr
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpMult;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpMult,$1,$3);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr '/' Expr
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpDiv;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpDiv,$1,$3);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	'-' Expr %prec UMINUS
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpUminus;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpUminus,$2,$2);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	'+' Expr %prec UPLUS
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpUplus;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpUplus,$2,$2);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		;

LogicalExpr:
			'(' LogicalExpr ')'
			 {
			 $$ = $2;
			 }

		|	NOT LogicalExpr
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpNot;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpNot,$2,$2);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr LT ExprOrBitNumber
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpLt;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpLt,$1,$3);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr LE ExprOrBitNumber
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpLe;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpLe,$1,$3);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr GT ExprOrBitNumber
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpGt;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpGt,$1,$3);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr GE ExprOrBitNumber
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpGe;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpGe,$1,$3);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr EQ ExprOrBitNumber
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpEq;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpEq,$1,$3);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr NE ExprOrBitNumber
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpNe;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpNe,$1,$3);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	LogicalExpr AND LogicalExpr
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpAnd;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpAnd,$1,$3);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	LogicalExpr OR LogicalExpr
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpOr;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpOr,$1,$3);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr BETWEEN Expr AND Expr
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpBetween;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOpBetw($1,$3,$5);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr BETWEEN BitNumberExpr AND BitNumberExpr
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpBetween;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOpBetw($1,$3,$5);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr NOT BETWEEN Expr AND Expr
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpNotBetween;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOpBetw($1,$4,$6);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr NOT BETWEEN BitNumberExpr AND BitNumberExpr
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpNotBetween;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOpBetw($1,$4,$6);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr IN '(' ExprList ')'
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpIn;
			    rec->expr.link = nInList;
			    rec->expr.kind = TypeOfOperation(OpIn,$1,$4);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr IN '(' BitNumberExprList ')'
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpIn;
			    rec->expr.link = nInList;
			    rec->expr.kind = TypeOfOperation(OpIn,$1,$4);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr NOT IN '(' ExprList ')'
			 {
			 if (lex.kind == LxCompile)
			    {
			     rec->expr.op = OpNotIn;
			     rec->expr.link = nInList;
			     rec->expr.kind = TypeOfOperation(OpNotIn,$1,$5);
			     $$ = rec->expr.kind;
			     ADD_RECORD(TbExpr);
			    }
			 }
			  
		|	Expr NOT IN '(' BitNumberExprList ')'
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpNotIn;
			    rec->expr.link = nInList;
			    rec->expr.kind = TypeOfOperation(OpNotIn,$1,$5);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		;

ExprOrBitNumber:
			Expr
		|	BitNumberExpr
		;

BitNumberExpr:
			'#' Expr
			 {
			 if (lex.kind == LxCompile)
			    {
			    rec->expr.op = OpBit;
			    rec->expr.link = DUMMY;
			    rec->expr.kind = TypeOfOperation(OpBit,$2,$2);
			    $$ = rec->expr.kind;
			    ADD_RECORD(TbExpr);
			    }
			 }
			  
		;

ExprList:
			ExprList ',' Expr
			 {
			 nInList++;
			 if ($1 != $3)
			    Error(13);
			 $$ = $3;
			 }
		|	Expr
			 {
			 nInList =1;
			 }
		;

BitNumberExprList:
			BitNumberExprList ',' BitNumberExpr
			 {
			 nInList++;
			 }
		|	BitNumberExpr
			 {
			 nInList =1;
			 }
		;

/*=====================  RunSelection  ========================*/

RunSelectionTerm:
			'(' RunSelection ')'
			 {
			  $$ = $2;
			 }

		|	RUN LT INT
			 {
			 rec->run.beg = FIRST_RUN_NUMBER;
			 rec->run.end = GetInt($3) - 1;
			 ADD_RECORD(TbRunTab);
			 $$ = ADD_RECORD(TbRun);
			 }
			  
		|	RUN LE INT
			 {
			 rec->run.beg = FIRST_RUN_NUMBER;
			 rec->run.end = GetInt($3);
			 ADD_RECORD(TbRunTab);
			 $$ = ADD_RECORD(TbRun);
			 }
			  
		|	RUN GT INT
			 {
			 rec->run.beg = GetInt($3) + 1;
			 rec->run.end = LAST_RUN_NUMBER;
			 ADD_RECORD(TbRunTab);
			 $$ = ADD_RECORD(TbRun);
			 }
			  
		|	RUN GE INT
			 {
			 rec->run.beg = GetInt($3);
			 rec->run.end = LAST_RUN_NUMBER;
			 ADD_RECORD(TbRunTab);
			 $$ = ADD_RECORD(TbRun);
			 }
			  
		|	RUN EQ INT
			 {
			 rec->run.beg = GetInt($3);
			 rec->run.end = GetInt($3);
			 ADD_RECORD(TbRunTab);
			 $$ = ADD_RECORD(TbRun);
			 }
			  
		|	RUN NE INT
			 {
			 //   LT
			 rec->run.beg = FIRST_RUN_NUMBER;
			 rec->run.end = GetInt($3) - 1;
			 ADD_RECORD(TbRunTab);			 
			 //   GT
			 rec->run.beg = GetInt($3) + 1;
			 rec->run.end = LAST_RUN_NUMBER;
			 ADD_RECORD(TbRunTab);
			 $$ = ADD_RECORD(TbRun);
			 }
			  
		|	NOT RunSelectionTerm
			 {
			 $$ = RunOperation(OpNot,$2,$2);
			 }
			  
		|	RunSelectionTerm AND RunSelectionTerm
			 {
			 $$ = RunOperation(OpAnd,$1,$3);
			 }
			  
		|	RUN BETWEEN INT AND INT
			 {
			 rec->run.beg = GetInt($3);
			 rec->run.end = GetInt($5);
			 ADD_RECORD(TbRunTab);
			 $$ = ADD_RECORD(TbRun);
			 }
			  
		|	RUN NOT BETWEEN INT AND INT
			 {
			 //   LT
			 rec->run.beg = FIRST_RUN_NUMBER;
			 rec->run.end = GetInt($4) - 1;
			 ADD_RECORD(TbRunTab);			 
			 //   GT
			 rec->run.beg = GetInt($6) + 1;
			 rec->run.end = LAST_RUN_NUMBER;
			 ADD_RECORD(TbRunTab);
			 $$ = ADD_RECORD(TbRun);
			 }
			  
		|	RUN IN '(' RunNumberList ')'
			 {
			 $$ = ADD_RECORD(TbRun);
			 }
			  
		|	RUN NOT IN '(' RunNumberList ')'
			 {
			 $$ = ADD_RECORD(TbRun);
			 $$ = RunOperation(OpNot,$$,$$);
			 }
			  
		;

RunSelection:
			RunSelectionTerm

		|	RunSelection OR RunSelectionTerm
			 {
			 $$ = RunOperation(OpOr,$1,$3);
			 }

		|	RunSelectionTerm OR RunSelection
			 {
			 $$ = RunOperation(OpOr,$1,$3);
			 }
		;	  

RunNumberList:
			RunNumberList ',' INT
			 {
			 rec->run.beg = GetInt($3);
			 rec->run.end = GetInt($3);
			 ADD_RECORD(TbRunTab);
			 }
		|	INT
			 {
			 rec->run.beg = GetInt($1);
			 rec->run.end = GetInt($1);
			 ADD_RECORD(TbRunTab);
			 }
		;

%%

