/*
*************************************************************************
**
** Description: class EvdirSql:
**	        implementation
**
** Language:	C++
**
** Created: 20 Aug 97
** Updated:  
**
** Author:  Sergey Moukhine
**          HERA-B Collaboration
**          JINR-Dubna
**          Russia
**
*************************************************************************
*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include <evdir/EvdCommon.h>
#include <evdir/EvdPrint.h>
#include <evdir/EvdTable.h>
#include <evdir/EvdCompiler.h>
#include <evdir/EvdSql.h>

#define MY_ALLOC(v,t,n) {			\
			if (n)			\
			   {			\
			   v = new t[n];	\
			   if (!v)		\
			      Error(1);		\
			   }			\
                        }

#define VERY_SMALL_NUMBER	1.E-20

#ifdef DEBUG
static EvdirSqlPrint prt;
#endif

//////////////////////////////////////////////////////////////////////////

EvdirSql::EvdirSql()		// Constructor
{
tcode      = NULL;

nOfNumbers = 0;
numberTab  = NULL;

nOfNames   = 0;
nameTab    = NULL;

nOfDates   = 0;
dateTab    = NULL;

nOfTimes   = 0;
timeTab    = NULL;

nOfFields  = 0;
fieldTab   = NULL;
fieldValue = NULL;

nOfPrimary = 0;
primTab    = NULL;

nOfExpr    = 0;
exprProg   = NULL;
compFlag   = FALSE;

nOfRun     = 0;
runTab     = NULL;
runInd     = DUMMY;
run        = 0;

nOfValues  = 0;
valueStk   = NULL;

}

//////////////////////////////////////////////////////////////////////////

EvdirSql::~EvdirSql()		// Destructor
{
this->Drop();
}

//////////////////////////////////////////////////////////////////////////

int EvdirSql::Compile(
              char*  sqlText,	// text to be compiled
              int    nFields,	// n of fields in "fields"
              NameAndValueT *fields, // names and types of fields
              char** error	// error message 
                     )
{

if (compFlag == TRUE)
   Drop();
   
//	Compile SQL-like expression

int res = compiler.SelectCompiler(sqlText, nFields, fields, error, &tcode);
if (res != OK)
   {
   compFlag = FALSE;
   if (tcode)
      delete(tcode);
   return(FAILURE);
   }
compFlag = TRUE;
   
if (setjmp(longJmpBuf) != 0)
   {
   *error = errMess;
   return(FAILURE);	// if error in GetTcode
   }

GetTcode();

return(OK);
}

//////////////////////////////////////////////////////////////////////////

void EvdirSql::Drop()
{
if (tcode)
   delete(tcode);
tcode = NULL;


if (numberTab)
   delete(numberTab);
nOfNumbers = 0;
numberTab  = NULL;

if (nameTab)
   {
   delete(nameTab);
   }
nOfNames   = 0;
nameTab    = NULL;

if (dateTab)
  delete(dateTab);
nOfDates   = 0;
dateTab    = NULL;

if (timeTab)
  delete(timeTab);
nOfTimes   = 0;
timeTab    = NULL;

if (fieldTab)
  delete(fieldTab);
if (fieldValue)
  delete(fieldValue);
nOfFields  = 0;
fieldTab   = NULL;
fieldValue = NULL;

if (primTab)
  delete(primTab);
nOfPrimary = 0;
primTab    = NULL;

if (exprProg)
  delete(exprProg);
nOfExpr    = 0;
exprProg   = NULL;
compFlag   = FALSE;

if (runTab)
  delete(runTab);
nOfRun     = 0;
runTab     = NULL;
runInd     = DUMMY;
run        = 0;

if (valueStk)
  delete(valueStk);
nOfValues  = 0;
valueStk   = NULL;

}

//////////////////////////////////////////////////////////////////////////

void EvdirSql::GetTcode()
{

//	Fill private arrays from T-code

/*
**	T-code will be helpfull if Compiler and Interpreter
**	will run as separate processes. In this case T-code
**	can be moved trough the pipe.
**
**	T-code structure:
**
**	Total_length_in_bytes
**	N_names,	(list of (strlen+1) + 0-term. strings: names);
**	N_numbers,	(list of NumberT);
**	N_dates,	(list of DateT);
**	N_times,	(list of TimeT);
**	N_fields,	(list of FieldT);
**	N_primaries,	(list of PrimaryT);
**	N_exprs,	(list of ExprT);
**	N_runs,		(list of RunPairT);
**	
*/

int n;
int i;
int len;
char* b = tcode;

memcpy((void *)&len, b, sizeof(int));
b += sizeof(int);

//	Names

memcpy((void *)&n, b, sizeof(int));
b += sizeof(int);
nOfNames = n;
MY_ALLOC(nameTab,NameT,n);

for (i=0; i<n; i++)
   {
   memcpy((void *)&len, b, sizeof(int));
   b += sizeof(int);
   nameTab[i].text = b;
   b += len;
   }

//	Numbers

memcpy((void *)&n, b, sizeof(int));
b += sizeof(int);
nOfNumbers = n;
MY_ALLOC(numberTab,ValueT,n);

for (i=0; i<n; i++)
   {
   NumberT nu;
   
   memcpy((void *)&nu, b, sizeof(NumberT));
   b += sizeof(NumberT);
   switch (nu.kind)
      {
      case NuInt:
         numberTab[i].kind = ValInt;
         numberTab[i].v.intV = nu.value.i;
         break;
         
      case NuDouble:
         numberTab[i].kind = ValDouble;
         numberTab[i].v.doubleV = nu.value.d;
         break;
      }
   }
   
//	Dates

memcpy((void *)&n, b, sizeof(int));
b += sizeof(int);
nOfDates = n;
MY_ALLOC(dateTab,ValueT,n);

for (i=0; i<n; i++)
   {
   DateT dt;
   
   memcpy((void *)&dt, b, sizeof(DateT));
   b += sizeof(DateT);
   dateTab[i].kind = ValDate;
   dateTab[i].v.dateV = dt.year*10000 + dt.month*100 + dt.day;

   }
   
//	Times

memcpy((void *)&n, b, sizeof(int));
b += sizeof(int);
nOfTimes = n;
MY_ALLOC(timeTab,ValueT,n);

for (i=0; i<n; i++)
   {
   TimeT ti;
   
   memcpy((void *)&ti, b, sizeof(TimeT));
   b += sizeof(TimeT);
   timeTab[i].kind = ValTime;
   timeTab[i].v.timeV = ti.hours*3600 + ti.minutes*60 + ti.seconds;

   }

//	Fields

memcpy((void *)&n, b, sizeof(int));
b += sizeof(int);
nOfFields = n;
MY_ALLOC(fieldTab,FieldT,n);
MY_ALLOC(fieldValue,ValueT,n);

for (i=0; i<n; i++)
   {
   memcpy((void *)&fieldTab[i], b, sizeof(FieldT));
   b += sizeof(FieldT);
   }

//	Primary

memcpy((void *)&n, b, sizeof(int));
b += sizeof(int);
nOfPrimary = n;
MY_ALLOC(primTab,PrimaryT,n);

for (i=0; i<n; i++)
   {
   memcpy((void *)&primTab[i], b, sizeof(PrimaryT));
   b += sizeof(PrimaryT);
   }

//	Expr

memcpy((void *)&n, b, sizeof(int));
b += sizeof(int);
nOfExpr = n;
nOfValues = n;
MY_ALLOC(exprProg,ExprT,n);
MY_ALLOC(valueStk,ValueT,n);

for (i=0; i<n; i++)
   {
   memcpy((void *)&exprProg[i], b, sizeof(ExprT));
   b += sizeof(ExprT);
   }

//	Run

memcpy((void *)&n, b, sizeof(int));
b += sizeof(int);
nOfRun = n;
MY_ALLOC(runTab,RunPairT,n);

for (i=0; i<n; i++)
   {
   memcpy((void *)&runTab[i], b, sizeof(RunPairT));
   b += sizeof(RunPairT);
   } 

runInd = DUMMY;
run    = 0;

}

//////////////////////////////////////////////////////////////////////////

int EvdirSql::Evaluate	(
               int    nFields,		// n of fields in "fields"
               NameAndValueT *fields,	// names, types & values of fields
               char** error		// error message 
               		)
{
/* 
**	Evaluate expression for "fields"
**	return TRUE, FALSE or FAILURE  
**	if FAILURE then "error" contains
**	address of error message 
*/

*error = errMess;

if (nOfExpr == 0)	// empty expression ==> TRUE
   return(TRUE);
   
if (setjmp(longJmpBuf) != 0)
   return(FAILURE);	// if error somewhere in the interpreter

if (compFlag != TRUE)
   Error(4);

//	Check parameters, fill field values
//---------------------------------------------

if (nFields != nOfFields)
   Error(2);

int i;
for (i=0; i<nFields; i++)
   {
   if (fields[i].kind != fieldTab[i].kind)
      Error(2);

   char* s1 = fields[i].name;
   char* s2 = nameTab[fieldTab[i].field].text;
   int len = strlen(s1);
   if (len != (int)strlen(s2))
      Error(2);
   for (int j=0; j<len; j++)
      if (toupper(s1[j]) != toupper(s2[j]))
         Error(2);

   fieldValue[i].kind = fields[i].kind;
   fieldValue[i].v    = fields[i].v;
   }

	//////////////////////////////////
	//	Evaluate expression	//
	//////////////////////////////////

int mask;
int n;
int val;
double d1;
double d2;
double d3;
ValueKind kind;

cur = -1;

for (int ind=0; ind<nOfExpr; ind++)
   {
   ExprT* expr = &exprProg[ind];

#ifdef DEBUG
   printf(" %s\t",prt.NameOfOperationKind(expr->op));
#endif

   switch (expr->op)
      {
      case OpUminus:
      /////////////
         if (cur < 0)
            Error(3);

         switch (valueStk[cur].kind)
            {
            case ValInt:
               valueStk[cur].v.intV = - valueStk[cur].v.intV;
               break;
               
            case ValDouble:
               valueStk[cur].v.doubleV = - valueStk[cur].v.doubleV;
               break;
               
            default:
               Error(3);
            }
         break;
         
      case OpUplus:
      /////////////
         break;
         
      case OpNot:
      /////////////
         if (cur < 0)
            Error(3);

         if (valueStk[cur].v.intV == TRUE)
            valueStk[cur].v.intV = FALSE;
         else
         if (valueStk[cur].v.intV == FALSE)
            valueStk[cur].v.intV = TRUE;
         else
            Error(3);
         break;
         
      case OpBit:
      /////////////
         if (cur < 0)
            Error(3);

         d1 = GetStkValue(ValDouble,cur);
         if (d1 < 1.)
            Error(5);
         if (d1 > 32.)
            Error(5);
         valueStk[cur].v.intV = (int)d1;
         valueStk[cur].kind = ValBit;
         break;
         
      case OpPlus:
      /////////////
         if (cur < 1)
            Error(3);

         d1 = GetStkValue(ValDouble,cur-1);
         d2 = GetStkValue(ValDouble,cur);
         d3 = d1 + d2;
         if (valueStk[cur-1].kind == ValInt &&
             valueStk[cur].kind == ValInt)
            valueStk[cur-1].v.intV = (int)d3;
         else
            {
            valueStk[cur-1].v.doubleV = d3;
            valueStk[cur-1].kind = ValDouble;
            }
         cur--;
         break;
         
      case OpMinus:
      /////////////
         if (cur < 1)
            Error(3);

         d1 = GetStkValue(ValDouble,cur-1);
         d2 = GetStkValue(ValDouble,cur);
         d3 = d1 - d2;
         if (valueStk[cur-1].kind == ValInt &&
             valueStk[cur].kind == ValInt)
            valueStk[cur-1].v.intV = (int)d3;
         else
            {
            valueStk[cur-1].v.doubleV = d3;
            valueStk[cur-1].kind = ValDouble;
            }
         cur--;
         break;
                  
      case OpMult:
      /////////////
         if (cur < 1)
            Error(3);

         d1 = GetStkValue(ValDouble,cur-1);
         d2 = GetStkValue(ValDouble,cur);
         d3 = d1 * d2;
         if (valueStk[cur-1].kind == ValInt &&
             valueStk[cur].kind == ValInt)
            valueStk[cur-1].v.intV = (int)d3;
         else
            {
            valueStk[cur-1].v.doubleV = d3;
            valueStk[cur-1].kind = ValDouble;
            }
         cur--;
         break;
                  
      case OpDiv:
      /////////////
         if (cur < 1)
            Error(3);

         d1 = GetStkValue(ValDouble,cur-1);
         d2 = GetStkValue(ValDouble,cur);
         if (fabs(d2) < VERY_SMALL_NUMBER)
            Error(6);
         d3 = d1 / d2;
         valueStk[cur-1].v.doubleV = d3;
         valueStk[cur-1].kind = ValDouble;
         cur--;
         break;
                  
      case OpLt:
      /////////////
         if (cur < 1)
            Error(3);

         if (valueStk[cur-1].kind == ValInt &&
             valueStk[cur].kind == ValBit)
            {
            mask = 0;
            for (i=1; i < valueStk[cur].v.intV; i++)
               mask |= 1<<(i-1);
            if (valueStk[cur-1].v.intV & mask)
               valueStk[cur-1].v.intV = TRUE;
            else
               valueStk[cur-1].v.intV = FALSE;
            }
         else
            {
            kind = valueStk[cur-1].kind;
            d1 = GetStkValue(kind,cur-1);
            d2 = GetStkValue(kind,cur);
            if (d1 < d2)
               valueStk[cur-1].v.intV = TRUE;
            else
               valueStk[cur-1].v.intV = FALSE;
            }

         valueStk[cur-1].kind = ValBool;
         cur--;
         break;
         
      case OpLe:
      /////////////
         if (cur < 1)
            Error(3);

         if (valueStk[cur-1].kind == ValInt &&
             valueStk[cur].kind == ValBit)
            {
            mask = 0;
            for (i=1; i <= valueStk[cur].v.intV; i++)
               mask |= 1<<(i-1);
            if (valueStk[cur-1].v.intV & mask)
               valueStk[cur-1].v.intV = TRUE;
            else
               valueStk[cur-1].v.intV = FALSE;
            }
         else
            {
            kind = valueStk[cur-1].kind;
            d1 = GetStkValue(kind,cur-1);
            d2 = GetStkValue(kind,cur);
            if (d1 <= d2)
               valueStk[cur-1].v.intV = TRUE;
            else
               valueStk[cur-1].v.intV = FALSE;
            }

         valueStk[cur-1].kind = ValBool;
         cur--;
         break;
         
      case OpGt:
      /////////////
         if (cur < 1)
            Error(3);

         if (valueStk[cur-1].kind == ValInt &&
             valueStk[cur].kind == ValBit)
            {
            mask = 0;
            for (i=1; i <= valueStk[cur].v.intV; i++)
               mask |= 1<<(i-1);
            mask = ~mask;
            if (valueStk[cur-1].v.intV & mask)
               valueStk[cur-1].v.intV = TRUE;
            else
               valueStk[cur-1].v.intV = FALSE;
            }
         else
            {
            kind = valueStk[cur-1].kind;
            d1 = GetStkValue(kind,cur-1);
            d2 = GetStkValue(kind,cur);
            if (d1 > d2)
               valueStk[cur-1].v.intV = TRUE;
            else
               valueStk[cur-1].v.intV = FALSE;
            }

         valueStk[cur-1].kind = ValBool;
         cur--;
         break;
         
      case OpGe:
      /////////////
         if (cur < 1)
            Error(3);

         if (valueStk[cur-1].kind == ValInt &&
             valueStk[cur].kind == ValBit)
            {
            mask = 0;
            for (i=1; i < valueStk[cur].v.intV; i++)
               mask |= 1<<(i-1);
            mask = ~mask;
            if (valueStk[cur-1].v.intV & mask)
               valueStk[cur-1].v.intV = TRUE;
            else
               valueStk[cur-1].v.intV = FALSE;
            }
         else
            {
            kind = valueStk[cur-1].kind;
            d1 = GetStkValue(kind,cur-1);
            d2 = GetStkValue(kind,cur);
            if (d1 >= d2)
               valueStk[cur-1].v.intV = TRUE;
            else
               valueStk[cur-1].v.intV = FALSE;
            }

         valueStk[cur-1].kind = ValBool;
         cur--;
         break;
         
      case OpEq:
      /////////////
         if (cur < 1)
            Error(3);

         if (valueStk[cur-1].kind == ValInt &&
             valueStk[cur].kind == ValBit)
            {
            mask = 1<<(valueStk[cur].v.intV-1);
            if (valueStk[cur-1].v.intV & mask)
               valueStk[cur-1].v.intV = TRUE;
            else
               valueStk[cur-1].v.intV = FALSE;
            }
         else
            {
            kind = valueStk[cur-1].kind;
            d1 = GetStkValue(kind,cur-1);
            d2 = GetStkValue(kind,cur);
            if (d1 == d2)
               valueStk[cur-1].v.intV = TRUE;
            else
               valueStk[cur-1].v.intV = FALSE;
            }

         valueStk[cur-1].kind = ValBool;
         cur--;
         break;
         
      case OpNe:
      /////////////
         if (cur < 1)
            Error(3);

         if (valueStk[cur-1].kind == ValInt &&
             valueStk[cur].kind == ValBit)
            {
            mask = 1<<(valueStk[cur].v.intV-1);
            if (!(valueStk[cur-1].v.intV & mask))
               valueStk[cur-1].v.intV = TRUE;
            else
               valueStk[cur-1].v.intV = FALSE;
            }
         else
            {
            kind = valueStk[cur-1].kind;
            d1 = GetStkValue(kind,cur-1);
            d2 = GetStkValue(kind,cur);
            if (d1 != d2)
               valueStk[cur-1].v.intV = TRUE;
            else
               valueStk[cur-1].v.intV = FALSE;
            }

         valueStk[cur-1].kind = ValBool;
         cur--;
         break;
         
      case OpAnd:
      /////////////
         if (cur < 1)
            Error(3);

         if (valueStk[cur].kind != ValBool)
            Error(3);
         if (valueStk[cur-1].kind != ValBool)
            Error(3);

         if (valueStk[cur-1].v.intV == TRUE &&
             valueStk[cur].v.intV == TRUE
            )
            valueStk[cur-1].v.intV = TRUE;
         else
            valueStk[cur-1].v.intV = FALSE;
         cur--;
         break;
         
      case OpOr:
      /////////////
         if (cur < 1)
            Error(3);

         if (valueStk[cur].kind != ValBool)
            Error(3);
         if (valueStk[cur-1].kind != ValBool)
            Error(3);

         if (valueStk[cur-1].v.intV == TRUE ||
             valueStk[cur].v.intV == TRUE
            )
            valueStk[cur-1].v.intV = TRUE;
         else
            valueStk[cur-1].v.intV = FALSE;
         cur--;
         break;
         
      case OpBetween:
      /////////////
         if (cur < 2)
            Error(3);

         if (valueStk[cur-2].kind == ValInt &&
             valueStk[cur-1].kind == ValBit &&
             valueStk[cur].kind == ValBit)
            {
            mask = 0;
            for (i=valueStk[cur-1].v.intV; i <= valueStk[cur].v.intV; i++)
               mask |= 1<<(i-1);
            if (valueStk[cur-2].v.intV & mask)
               valueStk[cur-2].v.intV = TRUE;
            else
               valueStk[cur-2].v.intV = FALSE;
            }
         else
            {
            kind = valueStk[cur-2].kind;
            d1 = GetStkValue(kind,cur-2);
            d2 = GetStkValue(kind,cur-1);
            d3 = GetStkValue(kind,cur);
            if (d1 >= d2 && d1 <= d3)
               valueStk[cur-2].v.intV = TRUE;
            else
               valueStk[cur-2].v.intV = FALSE;
            }

         valueStk[cur-2].kind = ValBool;
         cur -= 2;
         break;
         
      case OpNotBetween:
      //////////////////
         if (cur < 2)
            Error(3);

         if (valueStk[cur-2].kind == ValInt &&
             valueStk[cur-1].kind == ValBit &&
             valueStk[cur].kind == ValBit)
            {
            mask = 0;
            for (i=valueStk[cur-1].v.intV; i <= valueStk[cur].v.intV; i++)
               mask |= 1<<(i-1);
            if (!(valueStk[cur-2].v.intV & mask))
               valueStk[cur-2].v.intV = TRUE;
            else
               valueStk[cur-2].v.intV = FALSE;
            }
         else
            {
            kind = valueStk[cur-2].kind;
            d1 = GetStkValue(kind,cur-2);
            d2 = GetStkValue(kind,cur-1);
            d3 = GetStkValue(kind,cur);
            if (!(d1 >= d2 && d1 <= d3))
               valueStk[cur-2].v.intV = TRUE;
            else
               valueStk[cur-2].v.intV = FALSE;
            }

         valueStk[cur-2].kind = ValBool;
         cur -= 2;
         break;
         
      case OpIn:
      case OpNotIn:
      /////////////
         n = expr->link;	// n of elements in the list
         if (cur < n)
            Error(3);

         cur -= n;

         if (valueStk[cur].kind == ValInt &&
             valueStk[cur+1].kind == ValBit)	// if Bits
            {
            val = valueStk[cur].v.intV;
            valueStk[cur].v.intV = FALSE;
            for (i=1; i <= n; i++)
               {
               mask = 1<<(valueStk[cur+i].v.intV-1);
               if (val & mask)
                  valueStk[cur].v.intV = TRUE;
               }
            }
         else
            {
            kind = valueStk[cur].kind;
            d1 = GetStkValue(kind,cur);
            valueStk[cur].v.intV = FALSE;
            for (i=1; i <= n; i++)
               {
               d2 = GetStkValue(kind,cur+i);
               if (d1 == d2)
                  valueStk[cur].v.intV = TRUE;
               }
            }

         valueStk[cur].kind = ValBool;
         if (expr->op == OpNotIn)
            if (valueStk[cur].v.intV == TRUE)
               valueStk[cur].v.intV = FALSE;
            else
               valueStk[cur].v.intV = TRUE;
         break;
         
      case OpPrimary:
      /////////////
         {
         cur++;
         if (cur >= nOfValues)
            Error(3);
         PrimaryT* pr = &primTab[expr->link];
         
         switch (pr->kind)
            {
            case PrNumber:
               valueStk[cur] = numberTab[pr->link];
               break;
               
            case PrDate:
               valueStk[cur] = dateTab[pr->link];
               break;
               
            case PrTime:
               valueStk[cur] = timeTab[pr->link];
               break;
               
            case PrField:
               valueStk[cur] = fieldValue[pr->link];
               break;
            
            default:
               Error(3);
            }
         }
         break;
         
      default:
         Error(3);
      }
#ifdef DEBUG
   printf("STK[%2d] = %s\n", cur, prt.Value(&valueStk[cur]));
#endif
   }

if (cur != 0)
   Error(3);

return (valueStk[0].v.intV);
}

//////////////////////////////////////////////////////////////////////////

void EvdirSql::Error(int er)
{

strcpy(errMess,"EvdirSql interpreter error: ");
switch (er)
   {
   case 1:
      strcat(errMess, "not enought memory");
      break;
	
   case 2:
      strcat(errMess, "different field description for Compile and Evaluate");
      break;
	
   case 3:
      strcat(errMess, "(internal) wrong Expr table structure");
      break;
	
   case 4:
      strcat(errMess, "Evaluation without Compilation");
      break;
	
   case 5:
      strcat(errMess, "bit number must be between 1 and 32");
      break;
	
   case 6:
      strcat(errMess, "division by zero");
      break;
	
  default:
      sprintf(errMess+strlen(errMess),
              "unknown interpreter error: %d", er);
   }

longjmp(longJmpBuf,1);	// emergency exit
}

//////////////////////////////////////////////////////////////////////////

int EvdirSql::NextRun()
{
if (runInd >= nOfRun)
   return(DUMMY);

if (nOfRun == 0)
   return(++run);

if (runInd == DUMMY)
   {
   runInd = 0;
   run = runTab[runInd].beg;
   return(run);
   }
   
if (run < runTab[runInd].end)
   return(++run);

runInd++;
if (runInd >= nOfRun)
   return(DUMMY);
run = runTab[runInd].beg;
return(run);
}

//////////////////////////////////////////////////////////////////////////

double EvdirSql::GetStkValue(ValueKind kind, int ind)
{
		// get value of kind "kind"
switch (kind)
   {
   case ValInt:
   case ValDouble:
      if (valueStk[ind].kind == ValInt)
         return((double)valueStk[ind].v.intV);
      else
      if (valueStk[ind].kind == ValDouble)
         return(valueStk[ind].v.doubleV);
      else
         Error(3);
      return(0.); // never reached; used to avoid compiler warnings
         
   case ValDate:
      if (valueStk[ind].kind != kind)
         Error(3);
      return((double)valueStk[ind].v.intV);
      
   case ValTime:
      if (valueStk[ind].kind != kind)
         Error(3);
      return(valueStk[ind].v.doubleV);
      
   default:
      Error(3);
      return(0.); // never reached; used to avoid compiler warnings
   }
}

//////////////////////////////////////////////////////////////////////////

int EvdirSql::Print(char *text)
{
int i;

printf("-----  %s  -----\n",text);
for (i=0; i<nOfFields; i++)
   printf("FIELD %d:   %s\n",i, nameTab[fieldTab[i].field].text);
return(0);
}
