
       SUBROUTINE CGGESL(CSL,MUTT,IR)

************************************************************************
*
* CGGESL 
* written by Siegmund Nowak 01/97
*
* This routines creates GEANT superlayer banks from ARTE GESL bank
*
*       called from every geometry routine
*
* Arguments  kind         description
*-----------------------------------------------------------------------
*   CSL      in           name of the superlayer
*   MUTT     in           mother name into which it should be positioned
*                         for the moment are foreseen: MOTH,MAGN,RPOT,MUON
*                         RPOT is equivalent to MOTH except that the medium
*                         vaccuum is used instead of air
*   IR       in           overall rotation matrix number (gean)
************************************************************************

      IMPLICIT NONE
      INTEGER I,J,ISL,IR,HSL,IMED,SHP,IVOL,IUCOMP,MEDNUM,NPAR 
      REAL PAR(8),XMEAN,YMEAN,ZMEAN,X00,Y00,Z00
      CHARACTER*4 CSL,MUTT,MUTTC,CMUTT(5),SHAPE
#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/ZGMOTH.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"
      DATA CMUTT /'MOTH','MAGN','RPOT','MUON','TRSL'/ 
#include "arte/Arte.ff"
#include "auto/GEO.ff"

      CALL UCTOH(CSL,HSL,4,4)
      DO  ISL=1,NGESL()
       IF(HSL.EQ.HGESL_NAME(ISL)) GO TO 1 
      ENDDO
      CALL MEERRO('CGGESL- Superlayer not found: '//CSL)

 1    SHP = IGESL_SHP(ISL)
      IF(SHP.EQ.1) THEN
         PAR(1) = .5*(FGESL_X2(ISL)-FGESL_X1(ISL))
         PAR(2) = .5*(FGESL_Y2(ISL)-FGESL_Y1(ISL))
         PAR(3) = .5*(FGESL_Z2(ISL)-FGESL_Z1(ISL))
         XMEAN  = .5*(FGESL_X2(ISL)+FGESL_X1(ISL))
         YMEAN  = .5*(FGESL_Y2(ISL)+FGESL_Y1(ISL))
         ZMEAN  = .5*(FGESL_Z2(ISL)+FGESL_Z1(ISL))
         SHAPE = 'BOX '
         NPAR = 3
      ELSEIF(SHP.EQ.3) THEN
         PAR(1) = .5*(FGESL_X2(ISL)-FGESL_X1(ISL))
         PAR(2) =  PAR(1) + FGESL_DX(ISL)
         PAR(3) = .5*(FGESL_Y2(ISL)-FGESL_Y1(ISL))
         PAR(4) =  PAR(3) + FGESL_DY(ISL)
         PAR(5) = .5*(FGESL_Z2(ISL)-FGESL_Z1(ISL))
         XMEAN  = .5*(FGESL_X2(ISL)+FGESL_X1(ISL))
         YMEAN  = .5*(FGESL_Y2(ISL)+FGESL_Y1(ISL))
         ZMEAN  = .5*(FGESL_Z2(ISL)+FGESL_Z1(ISL))
         SHAPE = 'TRD2'
         NPAR = 5
      ELSE 
         CALL MEERRO('CGGESL:other shapes then BOX/TRD2 not foreseen') 
      ENDIF

C--   check for the mother ( MOTH, MAGN, RPOT, MUON, TRD )
      I = IUCOMP(MUTT,CMUTT,5)
      IF(I.LE.0) 
     + CALL MEERRO('CGGESL- a wrong mother has been installed: '//MUTT)

      MUTTC = MUTT
      IF(I.EQ.1.OR.I.EQ.3) THEN
         X00 = XMEAN
         Y00 = YMEAN
         Z00 = ZZERO + ZMEAN
         IF(I.EQ.1) THEN  
            IMED = MEDNUM(1,'AIR')
         ELSE
            IMED = MEDNUM(1,'VACUUM')
            MUTTC = 'MOTH'
         ENDIF
      ELSEIF(I.EQ.2) THEN
         X00 = -.5*(FGESL_X1(MAGNSL)+FGESL_X2(MAGNSL)) + XMEAN
         Y00 = -.5*(FGESL_Y1(MAGNSL)+FGESL_Y2(MAGNSL)) + YMEAN
         Z00 = -ZMAGM + ZMEAN
         IMED = MEDNUM(2,'AIR')
      ELSEIF(I.EQ.4) THEN
         X00 = -.5*(FGESL_X1(MUONSL)+FGESL_X2(MUONSL)) + XMEAN
         Y00 = -.5*(FGESL_Y1(MUONSL)+FGESL_Y2(MUONSL)) + YMEAN
         Z00 = -.5*(FGESL_Z2(MUONSL)+FGESL_Z1(MUONSL)) + ZMEAN
         IMED = MEDNUM(1,'AIR')
      ELSEIF(I.EQ.5) THEN
         X00 = -.5*(FGESL_X1(TRDSL)+FGESL_X2(TRDSL)) + XMEAN
         Y00 = -.5*(FGESL_Y1(TRDSL)+FGESL_Y2(TRDSL)) + YMEAN
         Z00 = -.5*(FGESL_Z1(TRDSL)+FGESL_Z2(TRDSL)) + ZMEAN
         IMED = MEDNUM(1,'AIR')
      ENDIF

      CALL GSVOLU(CSL,SHAPE,IMED,PAR,NPAR,IVOL)

C--   posit the superlayer
      CALL GSPOS(CSL,ISL,MUTTC,X00,Y00,Z00,IR,'MANY')
      CALL GSATT(CSL,'SEEN',0) 
      C78 = 'Superlayer '//CSL//' in '//MUTTC//' installed: Z1/Z2 = '
      CALL CH78BF(FGESL_Z1(ISL),-4)
      CALL CH78BF(FGESL_Z2(ISL),-4)
      CALL ME78WA
 99   RETURN
      END

