
      SUBROUTINE GEOMAIN

************************************************************************
*
* GEOMAIN
* written by Siegmund Nowak
*
* This routines is the main routine for generating the detector geometry
*
*       called from UGINIT
*
************************************************************************

      IMPLICIT NONE

#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/gclist.fi"
#include "gean/gcnum.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGDETV.fi"
#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "arte/ZIOCOM.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"
      INTEGER I,K,IVOL,MEDNUM,IGEAN,IGEOM
      REAL PAR(5),XMAGM,YMAGM
      LOGICAL RUN97,RUN98,RUN00
      CHARACTER IEND*1
#include "arte/Arte.ff"
#include "auto/GEO.ff"

      IEND = CHAR(0)

c***  select geometry version
      IF(GEOMVERS.EQ.0.) CALL MEERRO('GEOMAIN: no GEOMVERS given')
      RUN97 = .FALSE.
      RUN98 = .FALSE.
      RUN00 = .FALSE.
      IF(INT(GEOMVERS).EQ.96) THEN
         C78 = '**in this geant version run 96 does not exist anymore**'
      ELSEIF(INT(GEOMVERS).EQ.97) THEN
         C78 = '********   ATTENTION: RUN 97 is handled   ********'
         RUN97 = .TRUE.
      ELSEIF(INT(GEOMVERS).EQ.98) THEN
         C78 = '********   ATTENTION: RUN 98 is handled   ********'
         RUN98 = .TRUE.
      ELSE
         C78 = '********   ATTENTION: RUN 00 is handled   ********'
         RUN00 = .TRUE.
      ENDIF
      CALL ME78AP
      IGEAN = 1000*(GEANVERS+1.e-5)
      IGEOM = 1000*(GEOMVERS+1.e-5) 
      IGEOM = IGEOM - INT(GEOMVERS)*1000
      IF(IGEAN.NE.IGEOM) THEN
         C78 = 'FILES HBGEAN AND HBGEO.DAT ARE NOT EQUIVALENT'
         CALL ME78AP
         C78 = 'GEANVERS = '
         CALL CH78AI(IGEAN)
         CALL CH78AC(' GEOMVERS = ')
         CALL CH78AI(IGEOM)
         CALL ME78AP
      ENDIF

C***  Create HB-geometry header bank GEHD
      CALL MMDROP(JGEHD)
      CALL MMBOOK (JGEHD, 1)
      CALL DATIME (IQ(IQGEHD_DATE_L()), IQ(IQGEHD_TIME_L()))
      Q(QGEHD_GEOM_L()) = GEOMVERS
      Q(QGEHD_GEAN_L()) = GEANVERS

      CALL MMBOOK (JGESL, 0)
      CALL MMBOOK (JGEDE, 0)
      CALL MMBOOK (JGCAL, 0)
      CALL MMBOOK (JGWAL, 0)

c     read in superlayer file     
      IF(RUN00) THEN 
         CALL GESLDAT('gesl.dat'//IEND)
      ELSEIF(RUN97) THEN
         CALL GESLDAT('gesl97.dat'//IEND)
      ELSEIF(RUN98) THEN
         CALL GESLDAT('gesl98.dat'//IEND)
      ENDIF

C***  geometry code selection (short/long/user versions)
      C78 = 'THE FOLLOWING DETECTOR VARIANTS HAVE BEEN SELECTED: '
      CALL ME78WA
      C78 = ' '
      K = 0
      DO I=1,NDETV
      IF(DETV(I).NE.' ') THEN
         IF(K.LE.70) THEN
            CALL CH78BC(DETV(I))
         ELSE
            CALL ME78WA
            K = 0
            C78 = ' '
            CALL CH78BC(DETV(I))
         ENDIF
      K = K + 5
      ENDIF            
      ENDDO
      CALL ME78WA

C***  Define materials
      CALL MMBWRK(JTMP0,0,4,500)    ! foreseen for MATE-TMED relations
                                    ! do not touch it during initialisation
      CALL MEDMAIN
  
C***  Initialize rotation matrix
      IF(JROTM.LE.0) THEN
         CALL MZBOOK(IXCONS,JROTM,JROTM,1,'ROTM',NROTM,NROTM,0,3,0)
         DO I=1,NROTM
         LQ(JROTM-I) = 0
         ENDDO
      ENDIF

C***  mother and magnet volume
      DO I=1,NGESL()
       IF(IGESL_CMP(I) .EQ. LGESL_MOTH) THEN
         PAR(1) = .5*(FGESL_X2(I)-FGESL_X1(I))
         PAR(2) = PAR(1) + FGESL_DX(I)
         PAR(3) = .5*(FGESL_Y2(I)-FGESL_Y1(I))
         PAR(4) = PAR(3) + FGESL_DY(I)
         PAR(5) = .5*(FGESL_Z2(I)-FGESL_Z1(I))
         MOTHSL = I
         ZZERO = -0.5*(FGESL_Z2(I)+FGESL_Z1(I))
         CALL GSVOLU('MOTH' ,'TRD2' ,MEDNUM(1,'AIR') ,PAR ,5 ,IVOL)
         CALL GSATT('MOTH','SEEN',0) 
         CALL GSATT('MOTH','FILL',0) 
       ELSEIF(IGESL_CMP(I) .EQ. LGESL_MAG) THEN
         MAGNSL = I
         PAR(1) = .5*(FGESL_X2(I)-FGESL_X1(I))
         PAR(2) =  PAR(1) + FGESL_DX(I)
         PAR(3) = .5*(FGESL_Y2(I)-FGESL_Y1(I))
         PAR(4) =  PAR(3) + FGESL_DY(I)
         PAR(5) = .5*(FGESL_Z2(I)-FGESL_Z1(I))
         XMAGM  = .5*(FGESL_X2(I)+FGESL_X1(I))
         YMAGM  = .5*(FGESL_Y2(I)+FGESL_Y1(I))
         ZMAGM  = .5*(FGESL_Z2(I)+FGESL_Z1(I))
         CALL GSVOLU('MAGN','TRD2',MEDNUM(2,'AIR'),PAR,5,IVOL)
         CALL GSPOS('MAGN',I,'MOTH',XMAGM,YMAGM,ZZERO+ZMAGM,0,'MANY')
         CALL GSATT('MAGN','SEEN',0) 
       ELSEIF(IGESL_CMP(I) .EQ. LGESL_RIC ) THEN
         RICHSL = I
       ELSEIF(IGESL_CMP(I) .EQ. LGESL_TRSL) THEN
         TRDSL = I
       ELSEIF(IGESL_CMP(I) .EQ. LGESL_ECA ) THEN
         CALOSL = I
       ELSEIF(IGESL_CMP(I) .EQ. LGESL_MUON) THEN
         MUONSL = I
       ELSEIF(IGESL_CMP(I) .EQ. LGESL_FLD ) THEN
         FELDSL = I
         IF(DETV(LGESL_FLD) .EQ. 'FLDL') THEN
            Q(QGESL_Y2(I)) = -8.
            C78 = 'Magnetic field from field map is used'
         ELSE IF(DETV(LGESL_FLD) .EQ. 'FLDS') THEN 
            IF(FGESL_Y2(I).NE.0.) THEN
               C78 = 'A constant magnetic field is used: B = '
               CALL CH78BF(FGESL_Y2(I),2)
            ELSE
               C78 = 'The Magnet is switched OFF !!!!'
            ENDIF
         ENDIF
         CALL ME78WA              
       ENDIF
      ENDDO

      IF(RUN97) THEN
         CALL G97MAIN
      ELSEIF(RUN98) THEN
         CALL G98MAIN
      ELSEIF(RUN00) THEN
         CALL G00MAIN
      ENDIF

      CALL KILLGEO

C***  CLOSE GEOMETRY BANKS. (OBLIGATORY SYSTEM ROUTINE)
      CALL GGCLOS

C***  free unused space in detector banks
      CALL MMFREE(JGESL)
      CALL MMFREE(JGEDE)
      CALL MMFREE(JGWAL)
      CALL MMDROP(JTMP0)

 999  RETURN
      END




