
      SUBROUTINE GGCKOV
C.
C.    ******************************************************************
C.    *                                                                *
C.    *   This routine is called for each tracking step of a charged   *
C.    *   particle in a radiator. A Poisson-distributed number of      *
C.    *   photons is generated according to the Cherenkov formula,     *
C.    *   distributed evenly along the track segment and uniformly     *
C.    *   azimuth w.r.t. the particle direction. The parameters are    *
C.    *   then transformed into the Master Reference System, and they  *
C.    *   are added to the particle stack.                             *
C.    *                                                                *
C.    *   ==>Called by : GTMUON, GTHADR, GTELEC                        *
C.    *      Authors     R.Jones, F.Carminati ********                 *
C.    *                                                                *
C.    ******************************************************************
C.
      INTEGER ITMP5
      REAL RPHOT(3),XXPHOT(11)
      LOGICAL ROTATE
      PARAMETER (RFACT=369.81E9)

      integer richcut
      external richcut

#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/gcjloc.fi"
#include "gean/gctmed.fi"
#include "gean/gcunit.fi"
#include "gean/gctrak.fi"
#include "gean/gckine.fi"
#include "gean/gcking.fi"
#include "gean/ZGMOTH.fi"
#include "auto/TMP2.fi" 
#include "auto/TMP5.fi" 
#include "arte/Arte.ff"
#include "auto/TMP2.ff" 
#include "auto/TMP5.ff" 
*
*   ------------------------------------------------------------------
*
* *** See whether we generate at least one photon
*
      IF(ITRTYP.NE.4.AND.ITRTYP.NE.8) CALL GNCKOV
      CALL GPOISS(DNDL*STEP,NGPHOT,1)
      IF(NGPHOT.EQ.0) GO TO 999
*
* ***  Set up rotation to Particle frame
*
      CALL GFANG(VECT(4),COSTH,SINTH,COSPH,SINPH,ROTATE)
*
* ***  Distribute the photons in origin, direction, momentum
      COSMX  = THRIND/Q(JINDEX+NPCKOV)
      SINMX2 = (1.-COSMX)*(1.+COSMX)
      DO 40 J=1,NGPHOT
         CALL GRNDM(RPHOT, 1)
         IF(IGNEXT.NE.0) THEN
            DS=(STEP-PREC)*RPHOT(1)+PREC
         ELSE
            DS = STEP*RPHOT(1)
         ENDIF
         XXPHOT(1) = VECT(1)-VECT(4)*DS
         XXPHOT(2) = VECT(2)-VECT(5)*DS
         XXPHOT(3) = VECT(3)-VECT(6)*DS
         XXPHOT(11)= TOFG+(STEP-DS)*GETOT/(VECT(7)*CLIGHT)
*
* *** Sample the momentum of the photon
   20    CALL GRNDM(RPHOT, 3)
         PPHOT=PMIN+RPHOT(1)*DP
*
* *** Find in which bin we are
         KMIN = JMIN
         KMAX = NPCKOV
   30    KMED = (KMIN+KMAX)/2
         IF(Q(JTCKOV+1+KMED).LT.PPHOT) THEN
            KMIN = KMED
         ELSE
            KMAX = KMED
         ENDIF
         IF(KMAX-KMIN.GT.1) GOTO 30
         RATIO = (PPHOT-Q(JTCKOV+1+KMIN))/
     +           (Q(JTCKOV+KMIN+2)-Q(JTCKOV+1+KMIN))
         RATI1  = (1.-RATIO)
*
* *** Find the density function value corresponding to the
* *** momentum sampled
         RINDEX = Q(JINDEX+KMIN)*RATI1+Q(JINDEX+KMIN+1)*RATIO
         COST   = THRIND/RINDEX
         SINT2  = (1.-COST)*(1.+COST)
*
* *** Perform hit-and-miss
         IF(RPHOT(2)*SINMX2.GT.SINT2) GO TO 20
         SINT = SQRT(SINT2)
         PHI  = TWOPI*RPHOT(3)
         SINP = SIN(PHI)
         COSP = COS(PHI)
         XXPHOT(4) = SINT*COSP
         XXPHOT(5) = SINT*SINP
         XXPHOT(6) = COST
         XXPHOT(7) = PPHOT
         XXPHOT(8) = COST*COSP
         XXPHOT(9) = COST*SINP
         XXPHOT(10) = -SINT
*
         IF(ROTATE) THEN
            CALL GDROT(XXPHOT(8),COSTH,SINTH,COSPH,SINPH)
            CALL GDROT(XXPHOT(4),COSTH,SINTH,COSPH,SINPH)
         ENDIF
C
C     store as MIMP
C
C ********** Perform (wavelength dependent) efficiency cut **********

        irichcut= richcut(pphot)
        if (irichcut.eq.0) goto 40

        CALL MMBOOK(JTMP5,1)
        ITMP5 = JTMP5 + 2 + (NTMP5()-1)*NCTMP5()
        Q(ITMP5+1) = XXPHOT(3) - ZZERO
        Q(ITMP5+2) = XXPHOT(1)
        Q(ITMP5+3) = XXPHOT(2)
        Q(ITMP5+4) = XXPHOT(4)*XXPHOT(7)
        Q(ITMP5+5) = XXPHOT(5)*XXPHOT(7)
        Q(ITMP5+6) = XXPHOT(6)*XXPHOT(7)     
C        
C try to compress polarisation vector, so that mimp can
C be used to store photon
C This algorithm only works, if the integer to float conversion
C work one to one for any integer in the range -10000000 to +10000000.
C This true for ieee(sgi,hp), pc running linux.
C
        phi=acos(xxphot(8))
        if (xxphot(9).lt.0) phi=twopi-phi
        iphi=phi*1000
        icos=xxphot(10)*1000
        ipol=icos*10000+iphi        
        IQ(ITMP5+7) = ipol
        IQ(ITMP5+8) = ITRA
        ITMP2 = JTMP2 + 4 + (ITRA-1)*NCTMP2()
        IQ(ITMP2) = IQ(ITMP2) + 1

   40 CONTINUE


  999 END
