*-- Author :    Siegmund Nowak   04/03/93
*-- updated:    Olya Igonkina    19/02/97 
C*
      SUBROUTINE GPART
      IMPLICIT NONE
C-----------------------------------------------------------------------
C.
C--      SPECIAL VERSION FOR LUND INPUT (NO DECAY OF KOS ET AL)
C
C.    ******************************************************************
C.    *
C.    *  Define standard GEANT particles plus selected decay modes
C.    *  and branching ratios.
C.    *
C.    *  All data taken from : M. AGUILAR-BENITEZ et al,
C.    *                        Review of Particle Properties,
C.    *                        Rev. Mod. Phys. 56(1984).
C.    *
C.    *  In the case of W and Z, the lifetimes are calculated from
C.    *  quoted upper limits on widths.
C.    *
C.    *  NPAR           Number of parent particles defined for decay.
C.    *  IPAR           List of parent partilces allowed to decay.
C.    *                 Currently set up for pi0,pi+,pi-,K0long,K+,K-,
C.    *                 K0short,eta,lambda,sigma+,sigma0,sigma-,xi0,
C.    *                 xi-,omega-,antilambda,antisigma -,antisigma 0,
C.    *                 antisigma +,antixi 0,antixi +,antiomega +
C.    *                 decays.
C.    *  MODE(I,J)      I'th decay mode of J'th particle defined in
C.    *                 IPAR.
C.    *  BRATIO(I,J)    Branching ratio for I'th decay mode of J'th
C.    *                 particle in IPAR.
C.    *
C.    *    ==>Called by : <USER>, UGINIT
C.    *       Author    G.Patrick  *********
C.    *
C.    ******************************************************************
C.
#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/gcphys.fi"
#include "gean/gcnum.fi"
#include "auto/TOPP.fi"

      INTEGER LUTOGE
      EXTERNAL LUTOGE

      REAL BRATIO(6,24),BRTIO1(6,20),BRTIO2(6,4)
      INTEGER IPAR(24),MODE(6,24),MODE1(6,20),MODE2(6,4),I,NPAR, IMAX

      INTEGER IGE, IPR
      REAL CHARGE, MASS, TLIFE
      CHARACTER *20 CNAME
      CHARACTER *12 CNAME1
C
      EQUIVALENCE(BRTIO1(1,1),BRATIO(1,1)),(BRTIO2(1,1),BRATIO(1,21))
      EQUIVALENCE(MODE1(1,1),MODE(1,1)),(MODE2(1,1),MODE(1,21))
C
      DATA IPAR/ 7, 8, 9,10,11,12,16,17,18,19,20,21,22,23,24,26,27,
     *          28,29,30,31,32,5,6/
      DATA BRTIO1/ 98.802, 1.198, 0.,    0.,    0.,    0.,
     *            100.00,  0.,    0.,    0.,    0.,    0.,
     *            100.00,  0.,    0.,    0.,    0.,    0.,
     *             21.50, 19.35, 19.35, 13.55, 13.55, 12.39,
     *             63.50, 21.16,  5.59,  4.82,  3.20,  1.73,
     *             63.50, 21.16,  5.59,  4.82,  3.20,  1.73,
     *             68.61, 31.39,  0.,    0.,    0.,    0.,
     *             39.10, 31.80, 23.7,   4.91,  0.5,   0.,
     *             64.20, 35.80,  0.,    0.,    0.,    0.,
     *             51.64, 48.36,  0.,    0.,    0.,    0.,
     *            100.00,  0.,    0.,    0.,    0.,    0.,
     *            100.00,  0.,    0.,    0.,    0.,    0.,
     *            100.00,  0.,    0.,    0.,    0.,    0.,
     *            100.00,  0.,    0.,    0.,    0.,    0.,
     *             68.60, 23.40,  8.00,  0.,    0.,    0.,
     *             64.20, 35.80,  0.,    0.,    0.,    0.,
     *             51.64, 48.36,  0.,    0.,    0.,    0.,
     *            100.00,  0.,    0.,    0.,    0.,    0.,
     *            100.00,  0.,    0.,    0.,    0.,    0.,
     *            100.00,  0.,    0.,    0.,    0.,    0./
      DATA BRTIO2/100.00,  0.,    0.,    0.,    0.,    0.,
     *             68.60, 23.40,  8.00,  0.,    0.,    0.,
     *            100.00,  0.,    0.,    0.,    0.,    0.,
     *            100.00,  0.,    0.,    0.,    0.,    0./
      DATA MODE1/  101,30201,    0,    0,    0,    0,
     *             405,    0,    0,    0,    0,    0,
     *             406,    0,    0,    0,    0,    0,
     *             70707,40308,40209,40608,40509,70908,
     *             405,  708,90808,70402,70405,70708,
     *             406,  709,80909,70403,70406,70709,
     *             908,  707,    0,    0,    0,    0,
     *             101,70707,70908,10908,10302,    0,
     *             914,  713,    0,    0,    0,    0,
     *             714,  813,    0,    0,    0,    0,
     *             118,    0,    0,    0,    0,    0,
     *             913,    0,    0,    0,    0,    0,
     *             718,    0,    0,    0,    0,    0,
     *             918,    0,    0,    0,    0,    0,
     *            1218,  922,  723,    0,    0,    0,
     *             815,  725,    0,    0,    0,    0,
     *             715,  925,    0,    0,    0,    0,
     *             126,    0,    0,    0,    0,    0,
     *             825,    0,    0,    0,    0,    0,
     *             726,    0,    0,    0,    0,    0/
      DATA MODE2/  826,    0,    0,    0,    0,    0,
     *            1126,  830,  731,    0,    0,    0,
     *           40402,    0,    0,    0,    0,    0,
     *           40403,    0,    0,    0,    0,    0/
      DATA NPAR/24/
      DATA IMAX/9/

#include "arte/Arte.ff"
#include "auto/TOPP.ff"
C.
C.    ------------------------------------------------------------------

C.  the part of the standart Particle table 

      DO 20 I=1, NTOPP()
        IGE = ITOPP_GEAN(I)
        IF( IGE.EQ.0.or.IGE.GT.58 ) GO TO 20   ! preliminary: new TOPP 
 15     CALL CHHTOC( HTOPP_NAME(1,I),CNAME1) 
        CHARGE = ITOPP_CHRG(I) / 3.
        IF(IGE.EQ.1) THEN
          IPR = 1
        ELSEIF(IGE.EQ.2 .OR. IGE.EQ.3) THEN
          IPR = 2
        ELSEIF(IGE.EQ.5.OR.IGE.EQ.6) THEN
          IPR = 5
        ELSEIF(IGE.EQ.41.AND.CHARGE.LT..5) THEN
          GO TO 20            !      preliminary up to new TOPP table
        ELSEIF(IGE.EQ.49) THEN
          GO TO 20
        ELSEIF(IGE.GE.45.AND.IGE.LE.49) THEN
          IPR = 8 
        ELSEIF( ABS(CHARGE).LT.0.1E-5 )THEN
          IPR = 3
        ELSEIF( ABS(CHARGE).GT.0.1E-5 )THEN
          IPR = 4
        ELSE
          CALL MEERRO('no itrtyp for particle'//CNAME1)
        ENDIF
        MASS = FTOPP_MASS(I)
        TLIFE = FTOPP_LIFE(I)
        IF(TLIFE.lt.0.1E-20) THEN
           TLIFE = 0.1E-20
        ELSEIF(TLIFE.gt.1.E+15) THEN
           TLIFE = 1.E+15
        ENDIF
        CNAME=' '
        CNAME(1:12)=CNAME1
        CALL GSPART( IGE,CNAME,IPR, MASS,CHARGE, TLIFE,0,0)
 20     CONTINUE


c      CALL GSPART( 1,'GAMMA$       ',1,0.      , 0.,1.000000E+15,0,0)
c      CALL GSPART( 2,'POSITRON$    ',2,0.000511, 1.,1.000000E+15,0,0)
c      CALL GSPART( 3,'ELECTRON$    ',2,0.000511,-1.,1.000000E+15,0,0)
c      CALL GSPART( 4,'NEUTRINO$    ',3,0.      , 0.,1.000000E+15,0,0)
c      CALL GSPART( 5,'MUON +$      ',5,.1056584, 1.,2.197030E-06,0,0)
c      CALL GSPART( 6,'MUON -$      ',5,.1056584,-1.,2.197030E-06,0,0)
c      CALL GSPART( 7,'PION 0$      ',3,0.134973, 0.,0.840000E-16,0,0)
c      CALL GSPART( 8,'PION +$      ',4,0.139567, 1.,2.603000E-08,0,0)
c      CALL GSPART( 9,'PION -$      ',4,0.139567,-1.,2.603000E-08,0,0)
c      CALL GSPART(46,'TRITON$      ',4,2.81448 ,+1.,1.000000E+15,0,0)
      CALL GSPART(48,'GEANTINO$    ',6,0.      , 0.,1.000000E+15,0,0)
      CALL GSPART(50,'CERENKOV$    ',7,0.      , 0.,1.000000E+15,0,0)

C     Define decay modes.

      IF (IDCAY.LE.0)                             GO TO 99
      DO 10 I=1,NPAR
       CALL GSDK(IPAR(I),BRATIO(1,I),MODE(1,I))
 10   CONTINUE

  99  RETURN
      END





