
      SUBROUTINE GUSTEP

************************************************************************
*-- Author :    Siegmund Nowak
*
*  This is the CERN user routine called at the end of each tracking step
*  Here actions are collected which are specific for HERA-B but general
*  for all HERA-B users. The individual user routine is called USSTEP 
*  The actions called from GUSTEP are:
*  - drawing of tracks
*  - filling of temporary MIMP/MCAL banks for sensitive detectors
*  - storing of new generated particles, restoring of STAK particles 
*  - calling of USSTEP
*  - limiting of spiraling particles in the magnet
*  - seting length and flag parameters for ending tracks
************************************************************************

      IMPLICIT NONE

#include "gean/gclink.fi"
#include "gean/gckine.fi"
#include "gean/gcking.fi"
#include "gean/gcflag.fi"
#include "gean/gctrak.fi"
#include "gean/gcvolu.fi"
#include "gean/gccuts.fi"
#include "gean/gcsets.fi"
#include "gean/gctmed.fi"
#include "gean/gcnum.fi"
#include "gean/ZGMOTH.fi"
#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "arte/ZIOCOM.fi"
#include "auto/GEO.fi"
#include "auto/MC.fi"
#include "gean/ZGESL.fi"
#include "gean/ZGDETV.fi"
#include "auto/TMP2.fi"
      INTEGER TURBFL
      COMMON /TURB/ TURBFL
      INTEGER I,JSPEC,MAXSPEC,K,IUCOMP,IFL0,ITMP2,ISTPU,ICHECK
      PARAMETER (MAXSPEC=9)
      REAL ZVECT
      CHARACTER*4 IDNAM(MAXSPEC),CNAME
      CHARACTER*1 IEND
      DATA IDNAM
     +  /'CAY1','CAY2','CAY3','CASC','CAPB','CATU','SCRP','POLS','POLC'/
#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/MC.ff"
#include "auto/TMP2.ff"

c--   define some useful quantities
      IEND=CHAR(0) 
      CALL UHTOC(names(nlevel),4,cname,4)
      JSPEC = IUCOMP(NAMES(NLEVEL),IDNAM,MAXSPEC)
      CALL MUSTCHECK(cname//iend,ICHECK)
      IF(ICHECK.NE.0) JSPEC = MAXSPEC+1
  
      ZVECT = VECT(3) - ZZERO

c--   draw tracks if wanted
      IF(IBATCH.NE.0) GO TO 10
      IF(ISWIT(1) .EQ. 0) GO TO 10
      IF (ABS(CHARGE).LE.1.E-2) GO TO 10
      CALL GSXYZ
      CALL GDCXYZ

c--   stop the particle if it enters in volume "KILL"     
 10   IF(CNAME.EQ.'KILL') THEN
         ISTOP=1 
         GO TO 60
      ENDIF

c--   call special code for TRD-simulation
      IF(CNAME(1:2).EQ.'TR'.OR.IPART.EQ.48) THEN
         IF(DETV(LGESL_TRSL).EQ.'TRDL') CALL TRADE
      ENDIF

C--   MIMPS will be written out
 20   IF (MIMPSW.EQ.0) GO TO 30
      IF(IDTYPE.EQ.0) GO TO 30
      IF(IDTYPE.NE.LGESL_ECA.AND.IDTYPE.NE.LGESL_HOD1.AND.
     +   IDTYPE.NE.LGESL_HOD2.AND.IDTYPE.NE.LGESL_HOD3.AND. 
     +   IDTYPE.NE.LGESL_TRD) THEN
C--      only if there is a new volume or a new track
         IF(INWVOL.NE.1) GO TO 30
C--      select only charged tracks for track chambers
         IF (ABS(CHARGE).LE.1.E-2) GO TO 30
         CALL CRHIBK(1)
      ELSEIF(IDTYPE.EQ.LGESL_ECA) THEN
C--      all particles for the calorimeter
         CALL CRHIBK(2)
      ELSEIF(IDTYPE.EQ.LGESL_HOD1.OR.
     +       IDTYPE.EQ.LGESL_HOD2.OR.IDTYPE.EQ.LGESL_HOD3) THEN
C--      all particles for the sz-hodoscopes
         IF(DESTEP.LE.0) GO TO 30
         CALL CRHIBK(3)
      ELSEIF(IDTYPE.EQ.LGESL_TRD) THEN
c--      trd-mimps
         IF(DETV(LGESL_TRSL).EQ.'TRDL') THEN 
            IF(ABS(CHARGE).GT.1.E-2.OR.IPART.EQ.48) CALL CRHIBK(4)
         ELSEIF(DETV(LGESL_TRSL).EQ.'TRDS') THEN
            IF(INWVOL.NE.1) GO TO 30
C--         select only charged tracks for track chambers
            IF (ABS(CHARGE).LE.1.E-2) GO TO 30
            CALL CRHIBK(1)
         ENDIF
      ENDIF

c     make a calorimeter MTCP  at the entry of the calorimeter 
 30   IF(JSPEC.GT.0.AND.JSPEC.LE.6.) THEN
         IF(ISTAK.EQ.0.AND.IQ(JTMP2+(ITRA-1)*NCTMP2()+7).EQ.0) 
     +                                                CALL CRHIBK(5)
      ENDIF

C--   store newly generated particles in JKINE/JVERTX if they are outside
C--   the volumes IDNAM and in STAK only if they are inside
 40   IF(NGKINE.GT.0) THEN
         IFL0 = 1
         IF(JSPEC.NE.0) IFL0 = 0
         DO 45 I=1,NGKINE
         IFLGK(I) = IFL0
 45   CONTINUE
         CALL GSKING(0)
      ENDIF

c--   restore stak particles in JKINE/JVERTX banks again, if they come
c--   out of volumes IDNAM
      IF(ISTAK.NE.0.AND.INWVOL.EQ.1) THEN
         IF(CNAME(1:2).EQ.'TR'.OR.IPART.EQ.48) GO TO 50
         IF(JSPEC.EQ.0) THEN
            CALL GHSTAK
            ISTOP = 1
         ENDIF
      ENDIF

c--   switch on fast electron tracing by means of MUTUTB
 50   IF(INT(GEOMVERS).NE.97 .AND.TURBFL.EQ.1
     +               .AND.ZVECT.GE.FGESL_Z1(MUONSL)) CALL MUTURB

c--        call user subroutine
 60   ISTPU = 0
      IF(ISTOP.NE.0) ISTPU = 1
      CALL USSTEP
      IF(ISTOP.NE.0.AND.ISTPU.EQ.0) ISTPU = 2

c--        react if stopping particles

      IF(ISTOP.NE.0) GO TO 91

C---  stop tracks to avoid spirals in magnetic field
      IF(ABS(CHARGE).GE..1.AND.IQ(JTMP2+4+ITRA-1)*NCTMP2().EQ.0)THEN
         CALL NSPIRL
         IF(ISTOP.NE.0.AND.ISTAK.EQ.0.AND.MIMPSW.NE.0) THEN
            ITMP2 = JTMP2 + 2 + (ITRA-1)*NCTMP2()
            Q (ITMP2+3) = SLENG
            IQ(ITMP2+4) = 7
         ENDIF
      ENDIF
      GO TO 99

c--   stopping tracks give length and flag
  90  ISTOP =1
  91  IF(ISTAK.EQ.0.AND.MIMPSW.NE.0) THEN
         ITMP2 = JTMP2 + 2 + (ITRA-1)*NCTMP2()
         Q (ITMP2+3) = SLENG
         IF(INWVOL.EQ.3) THEN          !  particle leaves the volumen
            IQ(ITMP2+4) = 5
            GO TO 99
         ENDIF
         DO I=1,NMEC
         IF(LMEC(I).EQ.5) THEN         !  decay
            IQ(ITMP2+4) = 3
            GO TO 99
         ELSE IF((LMEC(I).GE.6 .AND. LMEC(I).LE.8) .OR.
     +            LMEC(I).EQ.11.OR.LMEC(I).EQ.12) THEN
            IQ(ITMP2+4) = 4            !  interaction
            GO TO 99
         ELSE IF(LMEC(I).EQ.30) THEN   !  energy cutoff
            IQ(ITMP2+4) = 6
            GO TO 99
         ENDIF
         ENDDO
         IF(ISTPU.EQ.2) THEN
            IQ(ITMP2+4) = 8
            GO TO 99
         ENDIF
         IF(CNAME.EQ.'KILL') THEN      !  particle is in volume 'KILL'
            IQ(ITMP2+4) = 5
            GO TO 99
         ENDIF

         IQ(ITMP2+4) = 0
      ENDIF


  99  RETURN
      END



