
      SUBROUTINE  ITLGEO(ISL)

************************************************************************
* written by Siegmund Nowak 11/93 for MSGC
* updated by Thorsten Oest 02/97 for fibre detector
*---------------------------------------------------------------------
*  detector geometry:
*                                                         thickness
*              --------------------------------
*                         carbonfibre                       DWEXMT
*              --------------------------------
*                         fiber layers
*              --------------------------------
*                         carbonfibre                       DWALMT
*              --------------------------------
*                                                        = DLMT(ISL)
*
*
*  fiber layer:
*
*              ---------------------------------
*                          CARBONFIBRE                   DGL1MT+DGL2MT
*              ---------------------------------
*                          POLYSTYREN                        DGASMT
*              ---------------------------------
*
*
* outer dimensions are set to the outer tracker dimensions
*---------------------------------------------------------------------
*
* Arguments  kind         description
*-----------------------------------------------------------------------
* ISL         in          MT superlayer number
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/gcnum.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGITR.fi"
#include "auto/GEO.fi"

      INTEGER IFIRST,ISL,ILAY,IVOL,IVOLS(2),IMFLAG,IROTM(6)
      INTEGER IMTR,IR,MEDNUM,NPL,NSL,ISEC,ISC,NSEC,IUCOMP
      REAL PAR(5),ZP,Z00,PARSV(3),XPSV,YPSV,ZPSV,XMS,ANG
      REAL PARRG(5), PARLG(5), XRG0, YRG0, XRG, YRG, RADIUS(5), DR 
      REAL XLG0, YLG0, XLG, YLG, SWOP
      REAL XMIN,XMAX,YMIN,YMAX,COSX,SINX,COS1,SIN1,A0,B0,XUPWIR,XLOWIR
      CHARACTER*4  NAM, MTWA(2),CLLN(2),RGUIDE(2), LGUIDE(2)
      SAVE IFIRST
      DATA IFIRST/0/
      DATA MTWA /'SPWA','SPWM'/,CLLN /'MTLY','MTLM'/
      DATA RGUIDE /'MTRN','MTRM'/, LGUIDE /'MTGN','MTGM'/
      DATA RADIUS /10.0, 22.0, 23.0, 22.0, 30.0 /

#include "arte/Arte.ff"
#include "auto/GEO.ff"

      IF(IFIRST.EQ.0) THEN
*
C--      read in detector characteristics
         CALL GEOPEN('fiber.dat')
         CALL FFINIT(0)
         CALL MTLKEY
         CALL FFSET('LINP',LREAD)
         CALL FFSET('LOUT',LLOGF)
         CALL FFGO
         CALL IOCLOS(LREAD)
*        
C        define user materials
         CALL ITLMED
*
C--      define rotation matrices
         CALL VZERO(IROTM,6)
         CALL CRROTM(IROTM(2),90.,-ANGMT,90.,90.-ANGMT,0.,0.)
         CALL CRROTM(IROTM(3),90., ANGMT,90.,90.+ ANGMT,0.,0.)
         CALL CRROTM(IROTM(4),180., 0.,90.,90.,90.,0.)
         CALL CRROTM(IROTM(5),180.,0,90.,90.-ANGMT,90.,-ANGMT)
         CALL CRROTM(IROTM(6),180., 0,90.,90.+ ANGMT,90.,ANGMT)
 16      IF(IDEBUG.NE.0)WRITE(LPRNT,'(''MTSSTR: ROTMAT = '',3(1X,I2))')
     +                                  IROTM
* 
c        define angle
         ANG = ANGMT/180.*PI
         COS1 = COS(ANG)
         SIN1 = SIN(ANG)
*
C--      define volumina 
c        walls
         CALL GSVOLU('SPWA','BOX ',MEDNUM(1,'CARBONFIBRE'),PAR,0,IVOL)
         CALL GSVOLU('SPWM','BOX ',MEDNUM(2,'CARBONFIBRE'),PAR,0,IVOL)
         CALL GSATT ('SPWA','COLO',2)
         CALL GSATT ('SPWM','COLO',2)
c        detectors 
         CALL GSVOLU('MTLY','BOX ',MEDNUM(3,'FIBER'),PAR,0,IVOLS(1))
         CALL GSVOLU('MTLM','BOX ',MEDNUM(4,'FIBER'),PAR,0,IVOLS(2))
         CALL GSATT ('MTLY','COLO',4)
         CALL GSATT ('MTLM','COLO',4)
c        lightguides (tube segments)
         CALL GSVOLU('MTRN','TUBS',MEDNUM(1,'POLYSTYREN'),PAR,0,IVOL)
         CALL GSVOLU('MTRM','TUBS',MEDNUM(2,'POLYSTYREN'),PAR,0,IVOL)
         CALL GSATT ('MTRN','COLO',7)
         CALL GSATT ('MTRM','COLO',7)
c        lightguides (end pieces)
         CALL GSVOLU('MTGN','TRD2',MEDNUM(1,'POLYSTYREN'),PAR,0,IVOL)
         CALL GSVOLU('MTGM','TRD2',MEDNUM(2,'POLYSTYREN'),PAR,0,IVOL)
         CALL GSATT ('MTGN','COLO',7)
         CALL GSATT ('MTGM','COLO',7)
*

c        thickness of layer = 2 * thickness of detector
         DLOVMT = 2.*DCELMT

         IFIRST = 1
      ENDIF
* 
C--   define the superlayer as current SL and implement it in GEANT
      CURRSL = ISL
      CALL UHTOC(HGESL_NAME(ISL),4,NAM,4)
      IMFLAG = IGESL_MFL(ISL)
      IF(IMFLAG.EQ.0) THEN
         CALL CGGESL(NAM,'MOTH',0)
         Z00 = ZZERO 
      ELSE
         CALL CGGESL(NAM,'MAGN',0)
         Z00 = ZMAGM
      ENDIF
      NSL = IUCOMP(NAM,DEMT,NSLMT)
      IF(NSL.EQ.0) CALL MEERRO('ITLGEO: '//NAM//' not found in DEMT')
      IF(IDEBUG.NE.0) WRITE(LPRNT,*)
     +                  'START ',NAM,' ',FGESL_Z1(ISL),FGESL_Z2(ISL)
*
C--   deactivate superlayer if wanted
      NPL = IGESL_LAY(ISL)
      IF(NPL.EQ.0) THEN
         CALL MEWARN( 'FIBER-SUPERLAYER '//NAM//' IS NOT ACTIVATED')
         GO TO 90
      ENDIF
*
c-------------------------------------------------------------------
C ***  install front rohacel plate
c-------------------------------------------------------------------
      PAR(1) =  .5*(FGESL_X2(ISL) - FGESL_X1(ISL))
      PAR(2) =  .5*(FGESL_Y2(ISL) - FGESL_Y1(ISL))
      PAR(3) = .5 * DWALMT
      ZP = .5*(FGESL_Z2(ISL)-FGESL_Z1(ISL))      
      CALL CRPOS(MTWA(IMFLAG+1),NAM,0,0,0.,0.,PAR(3)-ZP,0,'MANY',PAR)
*
c-------------------------------------------------------------------
C ***  install detectors
c-------------------------------------------------------------------
      ZP = -(FLOAT(NPL-1))*DCELMT
      IMTR = IGESL_LTYP(ISL)
      DO ILAY=1,NPL
      IR =  IMTR / 10**(NPL-ILAY)
      IMTR = IMTR - IR*10**(NPL-ILAY)
      IF(IR.EQ.0) THEN
         COSX = 1.
         SINX = 0.
      ELSE IF(IR.EQ.1) THEN
         COSX = COS1
         SINX = -SIN1
      ELSE IF(IR.EQ.2) THEN
         COSX = COS1
         SINX = SIN1
      ENDIF
*
      DO 85 ISC=1,8,2
      IF(ISC.EQ.1.OR.ISC.EQ.5) THEN
         ZPSV = ZP - .5*DCELMT
      ELSEIF(ISC.EQ.3.OR.ISC.EQ.7) THEN
         ZPSV = ZP + .5*DCELMT
      ENDIF
*
      IF(ISC.EQ.1) THEN
         A0 =  XIMT(NSL) * (1. - 0.5*SINX - COSX)
         B0 =  XIMT(NSL) * (1. + 0.5*SINX - COSX)
      ELSE IF(ISC.EQ.3) THEN
         A0 =  XIMT(NSL) * (1. + 0.5*SINX - COSX)
         B0 = -XIMT(NSL) * (1. - 0.5*SINX - COSX)
      ELSE IF(ISC.EQ.5) THEN
         A0 = -XIMT(NSL) * (1. - 0.5*SINX - COSX)
         B0 = -XIMT(NSL) * (1. + 0.5*SINX - COSX)
      ELSE IF(ISC.EQ.7) THEN
         A0 = -XIMT(NSL) * (1. + 0.5*SINX - COSX)
         B0 =  XIMT(NSL) * (1. - 0.5*SINX - COSX)
      ENDIF
*
c     each real sector is divided into two GEAN sectors, nsec = 0
c     stands for the leftmost of these two sectors
      NSEC = 0
      XMS=0.
*
 70   ISEC = ISC + NSEC
      XMIN = XLMT(NSL,ISEC) - A0
      XMAX = XRMT(NSL,ISEC) - A0
      YMIN = YLMT(NSL,ISEC) - B0
      YMAX = YRMT(NSL,ISEC) - B0
      XRG0 = .5*(XMIN+XMAX)
      YRG0 = .5*(YMIN+YMAX)
*
c     first the chambers itself will be generated
      PARSV(1) = .5 * (XMAX-XMIN)
      PARSV(2) = .5 * (YMAX-YMIN)
      PARSV(3) = .5 * DCELMT
      IF(IR.EQ.0) THEN
         XPSV = XRG0
         YPSV = YRG0
      ELSE
         XPSV   = XRG0*COSX - YRG0*SINX
         YPSV   = XRG0*SINX + YRG0*COSX
      ENDIF
*
      CALL CRPOS(CLLN(IMFLAG+1),NAM,ILAY,ISEC,XPSV,YPSV,ZPSV,
     +                                  IROTM(IR+1),'ONLY',PARSV)
      Q(QGEDE_ACC_L()) = SIGMT
      Q(QGEDE_EFF_L()) = EFFMT
      Q(QGEDE_PIT_L()) = PITMT
      Q(QGEDE_DOD_L()) = DIMXMT
      IQ(IQGEDE_NSL_L())  = 1
*
      IF(NSEC.EQ.0) THEN
         Q(QGEDE_SXMN_L()) = XMIN + RANDMT
         Q(QGEDE_SXMX_L()) = XMAX
         XLOWIR            = XMIN + RANDMT + .5*PITMT
         XUPWIR            = XMAX          - .5*PITMT
      ELSE
         Q(QGEDE_SXMN_L()) = XMIN
         Q(QGEDE_SXMX_L()) = XMAX - RANDMT
         XLOWIR            = XMIN          + .5*PITMT
         XUPWIR            = XMAX - RANDMT + .5*PITMT
      ENDIF
      Q(QGEDE_SYMN_L()) = YMIN + RANDMT
      Q(QGEDE_SYMX_L()) = YMAX - RANDMT
*
      Q(QGEDE_XFW_L(1))  = XLOWIR
      Q(QGEDE_ZFW_L(1))  = FGEDE_ZMIN(NGEDE()) + DGL1MT + DAUXMT
      Q(QGEDE_ZFW_L(2))  = FGEDE_ZFW(1,NGEDE()) + DGASMT
      IQ(IQGEDE_NWI_L(1)) = (XUPWIR-XLOWIR)/PITMT + 1.001
*
      IF(NSEC.EQ.0) THEN
         CALL GRADLN(IVOLS(IMFLAG+1),XMS)
         XMS = DCELMT/XMS
      ENDIF
      Q(QGEDE_MS_L())= XMS*(.0136*(1.+.038*ALOG(XMS)))**2
*
c--------------------------------------------------------------------
c new add light guide
c--------------------------------------------------------------------
*
      IF(ISEC.EQ.2 .OR. ISEC.EQ.4 .or. ISEC.EQ.5 .or. ISEC.EQ.7) THEN
         DR       = MAX(ABS(XRMT(NSL,ISEC)),ABS(XLMT(NSL,ISEC)))
         PARRG(1) = RADIUS(MIN(5,NSL))
         PARRG(2) = PARRG(1)+ DR
c        1.4 is a factor for the different thickness
         PARRG(3) = 0.5 * MIN(2.*DCELMT, 1.4 * DGASMT)
         PARRG(4) = (ISC+1) * 45.
         PARRG(5) = PARRG(4) + 90.
*
C center of circle
         PARLG(5) = 0.5*(.5*(FGESL_X2(ISL)-FGESL_X1(ISL))-PARRG(2))

         IF(ISEC.EQ.2 .OR. ISEC.EQ.4) THEN
            XRG0 = XRG0 + PARSV(1) + PARRG(1)
            XLG0 = XRG0 + PARLG(5)
         ELSE
            XRG0 = XRG0 - PARSV(1) - PARRG(1)
            XLG0 = XRG0 - PARLG(5)
         ENDIF

         IF(ISEC.EQ.2 .OR. ISEC.EQ.7) THEN
            YRG0 = YRG0 + PARSV(2)
            YLG0 = YRG0 + 0.5*(PARRG(1) + PARRG(2))
         ELSE
            YRG0 = YRG0 - PARSV(2)
            YLG0 = YRG0 - 0.5*(PARRG(1) + PARRG(2))
         ENDIF

C now rotate the center
         IF(IR.EQ.0) THEN
           XRG = XRG0
           YRG = YRG0
           XLG = XLG0
           YLG = YLG0
         ELSE
           XRG   = XRG0*COSX-YRG0*SINX
           YRG   = XRG0*SINX+YRG0*COSX
           XLG   = XLG0*COSX-YLG0*SINX
           YLG   = XLG0*SINX+YLG0*COSX
         ENDIF
         CALL GSPOSP(RGUIDE(IMFLAG+1), NGEDE(), NAM, XRG, YRG, ZPSV,
     +             IROTM(IR+1), 'ONLY', PARRG, 5)

         PARLG(3) = 0.5 * DR 
         PARLG(1) = PARRG(3)

         IF(NSL.GE.6) THEN
            PARLG(4) = ABS(YLG0)
            PARLG(2) = PARRG(3) * PARLG(3) / PARLG(4)
         ELSE
            PARLG(4) = PARLG(3)
            PARLG(2) = PARLG(1)
         ENDIF

         IF(ISEC.EQ.5 .OR. ISEC.EQ.7) THEN
            SWOP     = PARLG(2)
            PARLG(2) = PARLG(1)
            PARLG(1) = SWOP
  
            SWOP     = PARLG(3)
            PARLG(3) = PARLG(4)
            PARLG(4) = SWOP
         ENDIF
         CALL GSPOSP(LGUIDE(IMFLAG+1), NGEDE(), NAM, XLG, YLG, ZPSV,
     +             IROTM(IR+4), 'ONLY', PARLG, 5)

      ENDIF

      NSEC = NSEC + 1
      IF(NSEC.EQ.1) GO TO 70
 85   CONTINUE 

      ZP = ZP + 2.*DCELMT
      ENDDO

c-------------------------------------------------------------------
C *** install  back rohacel plate
c-------------------------------------------------------------------

      PAR(1) =  .5*(FGESL_X2(ISL) - FGESL_X1(ISL))
      PAR(2) =  .5*(FGESL_Y2(ISL) - FGESL_Y1(ISL))
      PAR(3) = .5 * DWEXMT
      ZP = .5*(FGESL_Z2(ISL)-FGESL_Z1(ISL))      
      CALL CRPOS(MTWA(IMFLAG+1),NAM,0,0,0.,0.,ZP-PAR(3),0,'MANY',PAR)

 90   RETURN
      END
