/*
************************************************************************
*
* MAGGEO
* written by Siegmund Nowak
*
* This routines defines geometry of the beam pipe outside the si-region
* and up to the muon system
*
*       called from G..MAIN
*
************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/Arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteUtil.h"
#include "gean/gclink.hh"
#include "gean/ZGMOTH.hh"
#include "auto/GEO.hh"
#include "gean/ZGESL.h"
#include "gean/ZGMAG.hh"
#include "gean/ZGDETV.hh"

  struct FELD fld;  

  extern "C" {
    void maggeo_();
    void gsvolu_();
    void gsatt_();
    void crpos_();
    void crrotm_();    
    void mednumc_();
    void matnumc_();
    int  magdat_();
    extern ZGMAG magbuf_;
    extern CommonZGMOTH zgmoth_;
    extern CommonZGDETV zgdetv_;
    extern Commongclink gclink_;
  }

void maggeo_() {

    ZGMAG&  mag=magbuf_;
    CommonZGMOTH& mut=zgmoth_;
//    CommonZGDETV& detv=zgdetv_;
    Commongclink& glink=gclink_;
    int ivol, imed, imat, irot, len, color; 
    int zero=0, one=1; 
    char shap[4], colo[4],  only[4], mot[4],  vol[5],  mat[13];
    float fzero =0., fninety=90.;
    float par[5], xp, yp, zp;
    double vers;

// standard definitions 
    strncpy(mot,"BOX ",4); 
    strncpy(colo,"COLO",4);
    strncpy(only,"ONLY",4);
    crrotm_(&irot , &fzero, &fzero, &fninety,&fzero, &fninety, &fninety);

//  select file name 
    static char filnam[16];
    strcpy(filnam,"mag.dat");           

//  posit volumes within mother 

    for (;;)   {
    int fi = magdat_(filnam);
    if (fi == 0) continue;
    if (fi != 0) {
      union  { int ii; char bla[4];};
      int i=0, k=0;
      int nvol=*(gcbank_.iq + glink.jvolum -2); 
      while( i<nvol) {
        ii = *(gcbank_.iq+glink.jvolum+i);
        if(!strncmp(bla,mag.object,4)) {
           k=i+1; 
           break; }
        i++; }
      strncpy(mot,mag.mother,4);
      strncpy(vol,mag.object, 4);
      strncpy(shap,mag.shape, 4);
      len = strlen(mag.material);
      if(len>12) len = 12;
      strncpy(mat,mag.material, len);
      matnumc_(&mat,&len,&imat);
      if (k == 0)  { 
         mednumc_(&mag.sens_flag, &mat, &len, &imed);
	 gsvolu_(&vol,&shap, &imed, &par, &zero, &ivol);
         gsatt_(&vol, &colo, &one); }

      if(!strncmp(shap,"BOX ",4)) {         
         par[0] = 0.5 * mag.dx;
         par[1] = 0.5 * mag.dy;
         par[2] = 0.5 * mag.dz;   }
      else if (!strncmp(shap,"TRD1",4)) {
         par[0] = 0.5 * mag.dx;
         par[1] = 0.5 * mag.dy;
         par[2] = 0.5 * mag.dz;  
         par[3] = 0.5 * mag.du;   }
      else if (!strncmp(shap,"TUBE",4)) {
         par[0] = mag.dx;
         par[1] = mag.dy;
         par[2] = 0.5 * mag.dz;   }
      else if (!strncmp(shap,"TUBS",4)) {
         par[0] = mag.dx;
         par[1] = mag.dy;
         par[2] = 0.5 * mag.dz; 
         par[3] = mag.du;
         par[4] = mag.dv;         }

      xp = mag.xpos;
      yp = mag.ypos;
      zp = mag.zpos; 
      color = mag.color;
      if(mag.rotation == 0)
         crpos_(&vol,&mot,&zero,&zero,&xp,&yp,&zp,&zero,&only, &par);
      else
         crpos_(&vol,&mot,&zero,&zero,&xp,&yp,&zp,&irot,&only, &par);
      gsatt_(&vol,&colo,&color); 
      if ( fi <= 0 ) break; }}

      ArtePointer<GESL> geslfeld(mut.FELDSL);
      fld.x = geslfeld->x1;
      fld.y = geslfeld->y1;
      fld.z = geslfeld->z1; 
      fld.r = geslfeld->x2;
      fld.h = geslfeld->y2;
      int beg = 4*(LGESL_FLD-1) + 1; 
      char *fmp =zgdetv_.DETV + beg -1;  
      if(!strncmp(fmp,"FLDS",4)) geslfeld->mfl =0;
      else   geslfeld->mfl =1; 

      ArtePointer<FELD>::append(fld); 
}  
 /* maggeo_ */






