      subroutine otwl96(isl)
******************************************************************
*
* Defines walls for Zeuthen OTR superlayers for run 96/97.
* Geant volumes are defined and the corresponding Arte tables.
*
* Called from: otrsl.F
*
* The definition of walls was copied
* from wtlg96.F on 14-Apr-1997 (written by S.Nowak, C.Stegmann),
* and then adapted for calls from otrsl.F routine.
*
* External references:
*
* Arguments  in/out       Description
*
* isl        in           superlayer number (index for GESL table)
*
* Alexander Lanyov     May 1997
************************************************************************
      implicit none

#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGBPIP.fi"
#include "gean/ZGOTR.fi"
#include "gean/ZGDETV.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"

#include "gean/otrgeo.fi"

      integer nsl,isl,i,imfl,igede
ccc     real nwi(2),ng10(2)
      real par(8),xp,yp,zp
      character*4 nmsl,iron(2),plex(2),alum(2),roha(2),g10(2),cabl(2)

c Dimension of cable frame

c a) Dimension of gas frame
      real dxgb, dygb, dzgb
      parameter (dxgb=70.,dygb=134.,dzgb=8.)

c b) alu plate
      real dxalu, dyalu, dzalu
      parameter (dxalu=70.,dyalu=134.,dzalu=0.1)

c c) alu u-profile 80*45*6 beam off side
      real dxalup, dyalup, dzalup
      parameter (dxalup=6.8, dyalup=4.5, dzalup=0.6)

c d) rohacell plate
      real dxroh,dyroh, dzroh
      parameter (dxroh=70.,dyroh=100.,dzroh=0.4)

c d) space between rohacell wall and modules
      real dxplex, dyplex, dzplex
      parameter (dxplex=70., dyplex=15., dzplex=0.525)

c e) G10-strips (size and y-position)
c    and inefficient region around G10 strips (YDG10)
      real dyg10, dzg10, ydg10
      parameter (dyg10 = .6, dzg10 = .01, ydg10 = 1.)

c Dimension of modules
cccc      real dxmod(2), dymod(2), dzmod(2), dztyp(2)

c distance of closest approach of 10 and 5 mm tubes in x
      real dca
      parameter (dca=0.05)

c sector sizes
ccc      real dysec(12), ysec(12), zsec(12)
      real xsh,ysh,zsh
      real dxrack,dyrack,dzrack
      parameter (dxrack=10.,dyrack=287.,dzrack=1.)

        real x,y,z
        real xdiff,ydiff,zdiff
        real xmin_box,xmax_box,ymin_box,ymax_box,zmin_box,zmax_box
        real zmin,zmax

      data iron /'WTIR','WTIM'/, PLEX /'WTPL','WTPM'/,
     +     alum /'WTAL','WTAM'/, ROHA /'WTRO','WTRM'/,
     +     g10  /'WTGT','WTGM'/, CABL /'WTCA','WTCM'/

ccc      data nwi /32,14/, ng10/6,2/
ccc      data yg10 /18.5,17.5,.5,-0.5,17.5,-18.5,.5,-0.5/
cc
cc      data dymod /95.2, 97.9/, dzmod/1.5, 1.5/, dztyp/.5,1./
cc
cc      data dysec /30.1,    30.1,  19.0,     19.0,    19.0,    19.0,
cc     +            30.1,    30.1,  49.45,    48.45,   48.45,   49.45 /
cc      data ysec  /32.55,   32.55,  9.0 ,     9.0 ,   -9.0 ,   -9.0 ,
cc     +           -32.55,  -32.55, 24.225,   24.275, -24.275, -24.225 /
cc      data zsec  /  0.25,    0.,    -0.25,     -0.5,      0.25,    0.,
cc     +             -0.25,   -0.5,    0.25,     -0.25,     0.25,   -0.25/

#include "arte/Arte.ff"
#include "auto/GEO.ff"

c-- shifts of gas box with respect to superlayer
      xsh = 22.7
      ysh =  0.0
cc      ZSH = 0.5*(FGESL_Z2(ISL)-FGESL_Z1(ISL)) - DZGB
      ZSH = 0.5*9.4 - DZGB   ! = 0.5*9.4 - 8 = -3.3

c------------------ Positions of empty box to put the chambers

c horizontal u-profile 45*80*6 beam off side
        xmin_box = xsh - 0.5*dxalu + 2.2        ! WTAL
c horizontal wall beam side
        xmax_box = xsh + 0.5*dxalu + 2.2        ! WTAL

        ymin_box = -70.
        ymax_box = +70.

c For Z coordinate we have 3 approaches (all give the same result)
c
c 1. vertical endpiece ( front side & backside)
        zmin_box = zsh        + dzalu + dzalup          ! alum
        zmax_box = zsh + dzgb - dzalu - dzalup          ! alum

        if (iverbose.ge.2) then
           write (c78,48) 'BOX1',
     @        xmin_box,xmax_box,ymin_box,ymax_box,zmin_box,zmax_box
           call me78sp('T')
        end if

c 2. front and back alu plates
        zmin_box = zsh + dzalu                          ! alum
        zmax_box = zsh + dzgb - dzalu                   ! alum

        if (iverbose.ge.2) then
           write (c78,48) 'BOX2',
     @        xmin_box,xmax_box,ymin_box,ymax_box,zmin_box,zmax_box
           call me78sp('T')
        end if

c 3. Rohacell walls
        zmin_box = zsh + dzalu + 0.1 + dzroh            ! roha
        zmax_box = zsh + dzgb - 0.4 - 0.5*0.4           ! roha

        if (iverbose.ge.2) then
           write (c78,48) 'BOX3',
     @        xmin_box,xmax_box,ymin_box,ymax_box,zmin_box,zmax_box
           call me78sp('T')
        end if

c--------------------------------------------------------------------------pc01
c OTWL96: BOX1= [  -10.10,   59.90], [  -70.00,   70.00], [   -4.53,    2.07],
c OTWL96: BOX2= [  -10.10,   59.90], [  -70.00,   70.00], [   -5.13,    2.67],
c OTWL96: BOX3= [  -10.10,   59.90], [  -70.00,   70.00], [   -4.63,    2.17],
c OTWL96: BOX = [  -10.10,   59.90], [  -70.00,   70.00], [   -4.63,    2.17],
c OTWL96: BOXf= [  -81.98,  -11.98], [  -70.00,   70.00], [  732.00,  738.80],
c OTWL96: ALL = [  -76.84,  -17.13], [  -49.79,   49.79], [  732.62,  738.17],
c OTWL96: SL  = [  -75.53,  -19.28], [  -54.72,   54.72], [  732.62,  738.17],
c OTWL96: offsets are  -24.48     .00    1.23
c------------------------------------------------------------------------------


c------------------ Center the chambers in the empty box
        x = 0.5*(xmin_all+xmax_all) - 0.5*(xmin_box+xmax_box) - xp_sl
        y = 0.5*(ymin_all+ymax_all) - 0.5*(ymin_box+ymax_box) - yp_sl
        z = 0.5*(zmin_all+zmax_all) - 0.5*(zmin_box+zmax_box) - zp_sl

        if (iverbose.ge.2) then
           write (c78,48) 'BOX ',
     @        xmin_box,xmax_box,ymin_box,ymax_box,zmin_box,zmax_box
           call me78sp('T')
           write (c78,48) 'BOXf',
     @ xmin_box+x,xmax_box+x,ymin_box+y,ymax_box+y,zmin_box+z,zmax_box+z
           call me78sp('T')
           write (c78,48) 'ALL ',
     @        xmin_all,xmax_all,ymin_all,ymax_all,zmin_all,zmax_all
           call me78sp('T')
           write (c78,48) 'SL  ',
     @        xmin_sl,xmax_sl,ymin_sl,ymax_sl,zmin_sl,zmax_sl
           call me78sp('T')

48         format(' OTWL96: ',a,'= ',3('[',f8.2,',',f8.2,'], '))

           write (c78,'('' OTWL96: offsets are'',3f8.2)') x,y,z
           call me78sp('T')
        end if

c------------------ Do the chambers fit into the empty box?
        xdiff = (xmax_box - xmin_box) - (xmax_all - xmin_all)
        ydiff = (ymax_box - ymin_box) - (ymax_all - ymin_all)
        zdiff = (zmax_box - zmin_box) - (zmax_all - zmin_all)
        if (xdiff.lt.0.) then
           c78='OTWL96: Modules do not fit into on X axis: difference ='
           call ch78bf(xdiff,-2)
           call me78sp('T')
        end if
        if (ydiff.lt.0.) then
           c78='OTWL96: Modules do not fit into on Y axis: difference ='
           call ch78bf(ydiff,-2)
           call me78sp('T')
        end if
        if (zdiff.lt.0.) then
           c78='OTWL96: Modules do not fit into on Z axis: difference ='
           call ch78bf(zdiff,-2)
           call me78sp('T')
        end if

c------------------

C--   install the superlayer volume
      currsl = isl
      imfl = igesl_mfl(isl) + 1
      call uhtoc(hgesl_name(isl),4,nmsl,4)

      nsl = igesl_nsl(isl)

        if (ichwa.eq.1) goto 100        ! in this case install only chamber walls

c-- install the iron frame
      par(1) =0.5*110.0
      par(2) =0.5* 30.0
      par(3) =0.5*  0.4
      xp =  -5.0
      yp =  80.0
      zp =  -4.5
      CALL GSPOSP(IRON(IMFL),NGWAL()+1,NMSL,
     @                                   XP+x, YP+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(IRON(IMFL),NGWAL()+1)
      CALL GSPOSP(IRON(IMFL),NGWAL()+1,NMSL,
     @                                   XP+x,-YP+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(IRON(IMFL),NGWAL()+1)

      par(1) =0.5* 30.0
      par(2) =0.5*189.2
      par(3) =0.5*  0.4
      XP = -45.0
      YP =   0.0
      ZP =  -4.1
      CALL GSPOSP(IRON(IMFL),NGWAL()+1,NMSL,
     @                                   XP+x, YP+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(IRON(IMFL),NGWAL()+1)

      par(1) =0.5*110.0
      par(2) =0.5*  0.4
      par(3) =0.5*  9.0
      XP =  -5.0
      YP =  94.8
      ZP =   0.2
      CALL GSPOSP(IRON(IMFL),NGWAL()+1,NMSL,
     @                                   XP+x, YP+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(IRON(IMFL),NGWAL()+1)

      CALL GSPOSP(IRON(IMFL),NGWAL()+1,NMSL,
     @                                   XP+x,-YP+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(IRON(IMFL),NGWAL()+1)

      par(1) =0.5*  0.4
      par(2) =0.5*189.2
      par(3) =0.5*  8.6
      XP = -59.8
      YP =   0.0
      ZP =   0.4
      CALL GSPOSP(IRON(IMFL),NGWAL()+1,NMSL,
     +                             XP+x, YP+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(IRON(IMFL),NGWAL()+1)

c-- install the two plexi glas holder

      par(1) =0.5* 74.6
      par(2) =0.5*  7.0
      par(3) =0.5*  1.0
      XP =  22.7
      YP =  68.5
      ZP =  -3.8
      CALL GSPOSP(PLEX(IMFL),NGWAL()+1,NMSL,
     @                                   XP+x, YP+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(PLEX(IMFL),NGWAL()+1)
      CALL GSPOSP(PLEX(IMFL),NGWAL()+1,NMSL,
     @                                   XP+x,-YP+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(PLEX(IMFL),NGWAL()+1)

C install the front alu plate of the gas box

c front plate
      par(1) = 0.5*DXALU
      par(2) = 0.5*DYALU
      par(3) = 0.5*DZALU
      XP = 2.2
      YP = 0.
      ZP = PAR(3)
      CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,YP+YSH+y,ZP+ZSH+z,0,'ONLY',par,3)
      CALL CRGWAL(ALUM(IMFL),NGWAL()+1)

c horizontal u-profile 45*80*6 beam off side
      par(1) = 0.5*DYALUP
      par(2) = 0.5*DYALU
      par(3) = 0.5*DZALUP
      XP = -0.5*(DXALU+DYALUP) + 2.2
      YP =   0.
      ZP =   PAR(3)
      CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,YP+YSH+y,ZP+ZSH+z,0,'ONLY',par,3)
      CALL CRGWAL(ALUM(IMFL),NGWAL()+1)

      ZP =  DZGB-PAR(3)
      CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,YP+YSH+y,ZP+ZSH+z,0,'ONLY',par,3)
      CALL CRGWAL(ALUM(IMFL),NGWAL()+1)
      par(1) =0.5*DZALUP
      par(2) =0.5*DYALU
      par(3) =0.5*DXALUP
      XP = -0.5*DXALU-PAR(1) + 2.2
      YP =   0.
      ZP =  0.5*DZGB
      CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,YP+YSH+y,ZP+ZSH+z,0,'ONLY',par,3)
      CALL CRGWAL(ALUM(IMFL),NGWAL()+1)

c horizontal wall beam side
      par(1) =0.5*DZALU
      par(2) =0.5*DYALU
      par(3) =0.5*DZGB
      XP = 0.5*DXALU+PAR(1) + 2.2
      YP =   0.
      ZP = 0.5*DZGB
      CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,YP+YSH+y,ZP+ZSH+z,0,'ONLY',par,3)
      CALL CRGWAL(ALUM(IMFL),NGWAL()+1)

c vertical endpiece front side
      par(1) = 0.5*DXALU
      par(2) = 0.5*17.
      par(3) = 0.5*DZALUP
      XP =   2.2
      YP = 0.5*DYALU - PAR(2)
      ZP =  DZALU+PAR(3)
      CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x, YP+YSH+y,ZP+ZSH+z,0,'ONLY',par,3)
      CALL CRGWAL(ALUM(IMFL),NGWAL()+1)
      CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,-YP+YSH+y,ZP+ZSH+z,0,'ONLY',par,3)
      CALL CRGWAL(ALUM(IMFL),NGWAL()+1)

      ZP = ZSH + DZALU

c inside the gas box
c install the front rohacel wall
c G10 and rohacell front side
      par(1) =0.5*DXROH
      par(2) =0.5*DYROH
      par(3) =0.5*.1
      XP =   2.2
      YP =   0.
      CALL GSPOSP(G10(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,YP+YSH+y,ZP+PAR(3)+z,0,'ONLY',par,3)
      CALL CRGWAL(G10(IMFL),NGWAL()+1)
      ZP = ZP + 2.*PAR(3)
      par(3) =0.5*DZROH
      CALL GSPOSP(ROHA(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,YP+YSH+y,ZP+PAR(3)+z,0,'ONLY',par,3)
      CALL CRGWAL(ROHA(IMFL),NGWAL()+1)
      ZP = ZP + 2.*PAR(3)
      par(3) =   0.1/2.
      CALL GSPOSP(G10(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,YP+YSH+y,ZP+PAR(3)+z,0,'ONLY',par,3)
      CALL CRGWAL(G10(IMFL),NGWAL()+1)
      ZP = ZP + 2.*PAR(3)

c detector planes etc

100     continue
      DO I=1,IGESL_LAY(ISL)               ! loop over layers

c now find all GEDE planes from given layer and get their z
coordinates
         zmin = 1.e10
         zmax = -zmin
         do igede = igesl_fpl(isl), igesl_fpl(isl) + igesl_npl(isl) - 1
            if (igede_lay(igede) .eq. i) then
               zmin = min(zmin,fgede_zmin(igede))
               zmax = max(zmax,fgede_zmax(igede))
            end if
         end do

         if (zmin .ge.zmax) then
            c78 = 'OTWL96: zmin='
            call ch78af(zmin,-2)
            call ch78ac(' >= ')
            call ch78af(zmax,-2)
            call meerro (c78)
         end if

ccc         zp = zp +0.5*dzplex
cc layer center         zp = 0.5*(zmin + zmax) - zp_sl
         zp = zmin -z -dzplex - zp_sl

c-- lower alu material
         par(1) = 0.5*DXALU
         par(2) = 0.5*17.
         par(3) =   0.2/2.
         XP =   2.2
         YP = 0.5*DYALU - PAR(2)
         CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +               XP+XSH+x,-YP+YSH+y,ZP+z,0,'ONLY',par,3)
         CALL CRGWAL(ALUM(IMFL),NGWAL()+1)

c-- lower plexi glas material
         PAR(1) =0.5*DXPLEX
         par(2) =0.5*DYPLEX
         par(3) =0.5*DZPLEX
         YP = -53.0
         CALL GSPOSP(PLEX(IMFL),NGWAL()+1,NMSL,
     +               XP+XSH+x,YP+YSH+y,ZP+z,0,'ONLY',par,3)
         CALL CRGWAL(PLEX(IMFL),NGWAL()+1)

c-- upper alu material
c-- to include overlapping of rotated hv-boards
c-- with active parts of modules 17.0 --> 18.0
         par(2) =  18.0/2.
         par(3) =   0.2/2.
         YP =  58.0
         CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +               XP+XSH+x,YP+YSH+y,ZP+z,0,'ONLY',par,3)
         CALL CRGWAL(ALUM(IMFL),NGWAL()+1)

c-- install preamp's !1'
         par(1) =  28.0/2.
         par(2) =   7.0/2.
         par(3) =   0.04/2.
         XP =  17.2
         YP = 0.5*DYALU +PAR(2)
         CALL GSPOSP(CABL(IMFL),NGWAL()+1,NMSL,
     +               XP+XSH+x, YP+YSH+y,ZP+z,0,'ONLY',par,3)
         CALL CRGWAL(CABL(IMFL),NGWAL()+1)
         CALL GSPOSP(CABL(IMFL),NGWAL()+1,NMSL,
     +               XP+XSH+x,-YP+YSH+y,ZP+z,0,'ONLY',par,3)
         CALL CRGWAL(CABL(IMFL),NGWAL()+1)

         par(1) =  14.0/2.
         par(3) =   0.02/2.
         XP = -15.2
         CALL GSPOSP(CABL(IMFL),NGWAL()+1,NMSL,
     +               XP+XSH+x,YP+YSH+y,ZP+z,0,'ONLY',par,3)
         CALL CRGWAL(CABL(IMFL),NGWAL()+1)

         CALL GSPOSP(CABL(IMFL),NGWAL()+1,NMSL,
     +               XP+XSH+x,-YP+YSH+y,ZP+z,0,'ONLY',par,3)
         CALL CRGWAL(CABL(IMFL),NGWAL()+1)
         ZP = ZP +0.5*DZPLEX

c Here was installation of modules!

ccc         zp = zp +0.5*dzplex
cc layer center         zp = 0.5*(zmin + zmax) - zp_sl
         zp = zmax -z - zp_sl

      END DO                          !     layers

        if (ichwa.eq.1) return  ! in this case install only chamber walls

c install last alu and plexiglas material
      ZP = ZP +0.5*DZPLEX
c-- lower alu material
      par(1) = 0.5*DXALU
      par(2) = 0.5*17.
      par(3) =   0.2/2.
      XP =   2.2
      YP = 0.5*DYALU - PAR(2)
      CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,-YP+YSH+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(ALUM(IMFL),NGWAL()+1)

c-- lower plexi glas material
      par(2) =  15.0/2.
      par(3) =  0.5*DZPLEX
      YP = -53.0
      CALL GSPOSP(PLEX(IMFL),NGWAL()+1,NMSL,
     +               XP+XSH+x,YP+YSH+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(PLEX(IMFL),NGWAL()+1)

c-- upper alu material
c-- to include overlapping of rotated hv-boards
c-- with active parts of modules 17.0 --> 18.0
      par(2) =  18.0/2.
      par(3) =   0.2/2.
      YP =  58.0
      CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +               XP+XSH+x,YP+YSH+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(ALUM(IMFL),NGWAL()+1)
      ZP = ZP + 0.5*DZPLEX

c G10 and rohacell back side
      par(1) =0.5*DXROH
      par(2) =0.5*DYROH
      par(3) =   0.1/2.
      XP =   2.2
      YP =   0.
      ZP =   DZGB - .65
      CALL GSPOSP(G10(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,YP+YSH+y,ZP+ZSH+z,0,'ONLY',par,3)
      CALL CRGWAL(G10(IMFL),NGWAL()+1)

      par(3) =   0.4/2.
      ZP =   DZGB - .4
      CALL GSPOSP(ROHA(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,YP+YSH+y,ZP+ZSH+z,0,'ONLY',par,3)
      CALL CRGWAL(ROHA(IMFL),NGWAL()+1)

      par(3) =   0.1/2.
      ZP =   DZGB - .15
      CALL GSPOSP(G10(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,YP+YSH+y,ZP+ZSH+z,0,'ONLY',par,3)
      CALL CRGWAL(G10(IMFL),NGWAL()+1)

c back alu plate plate
      par(1) =0.5*DXALU
      par(2) =0.5*DYALU
      par(3) =0.5*DZALU
      XP =   2.2
      YP =   0.
      ZP = DZGB-PAR(3)
      CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,YP+YSH+y,ZP+ZSH+z,0,'ONLY',par,3)
      CALL CRGWAL(ALUM(IMFL),NGWAL()+1)

c vertical endpiece backside
      par(1) = 0.5*DXALU
      par(2) = 0.5*17.
      par(3) = 0.5*DZALUP
      XP =   2.2
      YP = 0.5*DYALU - PAR(2)
      ZP = DZGB - (DZALU+PAR(3))
      CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x, YP+YSH+y,ZP+ZSH+z,0,'ONLY',par,3)
      CALL CRGWAL(ALUM(IMFL),NGWAL()+1)
      CALL GSPOSP(ALUM(IMFL),NGWAL()+1,NMSL,
     +             XP+XSH+x,-YP+YSH+y,ZP+ZSH+z,0,'ONLY',par,3)
      CALL CRGWAL(ALUM(IMFL),NGWAL()+1)

c-- install cables

      par(1) = 110.0/2.
      par(2) =   6.0/2.
      par(3) =   0.5/2.
      XP =  -5.0
      YP =  92.0
      ZP =  -4.05
      CALL GSPOSP(CABL(IMFL),NGWAL()+1,NMSL,
     @                                   XP+x, YP+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(CABL(IMFL),NGWAL()+1)
      CALL GSPOSP(CABL(IMFL),NGWAL()+1,NMSL,
     @                                   XP+x,-YP+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(CABL(IMFL),NGWAL()+1)

      par(1) =   6.0/2.
      par(2) = 178.0/2.
      par(3) =   0.5/2.
      XP = -57.0
      YP =   0.0
      ZP =  -3.65
      CALL GSPOSP(CABL(IMFL),NGWAL()+1,NMSL,
     @                                   XP+x, YP+y,ZP+z,0,'ONLY',par,3)
      CALL CRGWAL(CABL(IMFL),NGWAL()+1)

c install the rack tube near the beam pipe and corner cross connection

c for PC03 only
      IF(NSL.EQ.4) THEN

cccccc IS IT NEEDED TO SHIFT x,y,z ??????????????
         x = 0.
         y = 0.
         z = 0.

         PAR(1) = 0.5 * DXRACK     !  10 cm
         PAR(2) = 0.5 * DYRACK     ! 287 cm
         PAR(3) = 0.5 * DZRACK     !   1 cm
         XP = 25.
         YP = -169.7 + 0.5*DYRACK
         ZP = 940.+ ZZERO

         CALL CRPOS('WTIR','MOTH',0,0,XP+x,YP+y,ZP-0.5*DXRACK+PAR(3)+z
     +                                                  ,0,'ONLY',PAR)
         CALL CRPOS('WTIR','MOTH',0,0,XP+x,YP+y,ZP+0.5*DXRACK-PAR(3)+z
     +                                                  ,0,'ONLY',PAR)
         PAR(1) = 0.5 * DZRACK
         PAR(3) = 0.5 *DXRACK - DZRACK
         CALL CRPOS('WTIR','MOTH',0,0,XP-0.5*DXRACK+PAR(1)+x,YP+y,ZP+z
     +                                                  ,0,'ONLY',PAR)
         CALL CRPOS('WTIR','MOTH',0,0,XP+0.5*DXRACK-PAR(1)+x,YP+y,ZP+z
     +                                                  ,0,'ONLY',PAR)

         ZP = ZP -0.5*DXRACK
         PAR(1) = 120.+0.5*DXRACK
         PAR(2) = .4
         PAR(3) =0.5*(8-0.8)
         XP = XP - 120.
         YP = -169.7 + 61.
         CALL CRPOS('WTIR','MOTH',0,0,
     @                        XP+x,YP+y,ZP-0.8-PAR(3)+z,0,'ONLY',PAR)
         PAR(2) =0.5*8.
         PAR(3) =0.5*.8
         CALL CRPOS('WTIR','MOTH',0,0,XP+x,YP-PAR(2)+y,ZP-PAR(3)+z,
     +                                                  0,'ONLY',PAR)
      ENDIF

 999  RETURN
      END
