/*
pppppppppp   tttttttttttt  ll          bbbbbbbbbbb   uu       uu  fffffffffff
ppppppppppp  tttttttttttt  ll          bbbbbbbbbbbb  uu       uu  fffffffffff
pp       pp       tt       ll          bb        bb  uu       uu  ff
pp       pp       tt       ll          bb        bb  uu       uu  ff
pp       pp       tt       ll          bb       bb   uu       uu  ff
ppppppppppp       tt       ll          bbbbbbbbbb    uu       uu  ffffff
pppppppppp        tt       ll          bbbbbbbbbb    uu       uu  ffffff
pp                tt       ll          bb       bb   uu       uu  ff
pp                tt       ll          bb        bb  uu       uu  ff
pp                tt       ll          bb        bb  uu       uu  ff
pp                tt       llllllllll  bbbbbbbbbbbb  uuuuuuuuuuu  ff
pp                tt       llllllllll  bbbbbbbbbbb    uuuuuuuuu   ff
*/

/**********************************************************************
 *
 * Author: I.Belyaev
 * Date: 1997/10/04 
 *
 * Created: Apr 10, 1997 by Ivan Belyaev
 *
 * Called from: PTLDAT 
 *
 * Description:
 *
 * It reads the lines from file and put them into 
 * FORTRAN common block     PTBUFF
 *
 * Arguments:
 *      
 * name        Argument description
 *
 * name  in     name of file to be opened and read.
 *
 **********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <iostream.h>
#include <math.h>
#include "gean/hgdat.h"
#include "arte/Arte.hh"
#include "arte/ArteUtil.h"

/*
  ------------------------------------------------------------------------
*/

static int  first=1       ;
static char filename[256] ;
FILE   *    file          ; 
int         ptlbuf( char*);
int const   debug=0       ;

typedef struct 
{
  int  buflen;
  char buffer[256];
} Commonptlbuf;

extern "C" 
{
  float  rndm_(   float const   & ); 
  void   me78wc_( int   const   & );
  void   me78ap_()         ;
  int    ptlbuf_( char *name) { return ptlbuf(name); }
  extern Commonptlbuf   ptbuff_    ;
}

/* inline function for C78 manipulation - 
   put name into c78, remove NULL at the end and fill blanks */
inline int c78put( char *text ) 
{
  int __l78 = 0;   
  for( int __i78 = 0 ; __i78 < 78 ; ++__i78 )
    {
      if( __i78 < strlen( text ) )
	{
	  __l78++; 
	  arteutil_.c78[ __i78 ] = text[ __i78 ] ; 
	}
      else
	{
	  arteutil_.c78[ __i78 ] = ' ' ;
	}
    }
  return __l78;
}

//Commonptlbuf   ptbuff_;
char *getenv (const char *name);

/* **************************************************************************
 *  Files are looked in the following directories:                          *
 *   PWD and then in $HB/$HBVERS/geo/                                       *
 ****************************************************************************
 */

/* **************************************************************************
 *                                                                          *
 *    ssssssssss   tttttttttttt   aaaaaaaaaa   rrrrrrrrrrr   tttttttttttt   *
 *   ssssssssssss  tttttttttttt  aaaaaaaaaaaa  rrrrrrrrrrrr  tttttttttttt   *
 *   ss        ss       tt       aa        aa  rr        rr       tt        *
 *   ss                 tt       aa        aa  rr        rr       tt        *
 *   sss                tt       aa        aa  rr        rr       tt        *
 *    sssssssss         tt       aaaaaaaaaaaa  rrrrrrrrrrrr       tt        *
 *     sssssssss        tt       aaaaaaaaaaaa  rrrrrrrrrrr        tt        *
 *            sss       tt       aa        aa  rr    rr           tt        *
 *             ss       tt       aa        aa  rr     rr          tt        *
 *   ss        ss       tt       aa        aa  rr      rr         tt        *
 *   ssssssssssss       tt       aa        aa  rr       rr        tt        *
 *    ssssssssss        tt       aa        aa  rr        rr       tt        *
 *                                                                          *
 ****************************************************************************
 */

int ptlbuf(char* name) 
{
  int    status=0   ;
  char   buffer[256];
  char   tmpbuf[78] ;
  float  rndm       ; 
  int    irnd       ; 
  char  *HB         ;
  char  *HBVERS     ; 
  Commonptlbuf   & PTLBuffer=ptbuff_;

  if( debug )
    {
      /* start message */
      c78put("=== PTLBUF start ===");
      me78ap_();
    }
  
  /* start the initialisation procedure */
  if(first)
    {
      /* try to open the file in the current working directory */ 
      strcpy( filename , name      );
      file = fopen( filename , "r" );
      if( !file ) {
	/* try to open the file in the standart HERA-B place */ 
	HB     = getenv( "HB" );
	if( HB==NULL )
	  {
	    c78put("PTLBUF==> variable HB is not defined");
	    /* Provide the unique warning counters */
	    irnd = 1000 + int ( 1000.0 * rndm_( rndm ) );                  
	    me78wc_( (gcbank_.iq[irnd]) );  
	  }
	HBVERS = getenv( "HBVERS");
	if( HBVERS==NULL )
	  {
	    c78put("PTLBUF==> variable HBVERS is not defined");
	    /* Provide the unique warning counters */
	    irnd = 1000 + int ( 1000.0 * rndm_( rndm ) );                  
	    me78wc_( (gcbank_.iq[irnd]));  
	  }
	/* construct name of file to be opened */
	strcpy( buffer , HB         );
	strcat( buffer , "/"        );
	strcat( buffer , HBVERS     );
	strcat( buffer , "/geo/ptl/");                               
	strcat( buffer , filename   );
	strcpy( filename  , buffer );
	file = fopen( filename , "r" ); 
	if(!file)
	  {
	    strcpy( tmpbuf , "PTLBUF=>Can't open file:"); 
	    strcat( tmpbuf , filename );
	    c78put( tmpbuf );
	    /* Provide the unique warning counters */
	    irnd = 1000 + int ( 1000.0 * rndm_( rndm ) );                  
	    me78wc_( (gcbank_.iq[irnd]) );  
	    first  = 1       ; 
	    status = -2      ;
	    return status;}
	else 
	  {
	    strcpy( tmpbuf , "=== OPEN  PTFILE:"); 
	    strcat( tmpbuf , filename);
	    c78put( tmpbuf );                 
	    me78ap_();  
	  }
      }
      else
	{
	  strcpy( tmpbuf , "=== OPEN PTFILE:"); 
	  strcat( tmpbuf , filename );
	  c78put( tmpbuf );
	  me78ap_();  
	}
      fseek(file,0L,0);
      first=0;
    }
  if(fgets(buffer,256,file) == NULL)
    {
      status = 0;
      if( feof( file ) ) 
	{
	  status = -1;
	  fclose ( file );
	  first = 1 ; 
	  strcpy( tmpbuf , "=== CLOSE PTFILE:"); 
	  strcat( tmpbuf , filename );
          c78put( tmpbuf );
	  me78ap_();  
	  return status; }
      return status;
    }
  {
    /* set buffer lengtht */ 
    int i = 0 ;
    while(buffer[i]!='\n')i++;
    PTLBuffer.buflen=i;
  }
  
  /* put buffer into FORTRAN common block */ 
  strcpy(PTLBuffer.buffer,buffer);
  
  if( debug )
    {
      /* bye message */
      c78put("=== PTLBUF Normal end  ===");
      me78ap_();
    }
  
  return 1;
}
/* **************************************************************************
 *                    end of routine(function) ptlbuf                       * 
 ****************************************************************************
 *                                                                          *
 *            EEEEEEEEEEEE  NN        NN  DDDDDDDDD                         *
 *            EEEEEEEEEEEE  NNN       NN  DDDDDDDDDD                        *
 *            EE            NNNN      NN  DD       DD                       *
 *            EE            NN NN     NN  DD        DD                      *
 *            EE            NN  NN    NN  DD        DD                      *
 *            EEEEEEEE      NN   NN   NN  DD        DD                      *
 *            EEEEEEEE      NN    NN  NN  DD        DD                      *
 *            EE            NN     NN NN  DD        DD                      *
 *            EE            NN      NNNN  DD        DD                      *
 *            EE            NN       NNN  DD       DD                       *  
 *            EEEEEEEEEEEE  NN        NN  DDDDDDDDDD                        *   
 *            EEEEEEEEEEEE  NN         N  DDDDDDDDD                         *
 *                                                                          *
 ****************************************************************************
 *                    end of routine(function) ptlbuf                       * 
 ****************************************************************************
 */
