************************************************************************
*
* SILG96
* written by Siegmund Nowak 12/92
*
* This routines defines silicon detector geometry  for the HERA-B detector
*
*       called from GEOMAIN
*
* Arguments  kind         description
*
*-----------------------------------------------------------------------
*   ISL      in           superlayer number in GESL bank
************************************************************************

      SUBROUTINE  SILG96(ISL)
C--------------------------------

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGBPIP.fi"
#include "gean/ZGSILD.fi"
#include "gean/ZGMAGN.fi"
#include "auto/GEO.fi"
      INTEGER ISL,NSL,IFIRST,IVOL(6),ISEC,MEDNUM,IMOD,IFRONT,NN,NMM
      REAL XPL, YPL, PXPLAN, PYPLAN, PZPLAN, ANGLE,ANG,COSX,SINX
      REAL PARBG(3),PARSI(3),PARCA(3),PARHY(3),PARAD(3),PARBS(3)
      REAL POSBG(3),POSSI(3),POSCA(3),POSHY(3),POSAD(3),POSBS(3)
      REAL ABS,COSD,SIND,ABSCD,ABSSD,XMINEF,XMAXEF,YMINEF,YMAXEF,XMS(6)
      CHARACTER*4 CSL
      SAVE IFIRST
      DATA IFIRST /0/
#include "arte/Arte.ff"
#include "auto/GEO.ff"

      IF(IFIRST.EQ.0) THEN
       CALL SILDAT('sil97.dat')
       CALL GSVOLU('BAGF','BOX ',MEDNUM(1,'ALUMINIUM' ),PARSI,0,IVOL(2))
       CALL GSVOLU('SILZ','BOX ',MEDNUM(3,'SILICON' ),PARSI,0,IVOL(3))
       CALL GSVOLU('SICA','BOX ',MEDNUM(1,'ALUMINIUM' ),PARSI,0,IVOL(4))
       CALL GSVOLU('SIAD','BOX ',MEDNUM(1,'AL2O3'),PARSI,0,IVOL(5))
       CALL GSVOLU('SIHY','BOX ',MEDNUM(1,'AL2O3'),PARSI,0,IVOL(6))

       CALL GRADLN(IVOL(2),XMS(2))
       XMS(2) = DZBGSI /XMS(2)
       XMS(2) = XMS(2)*(.0136*(1.+.038*ALOG(XMS(2))))**2
       CALL GRADLN(IVOL(3),XMS(3))
       XMS(3) = DZSISI /XMS(3)
       XMS(3) = XMS(3)*(.0136*(1.+.038*ALOG(XMS(3))))**2
       CALL GRADLN(IVOL(4),XMS(4))
       XMS(4) = .5*(DZC1SI+DZC2SI)/XMS(4)
       XMS(4) = XMS(4)*(.0136*(1.+.038*ALOG(XMS(4))))**2
       CALL GRADLN(IVOL(5),XMS(5))
       XMS(5) = .5*(DZA1SI+DZA2SI)/XMS(5)
       XMS(5) = XMS(5)*(.0136*(1.+.038*ALOG(XMS(5))))**2
       CALL GRADLN(IVOL(6),XMS(6))
       XMS(6) = DZHYSI/XMS(6)
       XMS(6) = XMS(2)*(.0136*(1.+.038*ALOG(XMS(6))))**2
       IFIRST = 1
       ANG = ANGSI/180.*PI
       COSX = COS(ANG)
       SINX = SIN(ANG)
      ENDIF

      CURRSL = ISL
      IF(IGESL_LAY(ISL).EQ.0) GO TO 90 

c      install superlayer
       CALL UHTOC(HGESL_NAME(ISL),4,CSL,4)
       CALL CGGESL(CSL,'MOTH',0)
       CALL GSATT(CSL,'COLO',-5)

       NSL = IGESL_NSL(ISL)
       XPL = XRSI(NSL) + OVSI(NSL) - .5*DXSISI
       YPL = XRSI(NSL) - (PYSISI(NSL) - .5*DYSISI)

       IMOD = IGESL_LTYP(ISL)
 
       ISEC = 2
        IF(ISEC.EQ.1) THEN
           PZPLAN = -PZC1SI(NSL)
           ANGLE = 0.
        ELSEIF(ISEC.EQ.2) THEN
           PZPLAN = PZC1SI(NSL)
           ANGLE = -PIBY2
        ENDIF

        COSD = COS(ANGLE)
        SIND = SIN(ANGLE)
        ABSCD = ABS(COSD)
        ABSSD = ABS(SIND)

        PXPLAN =  XPL*COSD + YPL*SIND
        PYPLAN = -XPL*SIND + YPL*COSD

C--   front alu bag dimensions and x/y positions
        PARBG(1) = .5 * (DXBGSI*ABSCD + DYBGSI*ABSSD)
        PARBG(2) = .5 * (DXBGSI*ABSSD + DYBGSI*ABSCD)
        PARBG(3) = .5 * DZBGSI
        POSBG(1) = PYBGSI(NSL)*SIND + PXPLAN
        POSBG(2) = PYBGSI(NSL)*COSD + PYPLAN
        POSBG(3) = PZPLAN - PZBGSI(NSL)
        CALL GSPOSP('BAGF',NGWAL()+1,CSL,
     +               POSBG(1),POSBG(2),POSBG(3),0,'ONLY',PARBG,3)
        CALL CRGWAL('BAGF',NGWAL()+1)

C--   silicon
        PARSI(1) = .5 * (DXSISI*ABSCD+DYSISI*ABSSD)
        PARSI(2) = .5 * (DXSISI*ABSSD+DYSISI*ABSCD)
        PARSI(3) = .5 * DZSISI
        POSSI(1) = PYSISI(NSL)*SIND + PXPLAN
        POSSI(2) = PYSISI(NSL)*COSD + PYPLAN
        POSSI(3) = PZPLAN - .5*(DZC1SI+DZSISI)

        XMINEF = POSSI(1) - PARSI(1) + RANDSI
        XMAXEF = POSSI(1) + PARSI(1) - RANDSI
        YMINEF = POSSI(2) - PARSI(2) + RANDSI
        YMAXEF = POSSI(2) + PARSI(2) - RANDSI

        CALL CRPOS('SILZ',CSL,1,ISEC,
     +                     POSSI(1),POSSI(2),POSSI(3),0,'ONLY',PARSI)

        Q (QGEDE_SXMN_L()) = XMINEF
        Q (QGEDE_SXMX_L()) = XMAXEF
        Q (QGEDE_SYMN_L()) = YMINEF
        Q (QGEDE_SYMX_L()) = YMAXEF
        PITSI = 0.005461 
        IF(IMOD.EQ.1) THEN
           Q (QGEDE_COS_L()) = sinx
           Q (QGEDE_SIN_L()) = cosx
           Q (QGEDE_XFW_L(1)) = XMINEF*COSX+YMINEF*SINX + 0.5*PITSI
          IQ(IQGEDE_NWI_L(1)) = 
     +       ((XMAXEF-XMINEF)*COSX + (YMAXEF-YMINEF)*SINX)/PITSI
        ELSEIF(IMOD.EQ.2) THEN
           Q (QGEDE_COS_L()) = sinx
           Q (QGEDE_SIN_L()) = -cosx
           Q (QGEDE_XFW_L(1)) = -XMAXEF*SINX-YMAXEF*COSX + 0.5*PITSI
          IQ(IQGEDE_NWI_L(1)) = 
     +       ((XMAXEF-XMINEF)*SINX + (YMAXEF-YMINEF)*COSX)/PITSI
        ENDIF
        Q (QGEDE_ZFW_L(1)) = FGEDE_ZMIN(NGEDE())
        Q (QGEDE_ACC_L()) = SIGSI
        Q (QGEDE_EFF_L()) = EFFSI
        Q (QGEDE_PIT_L()) = PITSI
        Q (QGEDE_DOD_L()) = DMSI
        IQ(IQGEDE_NSL_L())  = 2
        Q (QGEDE_MS_L())= XMS(3)*(.0136*(1.+.038*ALOG(XMS(3))))**2

c       take backside as a separate GEDE bank

        IFRONT = NGEDE()
        CALL MMBOOK(JGEDE,1)

        IQ(IQGEDE_CMP_L()) = IGEDE_CMP(IFRONT)
        IQ(IQGEDE_LAY_L()) = 2
        CALL UCOPY(IQ(IQGEDE_SEC(IFRONT)),IQ(IQGEDE_sec_L()),2)
        CALL UCOPY(Q(QGEDE_XMIN(IFRONT)),Q(QGEDE_XMIN_L()),10)
        CALL UCOPY(Q(QGEDE_ROT(1,IFRONT)),Q(QGEDE_ROT_L(1)),9)
        CALL UCOPY(Q(QGEDE_ACC(IFRONT)),Q(QGEDE_ACC_L()),4)
        PITSI = 0.00517
        IF(IMOD.EQ.1) THEN
           Q (QGEDE_COS_L()) = -cosx
           Q (QGEDE_SIN_L()) = sinx
           Q (QGEDE_XFW_L(1)) = XMINEF*SINX-YMAXEF*COSX + 0.5*PITSI
          IQ(IQGEDE_NWI_L(1)) = 
     +       ((XMAXEF-XMINEF)*SINX + (YMAXEF-YMINEF)*COSX)/PITSI
        ELSEIF(IMOD.EQ.2) THEN
           Q (QGEDE_COS_L()) = -cosx
           Q (QGEDE_SIN_L()) = -sinx
           Q (QGEDE_XFW_L(1)) = XMINEF*COSX-YMAXEF*SINX + 0.5*PITSI
          IQ(IQGEDE_NWI_L(1)) = 
     +       ((XMAXEF-XMINEF)*COSX + (YMAXEF-YMINEF)*SINX)/PITSI
        ENDIF
        Q (QGEDE_ZFW_L(1))  = FGEDE_ZMAX(IFRONT)
        Q (QGEDE_PIT_L()) = PITSI
        IQ(IQGEDE_NSL_L())  = 0
        NN = NGEDE() 
        CALL MMBREL(JGSGD,CURRSL,NN)
        IQ(IQGESL_NPL(CURRSL)) = IQ(IQGESL_NPL(CURRSL)) + 1
        IQ(HQGEDE_GVOL_L()) = IQ(HQGEDE_GVOL(IFRONT))
        NMM = OGEDE_GMAT(IFRONT)
        CALL MMBREL(JGMGD,NMM,NN)
        Q (QGEDE_MS_L()) =  Q (QGEDE_MS(IFRONT))

C--   adapter -front side
        PARAD(1) = .5 * (DXA1SI*ABSCD+DYA1SI*ABSSD)
        PARAD(2) = .5 * (DXA1SI*ABSSD+DYA1SI*ABSCD)
        PARAD(3) = .5 * DZA1SI
        POSAD(1) = PYA1SI(NSL)*SIND + PXPLAN
        POSAD(2) = PYA1SI(NSL)*COSD + PYPLAN
        POSAD(3) = PZPLAN - .5*(DZC1SI+DZA1SI)
        CALL GSPOSP('SIAD',NGWAL()+1,CSL,
     +            POSAD(1),POSAD(2),POSAD(3),0,'ONLY',PARAD,3)
        CALL CRGWAL('SIAD',NGWAL()+1)

        PARHY(1) = .5 * (DXHYSI*ABSCD+DYHYSI*ABSSD)
        PARHY(2) = .5 * (DXHYSI*ABSSD+DYHYSI*ABSCD)
        PARHY(3) = .5 * DZHYSI
        POSHY(1) = -PXH1SI(NSL)*COSD + PYH1SI(NSL)*SIND + PXPLAN
        POSHY(2) = -PXH1SI(NSL)*SIND + PYH1SI(NSL)*COSD + PYPLAN
        POSHY(3) = PZPLAN - .5*(DZC1SI+DZHYSI)
        CALL GSPOSP('SIHY',NGWAL()+1,CSL,
     +            POSHY(1),POSHY(2),POSHY(3),0,'ONLY',PARHY,3)
        CALL CRGWAL('SIHY',NGWAL()+1)
C--
        POSHY(1) = PXH1SI(NSL)*COSD + PYH1SI(NSL)*SIND + PXPLAN
        POSHY(2) = PXH1SI(NSL)*SIND + PYH1SI(NSL)*COSD + PYPLAN
        CALL GSPOSP('SIHY',NGWAL()+1,CSL,
     +            POSHY(1),POSHY(2),POSHY(3),0,'ONLY',PARHY,3)
        CALL CRGWAL('SIHY',NGWAL()+1)

C--    Carrier (beam-side)
        PARCA(1) = .5 * (DXC1SI*ABSCD+DYC1SI*ABSSD)
        PARCA(2) = .5 * (DXC1SI*ABSSD+DYC1SI*ABSCD)
        PARCA(3) = .5 * DZC1SI
        POSCA(1) = PYC1SI(NSL)*SIND + PXPLAN
        POSCA(2) = PYC1SI(NSL)*COSD + PYPLAN
        CALL GSPOSP('SICA',NGWAL()+1,CSL,
     +             POSCA(1),POSCA(2),PZPLAN,0,'ONLY', PARCA,3)
        CALL CRGWAL('SICA',NGWAL()+1)

C--    Carrier (beam off-side)
        PARCA(1) = .5 * (DXC2SI*ABSCD+DYC2SI*ABSSD)
        PARCA(2) = .5 * (DXC2SI*ABSSD+DYC2SI*ABSCD)
        PARCA(3) = .5 * DZC2SI
        POSCA(1) = PYC2SI(NSL)*SIND + PXPLAN
        POSCA(2) = PYC2SI(NSL)*COSD + PYPLAN
        CALL GSPOSP('SICA',NGWAL()+1  ,CSL,
     +             POSCA(1),POSCA(2),PZPLAN,0,'ONLY', PARCA,3)
        CALL CRGWAL('SICA',NGWAL()+1)

C--    Adapter (back side)
        PARAD(1) = .5 * (DXA2SI*ABSCD+DYA2SI*ABSSD)
        PARAD(2) = .5 * (DXA2SI*ABSSD+DYA2SI*ABSCD)
        PARAD(3) = .5 * DZA2SI
        POSAD(1) = PYA2SI(NSL)*SIND + PXPLAN
        POSAD(2) = PYA2SI(NSL)*COSD + PYPLAN
        POSAD(3) = PZPLAN + .5*(DZC1SI+DZA1SI)
        CALL GSPOSP('SIAD',NGWAL()+1,CSL,
     +            POSAD(1),POSAD(2),POSAD(3),0,'ONLY',PARAD,3)
        CALL CRGWAL('SIAD',NGWAL()+1)

C--
        POSHY(1) = -PXH2SI(NSL)*COSD + PYH2SI(NSL)*SIND + PXPLAN
        POSHY(2) = -PXH2SI(NSL)*SIND + PYH2SI(NSL)*COSD + PYPLAN
        POSHY(3) = PZPLAN + .5*(DZC1SI+DZHYSI)
        CALL GSPOSP('SIHY',NGWAL()+1,CSL,
     +            POSHY(1),POSHY(2),POSHY(3),0,'ONLY',PARHY,3)
        CALL CRGWAL('SIHY',NGWAL()+1)
C--
        POSHY(1) = PXH2SI(NSL)*COSD + PYH2SI(NSL)*SIND + PXPLAN
        POSHY(2) = PXH2SI(NSL)*SIND + PYH2SI(NSL)*COSD + PYPLAN
        CALL GSPOSP('SIHY',NGWAL()+1  ,CSL,
     +            POSHY(1),POSHY(2),POSHY(3),0,'ONLY',PARHY,3)
        CALL CRGWAL('SIHY',NGWAL()+1)

C--   back alu bag
        PARBG(1) = .5 * (DXBGSI*ABSCD+DYBGSI*ABSSD)
        PARBG(2) = .5 * (DXBGSI*ABSSD+DYBGSI*ABSCD)
        PARBG(3) = .5 * DZBGSI
        POSBG(1) = PYBGSI(NSL)*SIND + PXPLAN
        POSBG(2) = PYBGSI(NSL)*COSD + PYPLAN
        POSBG(3) = PZPLAN + PZBGSI(NSL)
        CALL GSPOSP('BAGF',NGWAL()+1,CSL,
     +               POSBG(1),POSBG(2),POSBG(3),0,'ONLY',PARBG,3)
        CALL CRGWAL('BAGF',NGWAL()+1)

C--   bag side walls
        PARBS(1) = .5*DZBGSI*ABSCD + DYBGSI*ABSSD
        PARBS(2) = .5*DZBGSI*ABSSD + DYBGSI*ABSCD
        PARBS(3) =  PZBGSI(NSL) - 0.5*DZBGSI
        POSBS(1) = ABSCD*(POSBG(1)-PARBG(1)) + ABSSD*POSBG(1)
        POSBS(2) = ABSSD*(POSBG(2)-PARBG(2)) + ABSCD*POSBG(2)
        CALL GSPOSP('BAGF',NGWAL()+1,CSL,
     +             POSBS(1),POSBS(2),PZPLAN,0,'ONLY',PARBS,3)
        CALL CRGWAL('BAGF',NGWAL()+1)
        POSBS(1) = ABSCD*(POSBG(1)+PARBG(1)) + ABSSD*POSBG(1)
        POSBS(2) = ABSSD*(POSBG(2)+PARBG(2)) + ABSCD*POSBG(2)
        CALL GSPOSP('BAGF',NGWAL()+1,CSL,
     +             POSBS(1),POSBS(2),PZPLAN,0,'ONLY',PARBS,3)
        CALL CRGWAL('BAGF',NGWAL()+1)
        PARBS(1) = ABSCD*(.5*DXBGSI-DZBGSI) + ABSSD*DZBGSI
        PARBS(1) = ABSSD*(.5*DXBGSI-DZBGSI) + ABSCD*DZBGSI
        PARBS(3) =  PZBGSI(NSL) - 0.5*DZBGSI
        POSBS(1) = ABSCD*(POSBG(1)-.5*COSD*(DYBGSI-DZBGSI))
     +      +    ABSSD*(POSBG(1)-.5*SIND*(DYBGSI-DZBGSI))
        POSBS(2) = ABSSD*(POSBG(2)-.5*SIND*(DYBGSI-DZBGSI))
     +      +    ABSCD*(POSBG(2)-.5*COSD*(DYBGSI-DZBGSI))
        CALL GSPOSP('BAGF',NGWAL()+1,CSL,
     +             POSBS(1),POSBS(2),PZPLAN,0,'ONLY',PARBS,3)
        CALL CRGWAL('BAGF',NGWAL()+1)

 90   RETURN
      END

