
      SUBROUTINE  SILGEO(ISL)

************************************************************************
*
* SILGEO
* written by Siegmund Nowak 12/92
*
* This routines defines silicon detector geometry  for the HERA-B detector
*
*       called from GEOMAIN
*
* Arguments  kind         description
*
*-----------------------------------------------------------------------
*   IS1      in           first silicon superlayer
*   IS2      in           last silicon superlayer
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGBPIP.fi"
#include "gean/ZGSILD.fi"
#include "gean/ZGMAGN.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"

      INTEGER NSL,ISL,IFIRST,IVOL,ISEC,IRS,NPL,NSEC,MEDNUM,IROTM(4)
      INTEGER IUCOMP,ILAY,MG,NN
      REAL XPL, YPL, PXMOD, PYMOD, PZMOD, PZPLAN, XP(3)
      REAL PARBG(3), PARSI(3), PARCA(3), PARAD(3), PARHY(3), PARBS(3)
      REAL POSBG(3), POSSI(3), POSCA(3), POSAD(3), POSHY(3), POSBS(3)
      REAL ANG,COSX,SINX, XMINEF,XMAXEF, YMINEF,YMAXEF
      CHARACTER*4 CNAM
      DATA IFIRST /0/
      SAVE IFIRST

#include "arte/Arte.ff"
#include "auto/GEO.ff"

C--   initialisation (volume definition,rad-length,angles etc)

      IF(IFIRST.EQ.0) THEN
         IROTM(1) = 0
         CALL CRROTM(IROTM(2), 90., 270.,  90.,   0.,  0.,  0.)
         CALL CRROTM(IROTM(3), 90., 180.,  90., 270.,  0.,  0.)
         CALL CRROTM(IROTM(4), 90.,  90.,  90., 180.,  0.,  0.)
         CALL SILDAT('sil.dat')
         CALL GSVOLU('BAGF','BOX ',MEDNUM(1,'ALUMINIUM'),PARBG,0,IVOL)
         CALL GSVOLU('SILZ','BOX ',MEDNUM(3,'SILICON'),PARSI,0,IVOL)
         CALL GSVOLU('SICA','BOX ',MEDNUM(1,'CARBONFIBRE'),PARSI,0,IVOL)
         CALL GSVOLU('SIAD','BOX ',MEDNUM(1,'AL2O3'),PARSI,0,IVOL)
         CALL GSVOLU('SIHY','BOX ',MEDNUM(1,'AL2O3'),PARSI,0,IVOL)

         ANG = ANGSI/180.*PI
         COSX = COS(ANG)
         SINX = SIN(ANG)
         IFIRST = 1
      ENDIF

      CURRSL = ISL 
      IF(IGESL_LAY(ISL).EQ.0) GO TO 90
 
      CALL UHTOC(HGESL_NAME(ISL),4,CNAM,4)

      NSL = IUCOMP(CNAM,DESI,NSLSI)
      IF(NSL.EQ.0) CALL MEERRO('SILGEO: '//CNAM//' not found in DESI')
      NPL  = IGESL_LAY(ISL)
      NSEC = 4
      CALL CGGESL(CNAM,'RPOT',0)
      CALL GSATT(CNAM,'COLO',-5)

C--   loop over the four sectors
      XPL = XRSI(NSL) + OVSI(NSL) - .5*DXSISI    
      YPL = XRSI(NSL)

      DO ISEC = 1,4
      IRS = IROTM(ISEC)  

       IF(ISEC.EQ.1) THEN
          PZPLAN = -PZLYSI(NSL)
       ELSEIF(ISEC.EQ.2) THEN
          PZPLAN = PZLYSI(NSL)
       ELSEIF(ISEC.EQ.3) THEN
          PZPLAN = -PZLYSI(NSL)
       ELSEIF(ISEC.EQ.4) THEN
          PZPLAN = PZLYSI(NSL)
       ENDIF

       PXMOD = -XPL 
       PYMOD =  YPL

C--    DIMENSIONS AND x/y POSITIONS

C--    alu bag: 

       DYBGSI = XOSI(NSL) - XBAG(NSL)
       PYBGSI(NSL) = XBAG(NSL) + .5*DYBGSI - XRSI(NSL)
       PARBG(1) = .5 * DXBGSI
       PARBG(2) = .5 * DYBGSI
       PARBG(3) = .5 * DZBGSI
       POSBG(1) = PXMOD
       POSBG(2) = PYBGSI(NSL) + PYMOD
       POSBG(3) = PZPLAN - PZC1SI(NSL) - PZBGSI(NSL)
       CALL GUROT(IRS,POSBG,XP)
       CALL CRPOS('BAGF',CNAM,1,ISEC,XP(1),XP(2),XP(3),IRS,'ONLY',PARBG)

       DO ILAY =1,3,2

       IF(ILAY.EQ.1) PZMOD = PZPLAN - PZC1SI(NSL)
       IF(ILAY.EQ.3) PZMOD = PZPLAN + PZC1SI(NSL) 


C--    silicon:
       PYSISI(NSL) = .5*DYSISI
       PARSI(1) = .5 * DXSISI
       PARSI(2) = .5 * DYSISI
       PARSI(3) = .5 * DZSISI
       POSSI(1) = PXMOD
       POSSI(2) = PYSISI(NSL) + PYMOD
       POSSI(3) = PZMOD -.5*(DZC1SI+DZSISI)
       CALL GUROT(IRS,POSSI,XP)       
       CALL CRPOS('SILZ',CNAM,ILAY,ISEC,
     +                            XP(1),XP(2),XP(3),IRS,'ONLY',PARSI)

       XMINEF = FGEDE_XMIN(NGEDE()) + RANDSI
       XMAXEF = FGEDE_XMAX(NGEDE()) - RANDSI
       YMINEF = FGEDE_YMIN(NGEDE()) + RANDSI
       YMAXEF = FGEDE_YMAX(NGEDE()) - RANDSI
       Q (QGEDE_SXMN_L()) = XMINEF
       Q (QGEDE_SXMX_L()) = XMAXEF
       Q (QGEDE_SYMN_L()) = YMINEF
       Q (QGEDE_SYMX_L()) = YMAXEF
       Q (QGEDE_ACC_L()) = SIGSI
       Q (QGEDE_EFF_L()) = EFFSI
       Q (QGEDE_PIT_L()) = PITSI
       Q (QGEDE_DOD_L()) = DMSI
       IQ(IQGEDE_NSL_L())  = 1

       IF(ILAY .EQ. 1) THEN
          Q (QGEDE_COS_L()) = COSX
          Q (QGEDE_SIN_L()) = -SINX
       ELSE IF(ILAY.eq.3) THEN
          Q (QGEDE_COS_L()) = COSX
          Q (QGEDE_SIN_L()) = SINX
       ENDIF

       Q (QGEDE_XFW_L(1)) = XMINEF*FGEDE_COS(NGEDE()) + 
     +                      YMINEF*FGEDE_SIN(NGEDE()) + 0.5*PITSI
       Q (QGEDE_ZFW_L(1)) = FGEDE_ZMIN(NGEDE())
       IQ(IQGEDE_NWI_L(1)) = ((XMAXEF-XMINEF)*FGEDE_COS(NGEDE()) + 
     +                        (YMAXEF-YMINEF)*FGEDE_SIN(NGEDE())) /PITSI

       CALL MMBOOK(JGEDE,1)
       NN = NGEDE()
       CALL UCOPY(Q(JGEDE+(NN-2)*NCGEDE()+3),
     +            Q(JGEDE+(NN-1)*NCGEDE()+3),NCGEDE())
       IQ(JGEDE+(NN-1)*NCGEDE()+2 + JGEDE_GESL) = 0
       CALL MMBREL(JGSGD,CURRSL,NN)
       MG = OGEDE_GMAT(NN-1) 
       IQ(JGEDE+(NN-1)*NCGEDE()+2 + JGEDE_GMAT) = 0
       CALL MMBREL(JGMGD,MG,NN)
       IQ(IQGESL_NPL(CURRSL)) = IQ(IQGESL_NPL(CURRSL)) + 1

       IF(ILAY .EQ. 1) THEN
          Q (QGEDE_COS_L()) = -SINX
          Q (QGEDE_SIN_L()) = -COSX
       ELSE IF(ILAY.eq.3) THEN
          Q (QGEDE_COS_L()) = SINX
          Q (QGEDE_SIN_L()) = -COSX
       ENDIF
       IQ(IQGEDE_LAY_L()) = ILAY + 1 

       Q (QGEDE_XFW_L(1)) = -XMINEF*FGEDE_COS(NGEDE()) - 
     +                      YMINEF*FGEDE_SIN(NGEDE()) + 0.5*PITSI
       Q (QGEDE_ZFW_L(1)) = FGEDE_ZMAX(NN)
       IQ(IQGEDE_NWI_L(1)) = -((XMAXEF-XMINEF)*FGEDE_COS(NGEDE()) + 
     +                        (YMAXEF-YMINEF)*FGEDE_SIN(NGEDE())) /PITSI
 
C--   adapter -front side
       PARAD(1) = .5 * DXA1SI
       PARAD(2) = .5 * DYA1SI
       PARAD(3) = .5 * DZA1SI
       POSAD(1) = PXMOD
       POSAD(2) = PYA1SI(NSL) + PYMOD
       POSAD(3) = PZMOD - .5*(DZC1SI+DZA1SI)
       CALL GUROT(IRS,POSAD,XP)              
       CALL CRPOS('SIAD',CNAM,ILAY,ISEC,
     +                   XP(1),XP(2),XP(3),IRS,'ONLY',PARAD)

c--   hybrid - front side  
       PARHY(1) = .5 * DXHYSI
       PARHY(2) = .5 * DYHYSI
       PARHY(3) = .5 * DZHYSI
       POSHY(1) = -PXH1SI(NSL) + PXMOD
       POSHY(2) =  PYH1SI(NSL) + PYMOD
       POSHY(3) = PZMOD - .5*(DZC1SI+DZHYSI)
       CALL GUROT(IRS,POSHY,XP)
       CALL CRPOS('SIHY',CNAM,ILAY,ISEC,
     +                         XP(1),XP(2),XP(3),IRS,'ONLY',PARHY)

       POSHY(1) = PXH1SI(NSL) + PXMOD
       POSHY(2) = PYH1SI(NSL) + PYMOD
       CALL GUROT(IRS,POSHY,XP)
       CALL CRPOS('SIHY',CNAM,ILAY,ISEC,
     +                         XP(1),XP(2),XP(3),IRS,'ONLY',PARHY)

C--    Carrier (beam-side)
       PARCA(1) = .5 * DXC1SI
       PARCA(2) = .5 * DYC1SI
       PARCA(3) = .5 * DZC1SI
       POSCA(1) = PXMOD
       POSCA(2) = PYC1SI(NSL) + PYMOD
       POSCA(3) = PZMOD
       CALL GUROT(IRS,POSCA,XP)
       CALL CRPOS('SICA',CNAM,ILAY,ISEC,
     +                         XP(1),XP(2),XP(3),IRS,'ONLY',PARCA)

C--    Carrier (beam off-side)
       PARCA(1) = .5 * DXC2SI
       PARCA(2) = .5 * DYC2SI
       PARCA(3) = .5 * DZC2SI
       POSCA(1) = PXMOD
       POSCA(2) = PYC2SI(NSL) + PYMOD
       POSCA(3) = PZMOD
       CALL GUROT(IRS,POSCA,XP)
       CALL CRPOS('SICA',CNAM,ILAY,ISEC,
     +                        XP(1),XP(2),XP(3),IRS,'ONLY',PARCA)

C--    Adapter (back side)
       PARAD(1) = .5 * DXA2SI
       PARAD(2) = .5 * DYA2SI
       PARAD(3) = .5 * DZA2SI
       POSAD(1) = PXMOD
       POSAD(2) = PYA2SI(NSL) + PYMOD
       POSAD(3) = PZMOD + .5*(DZC1SI+DZA1SI)
       CALL GUROT(IRS,POSAD,XP)
       CALL CRPOS('SIAD',CNAM,ILAY,ISEC,
     +                        XP(1),XP(2),XP(3),IRS,'ONLY',PARAD)

C--     hybrid (back side)
       POSHY(1) = -PXH2SI(NSL) + PXMOD
       POSHY(2) =  PYH2SI(NSL) + PYMOD
       POSHY(3) = PZMOD + .5*(DZC1SI+DZHYSI)
       CALL GUROT(IRS,POSHY,XP)
       CALL CRPOS('SIHY',CNAM,ILAY,ISEC,
     +                        XP(1),XP(2),XP(3),IRS,'ONLY',PARHY)
       POSHY(1) = PXH2SI(NSL) + PXMOD
       POSHY(2) = PYH2SI(NSL) + PYMOD
       CALL GUROT(IRS,POSHY,XP)
       CALL CRPOS('SIHY',CNAM,ILAY,ISEC,
     +                        XP(1),XP(2),XP(3),IRS,'ONLY',PARHY)
       enddo             ! end loop over layers

C--   back alu bag

       POSBG(3) = PZPLAN + PZC1SI(NSL) + PZBGSI(NSL)
       CALL GUROT(IRS,POSBG,XP)
       CALL CRPOS('BAGF',CNAM,0,ISEC,XP(1),XP(2),XP(3),IRS,'ONLY',PARBG)

C--   bag side walls
       PARBS(1) = .5*DZBGSI
       PARBS(2) = .5*DYBGSI
       PARBS(3) = PZBGSI(NSL) - 0.5*DZBGSI
       POSBS(1) = POSBG(1)-PARBG(1)
       POSBS(2) = POSBG(2)
       POSBS(3) = PZPLAN
       CALL GUROT(IRS,POSBS,XP)
       CALL CRPOS('BAGF',CNAM,0,ISEC,XP(1),XP(2),XP(3),IRS,'ONLY',PARBS)
       POSBS(1) = POSBG(1)+PARBG(1)
       POSBS(2) = POSBG(2)
       CALL GUROT(IRS,POSBS,XP)
       CALL CRPOS('BAGF',CNAM,0,ISEC,XP(1),XP(2),XP(3),IRS,'ONLY',PARBS)

       PARBS(1) = .5*DXBGSI-DZBGSI
       PARBS(2) = .5*DZBGSI
       PARBS(3) =  PZBGSI(NSL) - 0.5*DZBGSI
       POSBS(1) = POSBG(1)-.5*(DYBGSI-DZBGSI)
       POSBS(2) = POSBG(2)-.5*(DYBGSI-DZBGSI)
       CALL GUROT(IRS,POSBS,XP)
       CALL CRPOS('BAGF',CNAM,0,ISEC,XP(1),XP(2),XP(3),IRS,'ONLY',PARBS)


      ENDDO               ! end loop over sectors ISEC

 90   RETURN
      END

