#include <stdio.h>
#include <time.h>
#include <math.h>
#include <stdlib.h>
#include <gpack.h>
#include <gpackproto.h>


typedef struct Row
{
  float zf;
  float xf;
  float yf;
  float pf;
  float cvf[15];
  int   hsi;
  int   hin;
  int   hou;
  int   hri;
  float chi;
} Row;

int main()
{
  EventHdr  h;
  EventType eventType = Mini;
  Row       *r;

  int       ntab;
  char      *dt;
  int       ncols, nrows;
  int       rc;
  char      tname[20], desc[512];
  int       count, irows;
  unsigned  *data = NULL;
  int       dataSize = 0;
  int       lun, length;
  /*  char* crit = "Run = 728 and (Class between #1 and #15)"; */
  /*char* crit = "Run = 728 and Class = #2 or Run = 727 and Class = #1";*/
  char* crit = "Run = 728 and Class = #2";

  printf("Start job inpub ...\n");

  /*
   * Get first free stream lun (Logical Unit Number? or File Descriptor?)
   */

  lun = gp_setstream( PublicStream );
  if( lun < 0 )
    {
      printf("Cannot get lun. rc = %d\n", lun);
      return 1;
    }

  /*
   * Set internal buffer size 
   */

  if( (rc = gp_setbuf(lun,8192)) < 0 )
    {
      printf("Cannot set buffer. rc = %d : %s\n", rc, gp_errmsg(lun));
      return 1;
    }

  gp_setevtype(lun, eventType);
  
  if( (rc = gp_select(lun,crit)) < 0 )
    {
      printf("Select failed. rc=%d : %s\n",rc,gp_errmsg(lun));
      return 1;
    }

  printf("Selection criteria: %s\n",crit);
  
  if( (rc = gp_open(lun)) < 0 )
    {
      printf("Cannot open Public Event Dir. rc=%d : %s\n", rc, gp_errmsg(lun));
      return 1;
    }
  
  count = 0;
  while( (rc = gp_getevt(lun, &h)) >= 0 )
    {
      if( count == 0 ) 
	{
	  gp_getevtype(lun, &eventType);
	  if( eventType == Dst )
	    printf("\nThis is DST  Event\n===================\n\n");
	  if( eventType == Mini )
	    printf("\nThis is Mini Event\n==================\n\n");
	}

      count++;
      dt = ctime( (time_t*)&h.datime );
      printf("Count %d\n Exp %d Run %d Event %d\n\t classmask 0x%08x datime %s\n",
	     count,h.experiment,h.run,h.event,h.classmask,dt);

      /* One event read, the next skip? */
      /* if( h.event % 2 == 1 ) continue; */
      for( ntab = 0;; ntab ++ )
	{
	  length = gp_gettabinfo(lun,tname,&ncols,&nrows);
	  if(length == GP_END_OF_DATA) break;  /* No more tables in event */
	  if(length < 0)
	    {
	      printf("count %d ntab %d Error %d : %s\n",
		     count,ntab,length,gp_errmsg(lun));
	      return 1;
	    }

          ncols = ncols + 5;
          length = ncols * sizeof(int) * nrows; 

	  if(length > dataSize)
	    {
	      printf("datalength > buffer\n");
	      if( data != NULL ) free( data );
	      data = (unsigned*)malloc( length );
	      if( !data ) 
		{
		  printf("Out of memory.\n");
		  return 1;
		}
 	      dataSize = length;
	    }

	  rc = gp_gettab(lun, tname, ncols, nrows, desc, data);
	  if( rc == GP_END_OF_DATA || rc == 0 ) break;
	  if( rc < 0 )
	    {
	      printf("Cannot get table. rc=%d : %s\n",rc,gp_errmsg(lun));
	      return 1;
	    }

	  printf("%d  Table name '%s' ncols=%d nrows=%d\n",
		 count,tname,ncols,nrows);

	  length = rc / nrows;
	  r = (Row*)data;
	  for(irows = 0; irows < nrows; irows++)
	    {
	      if( irows < 2 ) /* print only first two rows */
		printf("zf=%f, xf=%f, yf=%f, hsi=%d, hri=%d\n",
		       r->zf,r->xf,r->yf,r->hsi,r->hri);
	      r++;
	    }
	  printf("%d rows read\n",irows);
	  printf("-----------------------------------------------\n");
	}
      printf("%d tables read\n",ntab);
    }
  printf("%d events read\n",count);

  if( rc < 0 && rc != GP_END_OF_DATA )
    {
      printf("rc=%d : %s\n",rc,gp_errmsg(lun));
      return 1;
    }
  //
  // Close stream
  // 
  rc = gp_close(lun);
  if( rc < 0 ) 
    {
      printf("Close error %d : %s\n",rc,gp_errmsg(lun));
      return 1;
    }
  printf("Job 'inpub' is finished successful!\n");
  return 0;
}
