//**********************************************************************/
// Created: 23.09.97,   Olya Igonkina   for   MARPLE package
//
// implementation of RcLimit class
//
//**********************************************************************/
#include "marple/marple.hh"
#include <iostream.h>
#include <iomanip.h>
#include <math.h>

 void RcLimit::set( const RcPoint & rp, int icheck )
{
  if( icheck < 0 ){ icheck = flag; }
  if( icheck >= 100000 )
    {
      if( rp.z()  < z_min  ) { z_min  = rp.z();  }
      if( rp.z()  > z_max  ) { z_max  = rp.z();  }
      icheck -= (int)floor(double(icheck+1)/100000)*100000; 
    }
  if( icheck >= 10000 )
    {
      if( rp.x()  < x_min  ) { x_min  = rp.x();  }
      if( rp.x()  > x_max  ) { x_max  = rp.x();  }
      icheck -=(int)floor(double(icheck+1)/10000)*10000;
    }
  if( icheck >= 1000  )
    {
      if( rp.y()  < y_min  ) { y_min  = rp.y();  }
      if( rp.y()  > y_max  ) { y_max  = rp.y();  }
      icheck -=(int)floor(double(icheck+1)/1000)*1000;
    }
  if( icheck >= 100   )
    {
      if( rp.tx() < tx_min ) { tx_min = rp.tx(); }
      if( rp.tx() > tx_max ) { tx_max = rp.tx(); }
      icheck -=(int)floor(double(icheck+1)/100)*100;
    }
  if( icheck >= 10 )
    {
      if( rp.ty() < ty_min ) { ty_min = rp.ty(); }
      if( rp.ty() > ty_max ) { ty_max = rp.ty(); }
      icheck -=(int)floor(double(icheck+1)/10)*10;
    }
  if( icheck >= 1)
    {
      if( rp.pe() < pe_min ) { pe_min = rp.pe(); }
      if( rp.pe() > pe_max ) { pe_max = rp.pe(); }
    }
}


 int RcLimit::valid( const RcPoint & rp, int icheck ) const
{
  if( icheck < 0 ){ icheck = flag; }
  if( icheck >= 100000 )
    {
      if( rp.z()  < z_min  || rp.z()  > z_max  ) { return 0;  }
      icheck -=(int)floor(double(icheck+1)/100000)*100000; 
    }
  if( icheck >= 10000 )
    {
      if( rp.x()  < x_min  || rp.x()  > x_max  ) { return 0;  }
      icheck -=(int)floor(double(icheck+1)/10000)*10000;
    }
  if( icheck >= 1000  )
    {
      if( rp.y()  < y_min  || rp.y()  > y_max  ) { return 0;  }
      icheck -=(int)floor(double(icheck+1)/1000)*1000;
    }
  if( icheck >= 100   )
    {
      if( rp.tx() < tx_min || rp.tx() > tx_max ) { return 0;  }
      icheck -=(int)floor(double(icheck+1)/100)*100;
    }
  if( icheck >= 10 )
    {
      if( rp.ty() < ty_min || rp.ty() > ty_max ) { return 0;  }
      icheck -=(int)floor(double(icheck+1)/10)*10;
    }
  if( icheck >= 1)
    {
      if( rp.pe() < pe_min || rp.pe() > pe_max ) { return 0;  }
    }
  return 1;
}

ostream & operator << (ostream & os, const RcLimit & rcl)
{
  os << "===== Limits used for selection (MARPLE) ======" << endl;
  os << " the point is accepted if the parameters {" << endl;
  int iflag = rcl.flag;
  
  if( iflag >= 100000 )
    { 
      os << rcl.z_min  << " < z  < " << rcl.z_max  << endl;
      iflag -= (int)floor(double(iflag+1)/100000)*100000; 
    }
  if( iflag >= 10000 )
    {
      os << rcl.x_min  << " < x  < " << rcl.x_max  << endl;
      iflag -= (int)floor(double(iflag+1)/10000)*10000;
    }
  if( iflag >= 1000  )
    {
      os << rcl.y_min  << " < y  < " << rcl.y_max  << endl;
      iflag -= (int)floor(double(iflag+1)/1000)*1000;
    }
  
  if( iflag >= 100   )
    {
      os << rcl.tx_min << " < tx < " << rcl.tx_max << endl;
      iflag -= (int)floor(double(iflag+1)/100)*100;
    }
  
  if( iflag >= 10 )
    {
      os << rcl.ty_min << " < ty < " << rcl.ty_max << endl;
      iflag -= (int)floor(double(iflag+1)/10)*10;
    }
  
  if( iflag >= 1)
    { 
      os << rcl.pe_min << " < pe < " << rcl.pe_max << endl;
    }
  return os;
}


