//**********************************************************************/
// Created: 23.09.97,   Olya Igonkina   for   MARPLE package
//
// implementation of RcMatchConst class
//
//**********************************************************************/
#include "marple/marple.hh"
#include <iostream.h>
#include <iomanip.h>

int RcMatchConst::delta_cut_ok( const RcPoint & rp1, 
				       const RcPoint & rp2, 
				       int iflag  ) const
{
  if( iflag < 0 ) { iflag = flag; }
  if( iflag >= 100000 ){ iflag -= (int)floor(double(iflag+1)/100000)*100000; }
  if( iflag >= 10000 )
    {
      if( abs( rp1.x()  - rp2.x()  ) > delta_x ){ return 0; }
      iflag -= (int)floor(double(iflag+1)/10000)*10000;
    }
  if( iflag >= 1000  )
    {
      if( abs( rp1.y()  - rp2.y()  ) > delta_y ){ return 0; }
      iflag -= (int)floor(double(iflag+1)/1000)*1000;
    }
  if( iflag >= 100   )
    {
      if( abs( rp1.tx() - rp2.tx() ) > delta_tx ){ return 0; }
      iflag -= (int)floor(double(iflag+1)/100)*100;
    }
  if( iflag >= 10 )
    {
      if( abs( rp1.ty() - rp2.ty() ) > delta_ty ){ return 0; }
      iflag -= (int)floor(double(iflag+1)/10)*10;
    }
  if( iflag >= 1)
    {
      if( abs( rp1.pe()  - rp2.pe()  ) > delta_pe ){ return 0; }
    }
  return 1;
}
//--------------------------------------------------------------

 int RcMatchConst::delta_cut_ok( const RcPoint & rp1, 
				     const RcLimit & rlimit, 
				     int iflag ) const
{
  if( iflag == -1 ) { iflag = flag; }
  if( iflag >= 100000 ){ iflag -= (int)floor(double(iflag+1)/100000)*100000; }
  if( iflag >= 10000 )
    {
      if( rp1.x()  < rlimit.x_min  - delta_x  || 
	  rp1.x()  > rlimit.x_max  + delta_x    ) { return 0; }
      iflag -= (int)floor(double(iflag+1)/10000)*10000;
    }
  
  if( iflag >= 1000  )
    {
      if( rp1.y()  < rlimit.y_min  - delta_y  || 
	  rp1.y()  > rlimit.y_max  + delta_y    ){ return 0; }
      iflag -= (int)floor(double(iflag+1)/1000)*1000;
    }

  if( iflag >= 100   )
    {
      if( rp1.tx() < rlimit.tx_min - delta_tx || 
	  rp1.tx() > rlimit.tx_max + delta_tx   ){ return 0; }
      iflag -= (int)floor(double(iflag+1)/100)*100;
    }

  if( iflag >= 10 )
    {
      if( rp1.ty() < rlimit.ty_min - delta_ty || 
	  rp1.ty() > rlimit.ty_max + delta_ty   ) { return 0; }
      iflag -= (int)floor(double(iflag+1)/10)*10;
    }

  if( iflag >= 1)
    {
      if( rp1.pe() < rlimit.pe_min - delta_pe ||
	  rp1.pe() > rlimit.pe_max + delta_pe   ) { return 0;  }
    }
  return 1;
}
//--------------------------------------------------------------

ostream & operator << (ostream & os, const RcMatchConst &rms)
{
  os << "=========  Constants for Matching  =========" << endl;
  os << " Z coordinate of Matching plane = "<< rms.z_plane << endl;
  os << " Z_window for matching          = [" << rms.z_min << " , " 
     << rms.z_max << "]" << endl;
  os << " the point is accepted if  chi2 < "
     << rms.chi2 << endl;

  if( rms.use_delta_cut )
    {
      os << " DELTAS_cut :  flag = " << rms.flag << endl ;
      os << "  the following cuts are used :" << endl;
      int iflag = rms.flag;
      
      if( iflag >= 100000 ){ iflag -= (int)floor(double(iflag+1)/100000)*100000; }
      if( iflag >= 10000 )
	{
	  os << "  | x1  - x2  | < " << rms.delta_x << endl;
	  iflag -= (int)floor(double(iflag+1)/10000)*10000;
	}
      if( iflag >= 1000  )
	{
	  os << "  | y1  - y2  | < " << rms.delta_y << endl;
	  iflag -= (int)floor(double(iflag+1)/1000)*1000;
	}
      
      if( iflag >= 100   )
	{
	  os << "  | tx1 - tx2 | < " << rms.delta_tx << endl;
	  iflag -= (int)floor(double(iflag+1)/100)*100;
	}

      if( iflag >= 10 )
	{
	  os << "  | ty1 - ty2 | < " << rms.delta_ty << endl;
	  iflag -= (int)floor(double(iflag+1)/10)*10;
	}
      
      if( iflag >= 1)
	{ 
	  os << "  | pe1 - pe2 | < " << rms.delta_pe << endl;
	}
    } else {
      os << " no DELTAS_cut is used on {x, y, tx, ty, pe } " << endl;
    }
  if( rms.use_pair_cut )
    {
      os << " PAIR_cut is used : " << endl;
      os << "  " << rms.n_pair_left  
	 << " combination with left object is allowed" << endl;
      os << "  " << rms.n_pair_right 
	 << " combination with right object is allowed" << endl;
    }else{
      os << " no PAIR_cut is used number of combination with objects" << endl;
    }

  return os ;
}

 
