//**********************************************************************/
// Created: 23.08.97,   Olya Igonkina   for   MARPLE package
//
// implementation of RcPoint class
//
//**********************************************************************/

#include "marple/marple.hh"

#include "arte/Arte.hh"
#include "arte/ArteMessage.hh"
#include "auto/TOPP.hh"
#include "auto/TOPX.hh"
#include "auto/GEDE.hh"

#include <iomanip.h>

#include <CLHEP/Matrix/Vector.h>

extern vector <RcMsOfGesl> msGeslList;


// Destructor
RcPoint::~RcPoint() 
{}

RcPoint::RcPoint(const Hep3Vector &rp,
		 HepDouble tx ,
		 HepDouble ty ,
		 HepDouble pe ,
		 int mark)
  : Hep3Vector(rp), ftx(tx), fty(ty), fpe(pe), fcov5(5,1), fmark(mark)
{
  //  HepSymMatrix m5(5,1);
  fcov5 = fcov5 * MARPLE_INFINITY;
}  

RcPoint::RcPoint(const Hep3Vector &rp,
		 HepDouble tx ,
		 HepDouble ty ,
		 HepDouble pe ,
		 const HepSymMatrix &cov,
		 int mark)
  : Hep3Vector(rp), ftx(tx), fty(ty), fpe(pe), fcov5(5,0), fmark(mark)
{
  setcov(cov);
}  

RcPoint::RcPoint(const Hep3Vector &rp,
		 HepDouble tx ,
		 HepDouble ty ,
		 const HepSymMatrix &cov,
		 int mark)
  : Hep3Vector(rp), ftx(tx), fty(ty), fcov5(5,0), fmark(mark)
{
  setcov(cov);
}  

RcPoint::RcPoint( const Hep3Vector &rp,
		  HepDouble tx ,
		  HepDouble ty ,
		  HepDouble pe ,
		  const double cov_array[15],
		 int mark)
  : Hep3Vector(rp), ftx(tx), fty(ty), fpe(pe), fcov5(5,0), fmark(mark)
{
  setcov(cov_array);
}

RcPoint::RcPoint( const Hep3Vector &rp,
		  HepDouble tx ,
		  HepDouble ty ,
		  HepDouble pe ,
		  const float cov_array[15],
		  int mark)
  : Hep3Vector(rp), ftx(tx), fty(ty), fpe(pe), fcov5(5,0), fmark(mark)
{
  setcov(cov_array);
}

RcPoint::RcPoint( const Hep3Vector &rp,
		  HepDouble tx ,
		  HepDouble ty ,
		  const double cov_array[15],
		  int mark)
  : Hep3Vector(rp), ftx(tx), fty(ty), fpe(0.), fcov5(5,0), fmark(mark)
{
  setcov(cov_array);
  fcov5(1,5) = fcov5(2,5) = fcov5(3,5) = fcov5(4,5) = 0.;
  fcov5(5,5) = MARPLE_INFINITY;
}

RcPoint::RcPoint( const Hep3Vector &rp,
		  HepDouble tx ,
		  HepDouble ty ,
		  const float cov_array[15],
		  int mark)
  : Hep3Vector(rp), ftx(tx), fty(ty), fpe(0.), fcov5(5,0), fmark(mark)
{
  setcov(cov_array);
  fcov5(1,5) = fcov5(2,5) = fcov5(3,5) = fcov5(4,5) = 0.;
  fcov5(5,5) = MARPLE_INFINITY;
}

//-----------------------------------------------------------------------
//equal 1 if diagonal elements of cov() are not equal '0', otherwise 0

int RcPoint::covMatrixOk() const
{
  for( int i=1; i<= 5 ;i++ )
    {
      if( cov()(i,i) == 0 ) return 0;
    }
  return 1;
}

//set diagonal elements of cov() to MARPLE_INFINITY instead of '0' 
int RcPoint::checkAndCorrectCov()
{
  int correct = 0;
  for( int i=1; i<= 5 ; i++ )
    {
      if( cov()(i,i) == 0 || cov()(i,i) > MARPLE_INFINITY ) 
	{
	  setcov( i,i, MARPLE_INFINITY) ;
	  correct = 1;
	}
    }
  return correct;
}

//-----------------------------------------------------------------------
// prolongate rp point to the z coordinate
// depends on the list of multiple scattering
void RcPoint::prolongate( HepDouble z_next )
{  
  int iErr=0;
  prolongate( z_next, iErr);
}

void RcPoint::prolongate( HepDouble z_next, 
			  int & iErr, 
			  int linear, 
			  int imsGesl)
{  
  iErr = 0;
  if(msGeslList.empty() )
    { 
      arteMsg << "MARPLE warning: msGeslList is empty "<< endl
	   << "call marpleinit() for correct MARPLE initialization";
      arteMsg.allp();
    }
  
  HepDouble z_start = z();
  unsigned int iumsGesl;
  if(  imsGesl < 0  )
    { 
      iumsGesl = 0; 
    }else{
      if( unsigned(imsGesl) >= msGeslList.size()  ) 
	{
	  iumsGesl = msGeslList.size() - 1; 
	}else{
	  iumsGesl = imsGesl;
	}
    }
  // step I find the closest GESL plane
  int istep = 0;
  if( z_start < msGeslList[iumsGesl].zcenter() )
    {
      for( istep=iumsGesl;  
	   istep>=0 && z_start < msGeslList[istep].zcenter(); istep-- )
	{}
      if( z_start < z_next ) istep += 1; 
    } else {
      for( istep=iumsGesl; istep>=0 && istep < msGeslList.size() &&
	     z_start > msGeslList[istep].zcenter() ; istep++ )
	{}
      if( z_start > z_next ) istep -= 1;
    }
  if( istep < 0 ) 
    { 
      lineswim ( z_next); 
      return;
    }

  // step II prolongate till z_next
  // swim RcPoint to the middle of the next GESL
  // then add multiple scattering
  HepDouble angle_correction = 1 + tx()*tx() + ty()*ty(); 
  angle_correction = sqrt( angle_correction);

  HepDouble insideMagnet = msGeslList[istep].insideMagnet();
  
  // prolongate forward direction ( along z axis)
  if( z_start < z_next )
    {
      // check if the start point is inside the medium
      HepDouble z_cor = correct_zback( z_start, 
				       msGeslList[istep].gesl() );
      int doCorrection = 0;
      HepDouble tmpMs = 0;
      if( z_cor > 0 )
	{
	  //correct the overestimation of the medium
	  doCorrection = 1;
	  tmpMs =  -msGeslList[istep].ms() * angle_correction *z_cor ;
	}else{
	  if( istep > 0 )
	    {
	      z_cor = correct_zforward( z_start, 
					msGeslList[istep-1].gesl() );
	      if( z_cor > 0 ) 
		{
		  //correct underestimation of medium
		  doCorrection = 1;
		  tmpMs = msGeslList[istep].ms() * angle_correction *z_cor;
		}
	    }
	}
      // second step
      for( ; istep>=0 && istep < msGeslList.size() && 
	     msGeslList[istep].zcenter() < z_next; istep++)
	{
	  // prolongate even points with pe()=0 
	  //             to have correct matrix in magnet
	  if( (insideMagnet || msGeslList[istep].insideMagnet() ) 
	      && !linear )
	  {
	    swimRunge( msGeslList[istep].zcenter(), iErr);
	    if( iErr == 1 ) return;
	  }else{
	      lineswim(  msGeslList[istep].zcenter());
	  }
	  insideMagnet = msGeslList[istep].insideMagnet();
	  if( msGeslList[istep].check(*this) )
	    { 
	      addMs( msGeslList[istep].ms() * angle_correction );
	      if( doCorrection )
		{  
		  addMs( tmpMs);
		  doCorrection = 0 ;
		}
	    }
	}
      // last step
      if( z() < z_next )
	{ 
	  if(  insideMagnet && !linear )
	    {
	      swimRunge( z_next, iErr );
	      if( iErr == 1 ) return;
	    }else{
	      lineswim( z_next );
	    }
	}
      // check if the end point is inside the medium
      if( istep > 0 )
	{
	  z_cor = correct_zforward( z_next, 
				    msGeslList[istep-1].gesl() );
	}else{
	  z_cor = 0;
	}
      if( z_cor > 0 ) 
	{
	  //correct overestimation of medium
	  if( istep > 0 && msGeslList[istep-1].check(*this) )
	    { 
	      addMs( -msGeslList[istep-1].ms() * angle_correction *z_cor);
	    }
	}else{
	  if( (istep) < msGeslList.size() )
	    {
	      z_cor = correct_zback( z_next, msGeslList[istep].gesl() );
	      if( z_cor > 0 )
		{
		  //correct the underestimation of the medium
		  if( msGeslList[istep].check(*this) )
		    { 
		      addMs( msGeslList[istep].ms() * 
			     angle_correction *z_cor);
		    }
		}
	    }
	}
    //
    // or prolongate back direction 
    //
    } else {
      // check if the start point is inside the medium
      HepDouble z_cor = correct_zforward( z_start, 
					  msGeslList[istep].gesl() );
      int doCorrection = 0;
      HepDouble tmpMs = 0;
      if( z_cor > 0 ) 
	{
	  //correct the overestimation of the medium
	  doCorrection = 1;
	  tmpMs = -msGeslList[istep].ms() * angle_correction *z_cor ;
	}else{
	  if( (istep+1) < msGeslList.size() )
	    {
	      z_cor = correct_zback( z_start, 
				     msGeslList[istep+1].gesl() );
	      if( z_cor > 0 )
		{
		  //correct the underestimation of the medium
		  doCorrection = 1;
		  tmpMs = msGeslList[istep].ms() * angle_correction *z_cor;
		}
	    }
	}
      // second step
      for( ;  istep>=0 && msGeslList[istep].zcenter() > z_next; istep--)
	{
	  if( (insideMagnet || msGeslList[istep].insideMagnet()) 
	      && !linear )
	    {
	      swimRunge( msGeslList[istep].zcenter(), iErr);
	      if( iErr == 1 ) return;
	    }else{
	      lineswim(  msGeslList[istep].zcenter());
	    }
	  insideMagnet = msGeslList[istep].insideMagnet();
	  if( msGeslList[istep].check(*this) )
	    { 
	      addMs( msGeslList[istep].ms() * angle_correction ); 
	      if( doCorrection )
		{  
		  addMs( tmpMs);
		  doCorrection = 0 ;
		}
	    }
	}
      // last step
      if( z() > z_next )
	{ 
	  if( insideMagnet && !linear )
	    {
	      swimRunge(z_next, iErr );
	      if( iErr == 1 ) return;
	    }else{
	      lineswim(  z_next);
	    }
	}
      //take into account the medium
      if( istep >=0 )
	{
	  z_cor = correct_zback( z_next, msGeslList[istep].gesl() );
	}else{
	  z_cor = 0;
	}
      if( z_cor > 0 )
	{
	  //correct the overestimation of the medium
	  if( istep>0 && msGeslList[istep-1].check(*this) )
	    { 
	      addMs( -msGeslList[istep].ms() * angle_correction *z_cor);
	    }
	}else{
	  if( istep > 0 )
	    {
	      z_cor = correct_zforward( z_start, 
					msGeslList[istep-1].gesl() );
	      if( z_cor > 0 && msGeslList[istep-1].check(*this) ) 
		{
		  //correct underestimation of medium
		  addMs( msGeslList[istep].ms() 
			 * angle_correction *z_cor);
		}
	    }
	}
    }
}


//-----------------------------------------------------------------------
//swim  the RcPoint to the z_next plane without multiple scattering

// assume that magnet parameters are :
// z_magnet = 450. , r_magnet = 140., Hmag =8.

void RcPoint::fastswim( HepDouble z_next, int & iErr )
{
  HepDouble ZPOSMG = 450;
  HepDouble RMAG   = 140;
  HepDouble HMAG   = 8; 
  iErr = 0;
  //if( pe() )
  //{ lineswim( z_next);    return; }
  if( (z_next - z() ) * ( z_next - ZPOSMG - RMAG ) < 0 || 
      (z_next - z() ) * ( z_next - ZPOSMG + RMAG ) < 0  )
    { lineswim( z_next );   return ; }

  // step I - swiming outside the magnet till its border
  if( (x()*x() + ( z()-ZPOSMG)*(z()-ZPOSMG) ) > RMAG*RMAG )
    {
      HepDouble AA = tx()*tx() + 1;
      HepDouble B1 = x() - tx()*z();
      HepDouble BB = tx()*B1 - ZPOSMG;

      HepDouble DD = BB*BB - AA*(B1*B1 + ZPOSMG*ZPOSMG - RMAG*RMAG);
      if( DD < MARPLE_NEGLIGIBLE )
	{  lineswim(z_next);  return;   }
      if( z() < ZPOSMG )
	{
	  HepDouble z_tmp = ( -BB - sqrt(DD) ) / AA ;
	  if( z_next < z_tmp )
	    { 
	      lineswim(z_next);
	      return;
	    }else{
	      lineswim(z_tmp);
	    }
	}else{
	  HepDouble z_tmp = ( -BB + sqrt(DD) ) / AA ;
	  if( z_next > z_tmp )
	    { 
	      lineswim(z_next);
	      return;
	    }else{
	      lineswim(z_tmp);
	    }
	}
    }

  // step II - swiming inside the magnet 

  HepDouble rho = 3336.1134 / HMAG * sqrt( 1+tx()*tx() ) /
    sqrt( 1 + tx()*tx() + ty()*ty() ) ;
  // pe should be equal Q/p 
  if (pe() != 0 )
    {
      rho /= pe();
    }else{
      //arteMsg << "MARPLE : swiming through magnet for point with Q/p = 0";
      //arteMsg.allp();
      lineswim(z_next);
      return;
    }
  HepDouble XR = x();
  if( abs( tx()) > MARPLE_NEGLIGIBLE )
    {
      XR += -rho * tx()/ abs(tx()) / sqrt( 1 + tx()*tx()) ;
    }
  HepDouble YR = tx()*( x()-XR ) + z();

  HepDouble B1 = (RMAG*RMAG -ZPOSMG*ZPOSMG +XR*XR +YR*YR -rho*rho)/2;
  HepDouble AA = (ZPOSMG - YR)*(ZPOSMG - YR) + XR*XR;
  HepDouble BB = (ZPOSMG-YR)*B1 -XR*XR*ZPOSMG;
  
  HepDouble DD = BB*BB - AA*(B1*B1 -XR*XR*( RMAG*RMAG -ZPOSMG*ZPOSMG));
  //the particle is assumed to staying in magnet
  if( DD < MARPLE_NEGLIGIBLE )
    {  swimRunge(z_next, iErr); return; }
  if( z() < ZPOSMG )
    {
      HepDouble z_tmp = ( -BB - sqrt(DD) ) / AA ;
      if( z_next < z_tmp )
	{ 
	  swimRunge(z_next, iErr);
	  return;
	}else{
	  swimRunge(z_tmp, iErr);
	  if( iErr == 1 ) return;
	}
    }else{
      HepDouble z_tmp = ( -BB + sqrt(DD) ) / AA ;
      if( z_next > z_tmp )
	{ 
	  swimRunge(z_next, iErr);
	  return;
	}else{
	  swimRunge(z_tmp, iErr);
	  if( iErr == 1 ) return;
	}
    }

  //step III - swiming outside the magnet
  lineswim( z_next);
}


//-----------------------------------------------------------------------
// linear prolongation of the RcPoint parameters to the z_next plane.

void RcPoint::lineswim( HepDouble z_next)
{
  // transport parameters 
  HepDouble dz = z_next - z();
  if( abs(dz) < 1.e-8) return;
  Hep3Vector shift( dz*tx(), dz*ty(), 0.0);  // x = x + dz * tx
  Hep3Vector::operator += (shift);
  setZ( z_next);
  
  // transport matrix
  HepMatrix cov_correction(5,5,1);  // assistance matrix for transport F
  cov_correction[0][2] = dz;  // (1,3)
  cov_correction[1][3] = dz;  // (2,4)
  
  setcov( cov().similarity(cov_correction) );
}

//-----------------------------------------------------------------------
extern "C" 
{ extern void FORTRAN_ROUTINE( rkmain )( int*,
					 double*,
					 double*,
					 double [6],
					 double [4],
					 double [6],
					 double [5][5],
					 int*);
}

void RcPoint::swimRunge( HepDouble z_next,   
			 int & iErr,
			 HepDouble FFstep )
{
  iErr = 0;
  double distanceCor = abs( z_next - z() );
  if( distanceCor < 0.1e-5 ) return;
  
  double Fstep;
  if( FFstep ) 
    {
      Fstep = FFstep;
    }else{
      Fstep = distanceCor;
    }
  if( Fstep < 0.1e-5 ){   return;  }

  //be sure to use Q/p as pe() value !!!
  double Pin[6]  = { z(), x(), y(), tx(), ty(), pe() };
  if( distanceCor < 1 ) distanceCor = 1;
  double Prec[4] = { 0.001/distanceCor, 0.001/distanceCor, 
		     0.0001/distanceCor, 0.0001/distanceCor};
  double Pout[6] ;
  double RKD[5][5];

  int Irkmain = 2; // default value - fast calculation of matrix
  FORTRAN_ROUTINE( rkmain )(&Irkmain,
                            &z_next, &Fstep, Pin, Prec, 
                            Pout, RKD, &iErr );
  //FORTRAN_ROUTINE( rkmain )( &z_next, &Fstep, Pin, Prec, 
  //			     Pout, RKD, &iErr );
  switch (iErr )
    {
    case 1 :
      arteMsg << " MARPLE : track curls during Runge-Kutta tracking" ;
      arteMsg.allp();
      break;
    case 2 :
      arteMsg << " MARPLE : zero FSTEP was used";
      arteMsg.allp();
      break;
    case 3 :
      arteMsg << " MARPLE :rkstep call limit exceeded";
      arteMsg.allp();
      break;
    }
  setX(  Pout[1]);
  setY(  Pout[2]);
  setZ(  z_next );
  settx( Pout[3]);
  setty( Pout[4]);
  setpe( Pout[5]);

  HepMatrix cov_correction(5,5);
  for(int i=0; i<5; i++ )
    { 
      for(int j=0; j<5; j++ )
	{
	  // C++ style of array is opposite to fortran's one
	  cov_correction( i+1, j+1) = RKD[j][i];
	}
    }
  setcov( cov().similarity(cov_correction) );
}

//-----------------------------------------------------------------------
// correct covariance matrix by the multiple scattering 

void RcPoint::addMs( const HepDouble ms )
{
  // use pe() = Q/p !!!
  if( pe() )
    { 
      HepDouble correction = cov()(3,3) + ms*pe()*pe() ;
      setcov( 3, 3, correction );
      correction = cov()(4,4) + ms*pe()*pe() ;
      setcov( 4, 4, correction );
    } else {
      HepDouble correction = cov()(3,3) + ms;
      setcov( 3, 3, correction );
      correction = cov()(4,4) ;
      setcov( 4, 4, correction );
    }
}

//-----------------------------------------------------------------------
// Kalman filter to update information with measured value HITB ( CHANGE RcPoint), return $\Delta \chi^2$ of procedure. 
HepDouble RcPoint::filter( ArtePointer< HITB> hitb, int & iErr)
{
  iErr = 0;
  prolongate( HepDouble(hitb->z), iErr );
 
  if( iErr ) 
    {
      arteMsg << "MARPLE: prolongation is impossible, filter aborted";
      arteMsg.allp();
      return 0 ; 
    }
  HepDouble sig = hitb->r; // the accuracy of SVD measurement
  HepDouble sig2 = sig*sig ;
  HepDouble co =  (ArtePointer<GEDE>(hitb->gede))->cos ;
  HepDouble si =  (ArtePointer<GEDE>(hitb->gede))->sin ;
  HepVector Hvec(5,0);
  HepDouble residual, hCh;
  HepDouble mU, Hp, cosThetaU, sicoThetaU;
  
  int cmp = (ArtePointer<GEDE>(hitb->gede))->cmp;
  switch ( cmp ) 
    {
    case 1 :
    case 3 :
      residual = hitb->x - co* x() + si * y();
      Hvec(1) = co;
      Hvec(2) = -si;
      
      hCh = cov()(1,1)*co*co + cov()(2,2)*si*si
	- 2*cov()(1,2)*co*si ;
      if( hCh <= 0 )
	{ 
	  int idMess = 345235;
	  arteMsg << "MARPLE: filter: matrix is not pos-defined!";
	  arteMsg.allp(idMess, 10);
	  hCh = abs(hCh);
	  iErr = 10;
	}
      break;
    case 2 :      
      mU = tx()*co - ty()*si; // tan ( track angle) in uz-plane
      cosThetaU = 1 / sqrt( 1 + mU*mU ); 
      sicoThetaU = mU * cosThetaU * cosThetaU;
      
      Hp = ( x()*co - hitb->x  - y()*si ) * cosThetaU; //xwire = hitb_x/co
      Hvec(1) =  co * cosThetaU;
      Hvec(2) = -si * cosThetaU;
      Hvec(3) = -co   * sicoThetaU * Hp;
      Hvec(4) = -(-si)* sicoThetaU * Hp;
      
      residual = hitb->q - Hp;
      hCh = cov().similarity(Hvec);
      break;
      
    default :
      arteMsg << "MARPLE: filter: no rules to add HITB " << hitb 
	   << " with CMP = " << cmp << " to RcPoint exist" ;
      arteMsg.allp();
      iErr = 11;
      return 0;
      break;
    }
  

  HepVector Gain(5,0);
  Gain = cov()*Hvec;
  Gain /= sig2+hCh ;
  
  setX(  x()  + Gain(1)*residual );
  setY(  y()  + Gain(2)*residual );
  settx( tx() + Gain(3)*residual );
  setty( ty() + Gain(4)*residual );
  setpe( pe() + Gain(5)*residual );
  
  HepMatrix correctM(5,5,1);
  correctM = correctM - Gain*Hvec.T();
  correctM = correctM * cov() ;
  HepSymMatrix newcov(5,0);
  newcov.assign(correctM);
  setcov( newcov );

  // chi2 = [(1-Hvec.T()*Gain)*residual]**2 / [(1-Hvec.T()*Gain)*sig2]
  
  HepDouble deltaChi2 = Gain(1)*Hvec(1) + Gain(2)*Hvec(2) +
    Gain(3)*Hvec(3) + Gain(4)*Hvec(4) + Gain(5)*Hvec(5); 
  deltaChi2 = residual * residual * ( 1 - deltaChi2) / sig2;

  return deltaChi2;
}

//-----------------------------------------------------------------------
bool comp_hitb_z( const ArtePointer<HITB> hitb1, 
		  const ArtePointer<HITB> hitb2)
{  return hitb1->z < hitb2->z ;    } 

// Kalman refit, return $\chi^2$ of procedure ( CHANGE RcPoint). 
HepDouble RcPoint::refit(int & iErr)
{
  iErr = 0;
  HepDouble deltaChi2 = 0;

  vector<ArtePointer<HITB> > hitbL = hitbList();
  sort( hitbL.begin(), hitbL.end(), comp_hitb_z);
  HepDouble z_first = hitbL[0]->z;
  HepDouble z_end = hitbL[ hitbL.size()-1 ]->z;

  int iErr1 = 0;
  // refit started from closest end of HITB vector
  if( abs( z() - z_first) < abs( z() - z_end) )
    {  
      for( unsigned int iHit=0; iHit < hitbL.size(); iHit++ )
	{ 
	  deltaChi2 += filter( hitbL[iHit], iErr1 ); 
	  if( iErr1 ) iErr = iErr1;
	}
    }else{
      for( unsigned int iHit=hitbL.size(); iHit > 0; iHit-- )
	{ 
	  deltaChi2 += filter( hitbL[iHit-1], iErr1  ); 	
	  if( iErr1 ) iErr = iErr1;
	}
    }
  return deltaChi2;
}

//-----------------------------------------------------------------------
HepDouble RcPoint::refitForward(int & iErr)
{
  iErr = 0;
  HepDouble deltaChi2 = 0;
  vector<ArtePointer<HITB> > hitbL = hitbList();
  sort( hitbL.begin(), hitbL.end(), comp_hitb_z);

  int iErr1 = 0;
  // refit started from closest Z of HITB vector to bigger one
  unsigned int iHit=0;
  for(; iHit < hitbL.size() && (hitbL[iHit]->z) < z(); iHit++ )
    {}
  for( ; iHit < hitbL.size() ; iHit++ )
    {
      deltaChi2 += filter( hitbL[iHit], iErr1 );
      if(iErr1 ) iErr = iErr1; 
    }
   
  return deltaChi2;
}

HepDouble RcPoint::refitBack(int & iErr)
{
  iErr = 0;
  HepDouble deltaChi2 = 0;
  vector<ArtePointer<HITB> > hitbL = hitbList();
  sort( hitbL.begin(), hitbL.end(), comp_hitb_z);

  // refit started from closest Z of HITB vector to smaller one
  unsigned int iHit=hitbL.size();
  for(; iHit > 0 && (hitbL[iHit-1]->z > z()); iHit-- )
    {}
  int iErr1 = 0;
  for( ; iHit > 0; iHit-- )
    {  
      deltaChi2 += filter( hitbL[iHit-1], iErr1  );     
      if( iErr1 ) iErr = iErr1;
    }
  
  return deltaChi2;
}

//----------------------------------------------------------------------------

//estimate momentum using target information and correct it (CHANGE RcPoint)
// using Peter Weyers algorithm
void RcPoint::estimatePEfromTAR( HepDouble tarX, HepDouble tarY, 
				 HepDouble tarZ,
				 HepDouble covTarXX, HepDouble covTarYY, 
				 HepDouble covTarXY,
				 int nIter, 
				 int & iErr )
{
  estimatePARfromTAR( 5, tarX, tarY, tarZ, 
		      covTarXX, covTarYY, covTarXY,
		      nIter, iErr );
}

void RcPoint::estimatePARfromTAR( int iPar,
				  HepDouble tarX, HepDouble tarY, 
				  HepDouble tarZ,
				  HepDouble covTarXX, HepDouble covTarYY, 
				  HepDouble covTarXY,
				  int nIter, 
				  int & iErr )
{
  iErr = 0;
  if( covTarXY == 0 )
    {
      if( covTarXX ) covTarXX = 1 / covTarXX;
      if( covTarYY ) covTarYY = 1 / covTarYY;
    }else{
      HepDouble determinant = 1/(covTarXX*covTarYY-covTarXY*covTarXY);
      covTarXY /= -determinant;
      HepDouble tmpXX = covTarXX;
      covTarXX = covTarYY/determinant;
      covTarYY = tmpXX/determinant;
    }

  HepDouble parNew = 0;

  for( int iter=0; !iErr && iter< nIter; iter++ )
    {
      RcPoint tmpPoint(*this);
      tmpPoint.prolongate( tarZ );

      // invert covariant matrix of target
      HepSymMatrix invCov = tmpPoint.cov().inverse(iErr);
      if( iErr != 0 ){ error_message(); continue; }

      invCov(1,1) +=covTarXX;
      invCov(2,2) +=covTarYY;
      invCov(1,2) +=covTarXY;
      invCov.invert(iErr);
      if( iErr != 0 ){ error_message(); continue; }

      HepDouble deltaX = tmpPoint.x() - tarX;
      HepDouble deltaY = tmpPoint.y() - tarY;
      
      parNew = -invCov(iPar,1)*( deltaX * covTarXX + deltaY*covTarXY )
	-       invCov(iPar,2)*( deltaY * covTarYY + deltaX*covTarXY );      
      switch (iPar) 
	{
	case 1 :
	  parNew += tmpPoint.x();
	  setX( parNew);
	  break;
	case 2 :
	  parNew += tmpPoint.y();
	  setY( parNew);
	  break;
	case 3 :
	  parNew += tmpPoint.tx();
	  settx( parNew);
	  break;
	case 4 :
	  parNew += tmpPoint.ty();
	  setty( parNew);
	  break;
	case 5 :
	  parNew += tmpPoint.pe();
	  setpe( parNew);
	  break;      
	}
      setcov( iPar,iPar, invCov(iPar,iPar) );
    }
}


//----------------------------------------------------------------

extern void maFindTarget( RcPoint & rp, 
			  HepDouble &tarX,   HepDouble &tarY,  
			  HepDouble &tarZ,
			  HepDouble &covTarXX, HepDouble &covTarYY, 
			  HepDouble &covTarXY, 
			  int & iErr);

//----------------------------------------------------------------

void RcPoint::estimatePEfromTAR( int nIter, 
				 int & iErr )
{
  estimatePARfromTAR( 5,  nIter, iErr );
}

void RcPoint::estimatePARfromTAR(  int iPar,
				   int nIter, 
				   int & iErr )
{
  HepDouble parNew = 0;
  HepDouble tarX, tarY, covTarXX, covTarYY, covTarXY;
  HepDouble tarZ =0;
  iErr = 0;
  for( int iter=0; !iErr && iter< nIter; iter++ )
    {
      RcPoint tmpPoint(*this);
      tmpPoint.prolongate( tarZ, iErr );
      if( iErr == 1 ) return;

      // to be replaced by VERTEX subroutine/function !!!
      maFindTarget( tmpPoint, tarX, tarY, tarZ, 
		    covTarXX, covTarYY, covTarXY, 
		    iErr );
      if( iErr ) return;

      // invert covariant matrix of target
      if( covTarXY == 0 )
	{
	  if( covTarXX ) covTarXX = 1 / covTarXX;
	  if( covTarYY ) covTarYY = 1 / covTarYY;
	}else{
	  HepDouble determinant = 1/(covTarXX*covTarYY-covTarXY*covTarXY);
	  covTarXY /= -determinant;
	  HepDouble tmpXX = covTarXX;
	  covTarXX = covTarYY/determinant;
	  covTarYY = tmpXX/determinant;
	}
      
      tmpPoint.lineswim( tarZ);
      HepSymMatrix invCov = tmpPoint.cov().inverse(iErr);
      if( iErr ){ error_message(); return; }

      invCov(1,1) +=covTarXX;
      invCov(2,2) +=covTarYY;
      invCov(1,2) +=covTarXY;
      invCov.invert(iErr);
      if( iErr ){ error_message();  return; }

      HepDouble deltaX = tmpPoint.x() - tarX;
      HepDouble deltaY = tmpPoint.y() - tarY;

      parNew = -invCov(iPar,1)*( deltaX * covTarXX + deltaY*covTarXY )
	-       invCov(iPar,2)*( deltaY * covTarYY + deltaX*covTarXY );      
      switch (iPar) 
	{
	case 1 :
	  parNew += tmpPoint.x();
	  setX( parNew);
	  break;
	case 2 :
	  parNew += tmpPoint.y();
	  setY( parNew);
	  break;
	case 3 :
	  parNew += tmpPoint.tx();
	  settx( parNew);
	  break;
	case 4 :
	  parNew += tmpPoint.ty();
	  setty( parNew);
	  break;
	case 5 :
	  parNew += tmpPoint.pe();
	  setpe( parNew);
	  break;      
	}
      setcov( iPar,iPar, invCov(iPar,iPar) );
    }
}


//-----------------------------------------------------------------------
// returns empty mtraList
vector<ArtePointer<MTRA> > RcPoint::mtraList() const
{
  vector<ArtePointer<MTRA> > dummy;
  return dummy;
}

// returns empty hitbList
vector<ArtePointer<HITB> > RcPoint::hitbList() const
{
  vector<ArtePointer<HITB> > dummy;
  return dummy;
}
// returns empty hitcList
vector<ArtePointer<HITC> > RcPoint::hitcList() const
{
  vector<ArtePointer<HITC> > dummy;
  return dummy;
}

//-----------------------------------------------------------------------
// update ARTE bank connected with point
void RcPoint::updateARTE(int & iErr ) const
{
  iErr = 1;
  arteMsg << "MARPLE: updateARTE : ARTE bank is unknown";
  arteMsg.allp();
}


//-----------------------------------------------------------------------
// return 1 if at least one element in mtraList1 == any element in mtraList2
int RcMtraOk( const vector<ArtePointer<MTRA> > & mtraList1, 
	      const vector<ArtePointer<MTRA> > & mtraList2 )
{
  for( unsigned int iMtra1 = 0; 
       iMtra1 < mtraList1.size(); iMtra1++ )
    {
      for( unsigned int iMtra2 = 0; 
	   iMtra2 < mtraList2.size(); iMtra2++ )
	{
	  if( mtraList1[iMtra1] == mtraList2[iMtra2] )
	    {   return 1; }
	}
    }
  return 0;
}

int RcMtraOk( ArtePointer<MTRA>  & mtraEntry, 
	      const vector<ArtePointer<MTRA> > & mtraList )
{
  for( unsigned int iMtra = 0; 
       iMtra < mtraList.size(); iMtra++ )
    {
      if( mtraEntry == mtraList[iMtra] )
	    {   return 1; }
    }
  return 0;
}


//-----------------------------------------------------------------------
//return 1 if RcPoint leaved by lepton from J/psi
int fromJpsi( const RcPoint & rp)
{
  vector<ArtePointer<MTRA> > mtraList = rp.mtraList();
  for( unsigned int iMtra = 0; 
       iMtra < mtraList.size(); iMtra++ )
    {
      ArtePointer<MTRA> mtra = mtraList[iMtra];
      int iLund = ArtePointer<TOPP>(mtra->topp)->lund;
      if( abs(iLund) == 11 || abs(iLund) == 13 )
	{
	  mtra  = ArtePointer<MTRA>(ArtePointer<MTRX>(mtra)->btra);
	  iLund = ArtePointer<TOPP>(mtra->topp)->lund;
	  if( abs(iLund) == 443 )
	    {  return 1;  }
	}
    }
  return 0;
}

//-----------------------------------------------------------------------
// print ostream

ostream & operator << (ostream & os, const RcPoint &rp)
{
  os << " z= " << setw(4) << rp.z();
  os << "; x = " <<  setw(4) << rp.x() ;
  os << ";  y = " << setw(4) << rp.y() ;
  os << "; tx = " << setw(4) << rp.tx() ;
  os << "; ty = " << setw(4) << rp.ty() ;
  os << "; pe = " << setw(4) << rp.pe() <<endl ;
  os << rp.cov();
  os << endl;
  os << "         marking flag = " << rp.mark() << endl;
  os <<  "            MTRA link: " ;
  vector<ArtePointer<MTRA> > mList( rp.mtraList());
  unsigned int i=0;
  for(i=0; i < mList.size(); i++)
    {
      os << mList[i];
    }
  os << endl;
  os <<  "            HITB link: " ;
  vector<ArtePointer<HITB> > hbList( rp.hitbList());
  for( i=0; i < hbList.size(); i++)
    {
      os << hbList[i];
    }
  os << endl;
  os <<  "            HITC link: " ;
  vector<ArtePointer<HITC> > hcList( rp.hitcList());
  for(i=0; i < hcList.size(); i++)
    {
      os << hcList[i];
    }
  os << endl;

  return os;
}

// ostream for mtraList

ostream & operator << (ostream & os, 
		       const vector<ArtePointer<MTRA> > mtraList)
{
  for(unsigned int i=0; i < mtraList.size(); i++ )
    {  os <<mtraList[i]; }
  return os;
}
