//**********************************************************************/
// Created: 23.08.97,   Olya Igonkina   for   MARPLE package
//
// implementation of RcSegPoint class
//
//**********************************************************************/
#include <iostream.h>
#include <iomanip.h>

#include "arte/Arte.hh"
//#include <Pointer.hh>
#include "auto/MTRA.h"

#include "marple/marple.hh"

RcSegPoint::RcSegPoint( const ArtePointer <RSEG>  rsegLink,
			int EndPoint)
  : RcPoint(), fSegLink(rsegLink)  
{
  if( rsegLink == 0 || rsegLink.index() > rsegLink.nrow() )
    { 
      arteMsg << " MARPLE: RSEG entry number " << rsegLink 
	      << "does not exist";
      arteMsg.allp();
      arteMsg << "- empty object is created" ;
      arteMsg.allp();
      return;
    }
  if( EndPoint )
    {
      setX(   rsegLink->xe  );
      setY(   rsegLink->ye  );
      setZ(   rsegLink->ze  );
      settx(  rsegLink->txe );
      setty(  rsegLink->tye );
      setpe(  rsegLink->pe );
      setcov( rsegLink->cve );
    }else{
      setX(   rsegLink->xf  );
      setY(   rsegLink->yf  );
      setZ(   rsegLink->zf  );
      settx(  rsegLink->txf );
      setty(  rsegLink->tyf );
      setpe(  rsegLink->pf );
      setcov( rsegLink->cvf );
    }
  checkAndCorrectCov();
}

// returns relation with MTRA 
vector<ArtePointer<MTRA> > RcSegPoint::mtraList() const
{
  vector<ArtePointer<MTRA> > mList;
  for( ArteRelationIterator<RSEG_MTRA> rsMtra( link()->mtra);
       rsMtra.valid();
       rsMtra.next() )
    {
      ArtePointer<MTRA> mEntry( rsMtra);
      mList.push_back(mEntry);
    }
  return mList;
}

// returns relation with HITB 
vector<ArtePointer<HITB> > RcSegPoint::hitbList() const
{
  vector<ArtePointer<HITB> > hList;
  for( ArteRelationIterator<RSEG_HITB> rsHitb( link()->hitb);
       rsHitb.valid();
       rsHitb.next() )
    {
      ArtePointer<HITB> hEntry( rsHitb);
      hList.push_back(hEntry);
    }
  return hList;
}
//------------------------------------------------------
// update RSEG bank by RcSegPoint information

void RcSegPoint::updateARTE( int & iErr ) const
{
  int EndPoint;
  if( abs( link()->ze - z() ) <  abs( link()->zf - z() ) )
    {
      EndPoint = 1;
    }else{
      EndPoint = 0;
    }
    updateARTE( iErr, EndPoint ); 
}

// update RSEG bank by RcSegPoint information
void RcSegPoint::updateARTE( int & iErr, int EndPoint ) const
{
  iErr = 0;
  ArtePointer<RSEG> rseg = link();
  if( rseg== 0 || rseg.index() > rseg.nrow() )
    {
      arteMsg << " MARPLE: updateARTE: RSEG link " 
	      << rseg << " does not exist";
      arteMsg.allp();
      iErr = 1;
      return;
    }
  if( EndPoint )
    {
      static int idMess1 = 75673456;
      arteMsg << "MARPLE:  updateARTE: RcSegPoint is saved " 
	      << "as end point of RSEG";
      arteMsg.allp(idMess1,1);
      rseg->xe  = x();
      rseg->ye  = y() ;
      rseg->ze  = z() ;
      rseg->txe = tx() ;
      rseg->tye = ty();
      rseg->pe  = pe();
      for( int i=0; i<5; i++){
	for(int j=i; j<5; j++){
	  rseg->cve[c_matrix_pack[i][j]] = cov()[i][j]; 
	  // start numbering from [0][0] as in struct RSEG
	}
      }
    }else{
      static int idMess2 = 342873456;
      arteMsg << "MARPLE:  updateARTE: RcSegPoint is saved " 
	      << "as first point of RSEG";
      arteMsg.allp(idMess2,1);
      rseg->xf  = x();
      rseg->yf  = y() ;
      rseg->zf  = z() ;
      rseg->txf = tx() ;
      rseg->tyf = ty();
      rseg->pf  = pe();
      for( int i=0; i<5; i++){
	for(int j=i; j<5; j++){
	  rseg->cvf[c_matrix_pack[i][j]] = cov()[i][j]; 
	  // start numbering from [0][0] as in struct RSEG
	}
      }
    }
}

ostream & operator << (ostream & os, const RcSegPoint &rp)
{
  //os << static_cast<RcPoint&>(rp) ;
  const RcPoint * ptPoint = &rp;
  os << *ptPoint;
  os << "            RSEG link: " << rp.link() << endl ;
  return os ;
}
