#ifndef _RCCLUSPOINT_HH_
#define _RCCLUSPOINT_HH_
//**********************************************************************/
// Created: 13.08.97,   Olya Igonkina
//
// MARPLE class of Points for Reconstruction which have link to RCLUS bank
//
//**********************************************************************/

#include <iostream.h>
#include "marple/RcPoint.hh"

#include "arte/Arte.hh"
#include "arte/ArteMessage.hh"
#include "auto/RCCL.hh"

/** class with link to RCCL bank.
    Class is inherited from RcPoint 
  */
class RcClusPoint : public RcPoint
{
public:
  ///                 default constructor
  inline RcClusPoint();

  ///                 constructor with link to RCCL bank
  inline RcClusPoint( const RcPoint &rp, 
		     const ArtePointer <RCCL> rClusLink = 0 );
  /// constructor from RCCL bank entry, if any diagonal element of covariant matrix equal 0 it is set to infinity
  RcClusPoint( const ArtePointer <RCCL> rClusLink);

  ///                 copy constructor
  inline RcClusPoint( const RcClusPoint &rp ); 

  ///                 destructor
  inline ~RcClusPoint(); 

  ///                 returns link to RCCL table
  inline ArtePointer <RCCL>  link() const; 

  ///                 set link to RCCL table
  inline void  setlink( const ArtePointer <RCCL> rClusLink = 0 );
  
  ///                 returns relation with MTRA 
  vector<ArtePointer<MTRA> > mtraList() const; 

  ///                 returns relation with HITB 
  vector<ArtePointer<HITC> > hitcList() const; 

  /// update RCCL bank by RcClusPoint information
  void updateARTE( int & iErr) const;

  /// estimate the incident angle using target information
  void estimateAngles( HepDouble tarX, HepDouble tarY, 
		       HepDouble tarZ,
		       HepDouble covTarXX, HepDouble covTarYY, 
		       HepDouble covTarXY,
		       int nIter, 
		       int & iErr );
  /// estimate the incident angle using GTAR information
  void estimateAngles( int nIter, 
		       int & iErr );

  ///                 assignment operator
  inline RcClusPoint & operator = (const RcClusPoint &rp);
  ///                 assignment operator
  inline RcClusPoint & operator = (const RcPoint &rp); 

private:
  ///                 ArtePointer to RCCL bank
  ArtePointer  <RCCL> fClusLink;
};


//                   ostream operator to print RcClusPoint information
ostream & operator << (ostream &, const RcClusPoint &);

#include "marple/RcClusPoint.icc"

#endif

