//**********************************************************************/
// Created: 23.08.97,   Olya Igonkina   for   MARPLE package
//
// implementation of RcMatchBuilder class
//
//**********************************************************************/

#include "marple/RcPoint.hh"
#include "marple/RcMatchPoint.hh"
#include "marple/RcMatchConst.h"
#include "auto/EVHD.hh"
#include "auto/MTRA.hh"
#include "auto/MTRX.hh"
#include "auto/TOPP.hh"
#include "auto/TOPX.hh"
#include "arte/ArteMessage.hh"

#include "marple/RcHbook.hh"
#include "marple/RcTuples.h"

#include "marple/RcMatchBuilder.hh"

template <class PointLeft, class PointRight>
RcMatchBuilder<PointLeft,PointRight >::RcMatchBuilder(  
                                       vector<PointLeft *> vecLeft,
				       vector<PointRight *> vecRight,
				       RcMatchConst & rcConst)
  :  fvecLeft(), fvecRight(), fmatchList(), fptRcConst(0),
     fNrealPair(0), fNbad(0), fNlost(0)

{
  addLeft(  vecLeft );
  addRight( vecRight);
  setConst( rcConst );
  doMatch();
}

template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::doMatch()
{
  //**********************************************************************/
  // create a list of matching points with chi2 
  //**********************************************************************/
  fNrealPair = 0;

 if( ptToMatchConst()->use_pair_cut )
   {
     for( unsigned int iLeft=0; iLeft < vectorLeft().size(); iLeft++)
       vectorLeft()[iLeft]->setmark(ptToMatchConst()->n_pair_left);  
     for( unsigned int iRight=0; iRight < vectorRight().size(); iRight++)
       vectorRight()[iRight]->setmark(ptToMatchConst()->n_pair_right);
   }

  for( unsigned int iLeft=0; iLeft < vectorLeft().size(); iLeft++)
    {
      for( unsigned int iRight=0; iRight < vectorRight().size(); iRight++)
        {
	  if( RcMtraOk( (*vectorLeft()[ iLeft ]).mtraList(), 
			(*vectorRight()[iRight]).mtraList() ) )
	    {   fNrealPair++;   }
          //*************************************************/
          // check combination 
          int takeCombination = 1;
          // in case of delta_cut is used, save pairs passed delta_cut
          if( (*ptToMatchConst()).use_delta_cut &&
              !(*ptToMatchConst()).delta_cut_ok( *vectorLeft()[  iLeft],
						 *vectorRight()[ iRight]))
                { takeCombination = 0 ; }


          // save combination passed cuts
          if( takeCombination )
            {
              fmatchList.push_back( new RcMatchPoint( *vectorLeft()[iLeft],
						      *vectorRight()[iRight]));
              if(  matchList().back()->mark() || 
		   (*fmatchList.back()).chi2() > (*ptToMatchConst()).chi2 )
                { 
                  delete fmatchList.back();
                  fmatchList.pop_back(); //remove points with big chi2
                }
            }
	}
    }	
  
  //**********************************************************************/
  // sort vector of RcMatchPoints by chi2() 
  //**********************************************************************/
  sort( fmatchList.begin(), fmatchList.end(), comp_chi2 );

  //**********************************************************************/
  // check the number of combinations with given RcPoints
  //**********************************************************************/

  int goodCombCut = 0;

  for( unsigned int iMatch=0; iMatch < fmatchList.size(); iMatch++ )
    {
      int checkOk = 1;
      PointLeft* ptLeft   =  leftOrigin(iMatch);
      PointRight* ptRight =  rightOrigin(iMatch);

      if( (*ptToMatchConst()).use_pair_cut ) 
	{
	  if( !(ptToMatchConst()->pair_cut_ok( *ptLeft  )) ) checkOk = 0 ;
	  if( !(ptToMatchConst()->pair_cut_ok( *ptRight )) ) checkOk = 0 ;
          if( !checkOk )
            { 
  	      delete( fmatchList[iMatch]);
              fmatchList.erase( &fmatchList[iMatch]) ; 
	      iMatch--;
            }else{
	      // mark origin
	      int mark = ptLeft->mark();
	      mark--;
	      ptLeft->setmark(mark );
	      mark = ptRight->mark();
	      mark--;
	      ptRight->setmark(mark );
	    }
	}
      
      // calculate MTRA statistics
      int mtraOk = 1;
      if( !RcMtraOk( ptLeft->mtraList(), ptRight->mtraList() ) )
	{  mtraOk = 0; }
      if( checkOk && mtraOk )
	{ goodCombCut++ ; }     
    }
  fNbad = matchList().size() - goodCombCut;
  fNlost = fNrealPair - goodCombCut ;
  
}

//--------------------------------------------------------------
template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::refitLeftFound(int & iErr)
{
  iErr = 0;
  int iErr1 = 0;
  for( unsigned int iMatch=0; iMatch< matchList().size(); iMatch++)
    {
      RcPoint * ptLeft = leftOrigin(iMatch);
      *ptLeft = *matchList()[iMatch];
      ptLeft->refit( iErr1);
      if( iErr1 ) iErr = iErr1;
    }
}

template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::refitRightFound(int & iErr)
{
  iErr = 0;
  int iErr1 = 0;
  for( unsigned int iMatch=0; iMatch< matchList().size(); iMatch++)
    {
      RcPoint * ptRight = rightOrigin(iMatch);
      *ptRight = *matchList()[iMatch];
      ptRight->refit( iErr1);
      if( iErr1 ) iErr = iErr1;
    }
}

//--------------------------------------------------------------
template <class PointL, class PointR>
void RcMatchBuilder<PointL,PointR >::updateARTELeftFound(int & iErr) const
{
  iErr = 0;
  int iErr1 = 0;
  for( unsigned int iMatch=0; iMatch< matchList().size(); iMatch++)
    {
      leftOrigin(iMatch)->updateARTE( iErr1);
      if( iErr1 ) iErr = iErr1;
    }
}

template <class PointL, class PointR>
void RcMatchBuilder<PointL,PointR >::updateARTERightFound(int & iErr) const
{
  iErr = 0;
  int iErr1 = 0;
  for( unsigned int iMatch=0; iMatch< matchList().size(); iMatch++)
    {
      rightOrigin(iMatch)->updateARTE( iErr1);
      if( iErr1 ) iErr = iErr1;
    }
}
//--------------------------------------------------------------
template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::stat( int iLong ) const
{
  unsigned int i;
  switch (iLong)
    {
    case 2 :
      arteMsg << "Left Points :" << vectorLeft().size(); 
      arteMsg.allp();
      for( i=0; i< vectorLeft().size(); i++)
        {  cout << *(vectorLeft()[i]) ; }
      
      arteMsg << "Right Points :" << vectorRight().size(); 
      arteMsg.allp();
      for( i=0; i< vectorRight().size(); i++)
        {  cout << *(vectorRight()[i]) ;  }
      
      arteMsg << "Match Points : "<< matchList().size(); 
      arteMsg.allp();
      for( i=0; i< matchList().size(); i++)
        {  cout << *(matchList()[i]) ;  }

    case 1 :
      cout << "MARPLE: *** constants used for matching ****" << endl 
	   << *ptToMatchConst(); 

    case 0 :
    default :     
      arteMsg << "*********** result of matching *******";
      arteMsg.allp();
      arteMsg << "Left Points      : " << vectorLeft().size(); 
      arteMsg.allp();
      arteMsg << "Right Points     : " << vectorRight().size(); 
      arteMsg.allp();
      arteMsg << "All pairs with the same MTRA: " << NrealPair(); 
      arteMsg.allp();
      arteMsg << "Match Points     : " << matchList().size();
      arteMsg.allp();
      arteMsg << "Bad  pairs       : " << Nbad() ; 
      arteMsg.allp();
      arteMsg << "Lost pairs       : " << Nlost(); 
      arteMsg.allp();
    }
}

//--------------------------------------------------------------
template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::fillDeltas(int iNtup) const
{
  for( unsigned int iMatch=0; iMatch < matchList().size(); iMatch++ )
    {
      RcMatchPoint * ptMatchPoint = matchList()[iMatch];
      FORTRAN_COMMON(deltas).chi2 = (*ptMatchPoint).chi2();
      
      RcPoint * ptRp1 = leftOrigin(iMatch);
      RcPoint * ptRp2 = rightOrigin(iMatch);
      
      FORTRAN_COMMON(deltas).deltaX  = (*ptRp1).x()  - (*ptRp2).x() ;
      FORTRAN_COMMON(deltas).deltaY  = (*ptRp1).y()  - (*ptRp2).y() ;
      FORTRAN_COMMON(deltas).deltaTX = (*ptRp1).tx() - (*ptRp2).tx();
      FORTRAN_COMMON(deltas).deltaTY = (*ptRp1).ty() - (*ptRp2).ty();
      FORTRAN_COMMON(deltas).deltaPE = (*ptRp1).pe() - (*ptRp2).pe();
      FORTRAN_COMMON(deltas).flag    =
	RcMtraOk( (*ptRp1).mtraList(), (*ptRp2).mtraList() );
      
      int ipart;
      ArtePointer<MTRA> mtra;
      if( ((*ptRp1).mtraList()).size() !=0)
	{
	  mtra  = (*ptRp1).mtraList()[0];
	  ipart = ArtePointer<TOPP>(mtra->topp)->lund;
	  FORTRAN_COMMON(deltas).ipart1  = ipart;
	  mtra  = ArtePointer<MTRX>(mtra)->btra;
	  ipart = ArtePointer<TOPP>(mtra->topp)->lund;
	  FORTRAN_COMMON(deltas).imoth1  = ipart;
	}else{
	  FORTRAN_COMMON(deltas).ipart1  = 0;
	  FORTRAN_COMMON(deltas).imoth1  = 0;
	}
      
      if( ((*ptRp2).mtraList()).size() != 0 )
	{
	  mtra  = (*ptRp2).mtraList()[0];
	  ipart = ArtePointer<TOPP>(mtra->topp)->lund;
	  FORTRAN_COMMON(deltas).ipart2  = ipart;
	  mtra  = ArtePointer<MTRX>(mtra)->btra;
	  ipart = ArtePointer<TOPP>(mtra->topp)->lund;
	  FORTRAN_COMMON(deltas).imoth2  = ipart;
	}else{
	  FORTRAN_COMMON(deltas).ipart2  = 0;
	  FORTRAN_COMMON(deltas).imoth2  = 0;
	}
      if( iNtup ) FORTRAN_ROUTINE( hfnt)(&iNtup); 
    } 
}

//--------------------------------------------------------------

template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::fillMatch( int iNtup) const
{
  ArtePointer<EVHD> event(1);
  FORTRAN_COMMON(matchnt).Nevt   = event->evt;
  FORTRAN_COMMON(matchnt).Nleft  = vectorLeft().size();
  FORTRAN_COMMON(matchnt).Nright = vectorRight().size();
  FORTRAN_COMMON(matchnt).Nmatch = matchList().size();
  FORTRAN_COMMON(matchnt).Nreal  = NrealPair();
  FORTRAN_COMMON(matchnt).Nbad   = Nbad();
  FORTRAN_COMMON(matchnt).Nlost  = Nlost() ;
  if( matchList().size() != 0 )
    {
      FORTRAN_COMMON(matchnt).Rbad = float(Nbad()) / float(matchList().size());
    }else{
      FORTRAN_COMMON(matchnt).Rbad = 0;
    }
  if( NrealPair() != 0)
    {
      FORTRAN_COMMON(matchnt).Rlost = float( Nlost()) / float( NrealPair());
    }else{
      FORTRAN_COMMON(matchnt).Rlost = 0;
    }
  if( iNtup )  FORTRAN_ROUTINE( hfnt)(&iNtup); 
}

//********************************************************/
// save information into ARTE banks
//********************************************************/

template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::saveRTRA() const
{}
                  
template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::saveRSEG() const
{}

template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::saveRCCL() const
{}

template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::saveRelation( int & iErr) const
{
  if( !matchList().size() ){ iErr = 0; return; }
  iErr = 1;
  for(unsigned int iMatch=0; iMatch < matchList().size(); iMatch++)
    {      
      int iErr1=1;
      saveRelation(leftOrigin(iMatch),
		   rightOrigin(iMatch), 
		   iErr1); 
      if( !iErr1 ){ iErr = 0; }
    }
}

