//--------------------------------------------------------------------
// Created: 02.09.97 by   Olya Igonkina


#include "marple/marple.hh"
#include "marple/marpleobjects.hh"

#include "arte/RSEG_cmp.hh"
#include "arte/ArteMessage.hh"
  
/* MARPLE function for matching of two sets of segment 
   [Rse1,Rs2] and [Rsf1,Rsf2]
  The end points of all RSEG entries irseg, Rse1~$<=$~irseg~$<=$Rse2,
  irseg\_fit(irseg)~$ >= 0$
  are passed as a left set of objects to be matched
  while the first points of RSEG entries 
  irseg2, Rsf1~$<=$~irseg2~$<=$Rsf2, irseg\_fit(irseg2)~$ >= 0$
  are passed as a right set of objects.
  The region [Rse1, Rse2] and [Rsf1, Rsf2] should not overlap.
  The parameters of the match should
  be optimized for each case and passed with {\tt MAPUTCONST} subroutine 
  */

extern "C"  
void FORTRAN_ROUTINE(matchrsersf)(int * Rse1, int * Rse2, 
				  int * Rsf1, int * Rsf2) 
{
  //--------------------------------------------------------------------
  // put constants to the MatchBuilder
  //--------------------------------------------------------------------
  if( SegmentsMatch.ptToMatchConst() == 0 )
    {
      arteMsg << "MARPLE: MatchRseRsf: no parameters of match defined, " 
	      << "used one for MATCHSVDMT " ;
      arteMsg.allp();
      SegmentsMatch.setConst(rcSvdMtConst);
    }

  if( SegmentsMatch.vectorLeft().size()  || 
      SegmentsMatch.vectorRight().size() ||
      SegmentsMatch.matchList().size() )
    {
      arteMsg << "MARPLE: MatchRseRsf: MatchBuilder is not empty, "
	      << "clear it before using";
      arteMsg.allp();
      arteMsg << "MARPLE reminder: clear information in time,"
	      <<" do not produce garbage!";
      arteMsg.allp();
      SegmentsMatch.deleteAll(); 
    }

  //--------------------------------------------------------------------
  // create vectors of segments 
  //--------------------------------------------------------------------
  
  int iErr;
  for( ArteTableIterator <RSEG> irseg;
        irseg.valid(); irseg.next() )
    {
      if( irseg->fit < 0 ) continue;
      if( irseg.index() >= *Rse1 && irseg.index() <= *Rse2 &&
	  SegmentsMatch.ptToMatchConst()->geomAccept(HepDouble(irseg->ze)) )
	{

	  RcSegPoint * ptToPoint = new RcSegPoint( irseg , 1 );
	  ptToPoint->prolongate( SegmentsMatch.ptToMatchConst()->z_plane,
				 iErr);
	  if( iErr ){ delete ptToPoint; ptToPoint = 0; continue; }
	  SegmentsMatch.addPointLeft( *ptToPoint);
	  ptToPoint = 0;

	}else  if(irseg.index() >= *Rsf1 && irseg.index() <= *Rsf2 ){

	  if( !SegmentsMatch.ptToMatchConst()->
	      geomAccept(HepDouble(irseg->zf))) continue;
	  RcSegPoint * ptToPoint = new RcSegPoint( irseg , 0 );
	  ptToPoint->prolongate( SegmentsMatch.ptToMatchConst()->z_plane,
				 iErr);
	  if( iErr ){ delete ptToPoint; ptToPoint = 0; continue; }
	  SegmentsMatch.addPointRight( *ptToPoint);
	  ptToPoint = 0;
	}
    }
  
  //--------------------------------------------------------------------
  // do Matching 
  //--------------------------------------------------------------------
  SegmentsMatch.doMatch();
  
  //--------------------------------------------------------------------
  // print result of matching
  //--------------------------------------------------------------------
  static int firstSegments = 1;
  if( firstSegments )
    {
      cout << *SegmentsMatch.ptToMatchConst();
      firstSegments = 0;
    }
  SegmentsMatch.stat(0);
  
  //SegmentsMatch.fillDeltas(2000);
  //SegmentsMatch.fillMatch( 1000);
  
  //--------------------------------------------------------------------
  // save found relation to the table
  //--------------------------------------------------------------------

  SegmentsMatch.saveRelation(iErr);
  if( iErr )
    { arteMsg << "MARPLE: some relations were not saved "; arteMsg.allp();}

  //    refit information
  //SegmentsMatch.refitLeftFound(       iErr);
  //SegmentsMatch.refitRightFound(      iErr);
  //    and update RSEG tables
  //SegmentsMatch.updateARTELeftFound(  iErr);
  //SegmentsMatch.updateARTERightFound( iErr);


  //--------------------------------------------------------------------
  // clean everything
  //--------------------------------------------------------------------
  SegmentsMatch.deleteAll();  
}


