//--------------------------------------------------------------------
// Created: 02.09.97 by   Olya Igonkina

// MARPLE function for matching SUSI and Ranger

//--------------------------------------------------------------------

#include "marple/marple.hh"
#include "marple/RcMatchBuilder.hh"

#include "arte/RSEG_cmp.hh"
#include "arte/ArteMessage.hh"

//definition of constants for SUSI-RANGER matching ( see \Ref{RcMatchConst})
// flag, z_plane, z_min, z_max, use_pair_cut, n_pair_left, n_pair_right, 
// use_delta_cut, delta_x, delta_y, delta_tx, delta_ty, delta_pe, chi2
RcMatchConst rcSvdMtConst = { 
  11110,     160.,   30.,  500.,      1,          1,            2,
            1,      5.,      5.,      0.1,      0.1,   1.e+10,   300.,  
};

//--------------------------------------------------------------------
//   SUSI - RANGER matching
//--------------------------------------------------------------------
  
extern "C"  
void FORTRAN_ROUTINE(matchsvdmt)() 
{
  RcMatchBuilder< RcSegPoint, RcSegPoint> SvdMtMatch;

  //--------------------------------------------------------------------
  // put constants to the MatchBuilder
  //--------------------------------------------------------------------
  SvdMtMatch.setConst(rcSvdMtConst);

  //--------------------------------------------------------------------
  // create vector of segments from SUSI
  //--------------------------------------------------------------------

  int iErr =0;
  for( ArteTableIterator <RSEG> irseg;
       irseg.valid(); irseg.next() )
    {
      if( irseg->fit >= 0 && irseg->cmp & Rsegc::vxd && 
	  SvdMtMatch.ptToMatchConst()->geomAccept(HepDouble(irseg->ze)) )
	{ 	  
	  RcSegPoint * ptToPoint = new RcSegPoint( irseg , 1); // endPoint
	  // prolongate to the plane of matching
	  (*ptToPoint).prolongate( SvdMtMatch.ptToMatchConst()->z_plane, iErr);
	  if( iErr ) 
	    { 
	      delete ptToPoint; 
	    }else{
	      SvdMtMatch.addPointLeft( *ptToPoint);
	    }
	  ptToPoint = 0;
	}
    }
  //--------------------------------------------------------------------
  // create vector of segments from Ranger
  //--------------------------------------------------------------------

  
  for( ArteTableIterator <RSEG> irseg2; 
       irseg2.valid(); irseg2.next() )
    {
      if( irseg2->fit >= 0 && irseg2->cmp & Rsegc::magt && 
	  SvdMtMatch.ptToMatchConst()->geomAccept(HepDouble(irseg2->zf)) )
	{
	  RcSegPoint * ptToPoint = new RcSegPoint( irseg2,0 );// firstPoint
	  (*ptToPoint).prolongate( SvdMtMatch.ptToMatchConst()->z_plane, iErr);
	  if( iErr ) 
	    {
	      delete ptToPoint;
	    }else{
	      SvdMtMatch.addPointRight( *ptToPoint);
	    }
	  ptToPoint = 0;	      
	}
    }  
  
  //--------------------------------------------------------------------
  // do Matching 
  //--------------------------------------------------------------------
  SvdMtMatch.doMatch();

  //--------------------------------------------------------------------
  // print result of matching
  //--------------------------------------------------------------------
  static int firstSVDMT = 1;
  if( firstSVDMT )
    {
      cout << *SvdMtMatch.ptToMatchConst();
      firstSVDMT = 0;
    }
  SvdMtMatch.stat(0);
  

  //SvdMtMatch.fillMatch(1000);
  //SvdMtMatch.fillDeltas(2000);

  //--------------------------------------------------------------------
  // save found relation to the table
  //--------------------------------------------------------------------

  SvdMtMatch.saveRelation(iErr);
  if( iErr )
    { arteMsg << "MARPLE: some relations were not saved "; arteMsg.allp();}

  //    refit information
  //SvdMtMatch.refitLeftFound(       iErr);
  //SvdMtMatch.refitRightFound(      iErr);
  //    and update RSEG tables
  //SvdMtMatch.updateARTELeftFound(  iErr);
  //SvdMtMatch.updateARTERightFound( iErr);

  //--------------------------------------------------------------------
  // clean everything
  //--------------------------------------------------------------------
  SvdMtMatch.deleteAll();  
}

