#include "rich/FED.h"
       
#include <sys/time.h>
#include <proto.h>

// -------------------------------------------------------------------------

int FEDBuffer::write(FILE *fp) {
  int err=(fwrite(&buffer,sizeof(buffer),1,fp)!=1);
  return err;
  }

void FEDBuffer::dump(FILE *fp) {
  unsigned short int *value=&buffer.WC;
  unsigned int i;
  fprintf(fp,"\n");
  for (i=10;i-10<sizeof(buffer)/sizeof(unsigned short int);i++) {
    fprintf(fp,"%5.4x",*value);
    if(i%16==15) fprintf(fp,"\n");
    value++;
    }
  fprintf(fp,"\n");
  fflush(fp);
  }
  
void FEDBuffer::vtdump() {
  static int ifl=1;
  if (ifl) {
    printf("\033[H");
    printf("\033[J");
    ifl=0;
    }
  else printf("\033[H");
  dump(stdout);
  printf("\n");
  printf("\033[H");
  }

int FEDBuffer::ZeroData() {
  int i;
  int nc=sizeof(buffer.data)/NumDaughter();
  for (i=0;i<NumDaughter();i++) {
    if (DaughterIsConnected(i)) { 
      int j;
      int j0=i*nc;
      for (j=0;j<nc;j++) {
        if (buffer.data[j0+j]!=0) return 0;
        }
      }
    }
  return 1;
  }
  
int operator==(const FEDBuffer &x,const FEDBuffer &y) { 
  int i;
//  for (i=0;i<x.NumConnector();i++) {
//    if (x.buffer.data[i]!=y.buffer.data[i]&&x.ASDWorks(i)) return 0;
//    }
//  return 1;
  return (memcmp(&x.buffer.data[i],&y.buffer.data[i],sizeof(x.buffer.data[i]))!=0);
  }
  
int operator!=(const FEDBuffer &x,const FEDBuffer &y) {
  return !(x==y);
  }

void FEDBuffer::operator=(const FEDBuffer& x) { 
  memcpy(this,&x,sizeof(FEDBuffer));
  }

FEDBuffer& FEDBuffer::operator|=(const FEDBuffer &x)  {
  const unsigned short int *bx=&x.buffer.WC;
  unsigned short int *b =&buffer.WC;
  unsigned int i; 
  for (i=0;i<sizeof(buffer)/sizeof(short int);i++) *b++|=*bx++;
  return *this;
  }


// -------------------------------------------------------------------------

OfflineFEDBuffer::OfflineFEDBuffer() : FEDBuffer() {
  byteswap=0;
  buffer.WC=0x0090;
  buffer.R_ID=0x0002;
  }
  
int OfflineFEDBuffer::read(FILE *fp) {
  //  int fn=FLTEventNo();
  if (fread(&buffer,sizeof(buffer),1,fp)!=1) return 0;
  if (byteswap) RICHswap(&buffer.WC,sizeof(buffer));
  /*
  if (FLTEventNo()<fn) fltcount+=(((unsigned short)0xFFFF)-fn+FLTEventNo()+1);
  else fltcount+=(FLTEventNo()-fn);
  */
  fltcount=FLTEventNo();
  return 1;
  }

int OfflineFEDBuffer::wind(FILE* fp,int numevent,unsigned long start) {
  long pos=ftell(fp);
  long size=(numevent-1)*sizeof(buffer);
  if (pos+size<start) fseek(fp,start,SEEK_SET);
  else fseek(fp,size,SEEK_CUR);
  return (ftell(fp)-pos)/sizeof(buffer);
  }
  
// ----------------------------------------------------------------------

FEDHits::FEDHits(FEDBuffer* f,ChannelInfo& cc)  { 
  fed=f;
  c=&cc;
  bitmask=1;
  bit=-1;
  }

int FEDHits::Search() {
  bit++;
  bitmask=bitmask>>1;
  if (bitmask==0) {
    bitmask=0x8000;
    int connector=bit/16;
    while (connector<NumConnector()&&!ASDWorks(connector)) {
      bit+=16; 
      connector++;
      }
    if (connector==NumConnector()) return 0;
    }
  int connector=bit/16;
  while ((fed->Data(connector)&bitmask)==0||!ChannelWorks(bit)) {
    bitmask=bitmask>>1;
    bit++;
    if (bitmask==0) {
      bitmask=0x8000;      
      while ((connector=bit/16)<NumConnector()
             &&!ASDWorks(connector)) bit+=16;
      if (connector==NumConnector()) return 0;
      }
    }
  return 1;
  }

  
// ----------------------------------------------------------------------

int RunHeader::write(FILE *fp) {
  if (!(fwrite(&runheader,sizeof(runheader),1,fp)==1)) return 0;
  }
  
int RunHeader::read(FILE *fp) {
  if (!(fread(&runheader,sizeof(runheader),1,fp)==1)) return 0;
  if (byteswap) {
    runheader.wc            =RICHswap(runheader.wc);    
    runheader.rid           =RICHswap(runheader.rid);    
    runheader.RunNo         =RICHswap(runheader.RunNo);
    runheader.RunFlag       =RICHswap(runheader.RunFlag);
    runheader.duration      =RICHswap(runheader.duration);
    runheader.numtrigger    =RICHswap(runheader.numtrigger);
    runheader.numeventinfile=RICHswap(runheader.numeventinfile);
    runheader.year          =RICHswap(runheader.year);
    runheader.month         =RICHswap(runheader.month);
    runheader.mday          =RICHswap(runheader.mday);
    runheader.wday          =RICHswap(runheader.wday);
    runheader.hour          =RICHswap(runheader.hour);
    runheader.minutes       =RICHswap(runheader.minutes);
    runheader.seconds       =RICHswap(runheader.seconds);
    runheader.millisec      =RICHswap(runheader.millisec);
    int i;
    for (i=0;i<7;i++) 
      runheader.spare[i]    =RICHswap(runheader.spare[i]);
    for (i=0;i<sizeof(runheader.morespare)/2;i++) 
      runheader.morespare[i]=RICHswap(runheader.morespare[i]);
    }
  }
  
int RunHeader::printheader(FILE *fp) {
  fprintf(fp,"WordCount:                      %04X\n",runheader.wc);
  fprintf(fp,"RecordID:                       %04X\n",runheader.rid);
  fprintf(fp,"Run Number:                     %d\n",RunNo());
  fprintf(fp,"Run Flag:                       %d\n",RunFlag());
  fprintf(fp,"Run Duration:                   %7.3f\n",RunDuration()/1000.);
  fprintf(fp,"Start Time:                     %4d/%02d/%02d %02d:%02d.%02d,%03d\n",
     Year(),Month(),Day(),Hour(),Minutes(),Seconds(),MilliSeconds());
  fprintf(fp,"Number of triggers:             %d\n",NumTrigger());
  fprintf(fp,"Number of events in the file:   %d\n",EventsInFile());
  fprintf(fp,"HV-M16a                         %7.3f\n",HV_M16a()/1000.);
  fprintf(fp,"HV-M16b                         %7.3f\n",HV_M16b()/1000.);
  fprintf(fp,"HV-M4a                          %7.3f\n",HV_M4a()/1000.);
  fprintf(fp,"HV-M4b                          %7.3f\n",HV_M4b()/1000.);
  fprintf(fp,"LV-plus                         %7.3f\n",LV_plus()/1000.);
  fprintf(fp,"LV-minus                        %7.3f\n",LV_minus()/1000.);
  fprintf(fp,"Threshold                       %7.3f\n",Threshold()/1000.);
  fprintf(fp,"FileName:                       %s\n",FileName());
  fprintf(fp,"Person to ask :                 %s\n",UserName());
  fprintf(fp,"Comment:                        %s\n",Comment());
  }

// -------------------------------------------------------------------------------------
int RunInfo::write(FILE *fp) {
  if (!(fwrite(&runinfo,sizeof(runinfo),1,fp)==1)) return 0;
  }
  
int RunInfo::read(FILE *fp) {
  if (!(fread(&runinfo,sizeof(runinfo),1,fp)==1)) return 0;
  if (byteswap) {
    runinfo.wc               =RICHswap(runinfo.wc);
    runinfo.rid              =RICHswap(runinfo.rid);
    runinfo.numevent         =RICHswap(runinfo.numevent);
    runinfo.EventNo          =RICHswap(runinfo.EventNo);
    runinfo.duration         =RICHswap(runinfo.duration);
    }
  return 1;
  }
  



