#include "rich/RICHFED.h"
#include "rich/PhotonDetection.h"

// --------------------------------------------------------------------

WiremapM16::WiremapM16(ScriptFile& f) {
  int pm,channel,asd,asdchan;
  int numchan=0;
  char s[256];
  while (f.getline(s)) {
    if (sscanf(s,"%d %d %d %d\n",&pm,&channel,&asd,&asdchan)==4) {
      pmt(asd,asdchan)=pm;
      pmtChannel(asd,asdchan)=channel;
      numchan++;
      }
    }
  if (numchan!=64) {
    fprintf(stderr,"M16Base: missing channels: check file m16wiremap\n");
    exit(1);
    }
  }
  
// --------------------------------------------------------------------

WiremapM4::WiremapM4(ScriptFile& f) {
  int pm,channel,asd,asdchan;
  int numchan=0;
  char s[256];
  while (f.getline(s)) {
    if (sscanf(s,"%d %d %d %d\n",&pm,&channel,&asdchan)==3) {
      pmt(asdchan)=pm;
      pmtChannel(asdchan)=channel;
      numchan++;
      }
    }
  if (numchan!=16) {
    fprintf(stderr,"M4Base: missing channels: check file m4wiremap\n");
    exit(1);
    }
  }

// --------------------------------------------------------------------

M16ASD::M16ASD(char *s) {
  if (sscanf(s,"%d %d %d %d %d %d %d",
	     &asdid,&sm,&r,&c,&motherID,&daughterID,&connector)!=7) {
    fprintf(stderr,"M16ASD: Error scanning %s\n",s);
    exit(1);
    }
  }
 
void M16ASD::print(FILE *fp) {
  fprintf(fp,"M16 %1d %2d %2d %2d %4d %1d %1d",
            asdid,sm,r,c,motherID,daughterID,connector);
  }

// --------------------------------------------------------------------

M4ASD::M4ASD(char *s) {
  if (sscanf(s,"%d %d %d %d %d %d",
	      &sm,&r,&c,&motherID,&daughterID,&connector)!=6) {
    fprintf(stderr,"M4ASD: Error scanning %s\n",s);
    exit(1);
    }
  }
 
void M4ASD::print(FILE *fp) {
  fprintf(fp,"M4 %1d %2d %2d %2d %4d %1d %1d",
            sm,r,c,motherID,daughterID,connector);
  }
  

// ----------------------------------------------------------------------

RICHFED::RICHFED(int mid,List<RICHASD> ASDList)  {
  if (!RICH_Detector) {
    fprintf(stderr,"RICHFED(int,List<RICHASD>):\n");
    fprintf(stderr,"   RICH_Detector not initialized. Cannot proceed.\n");
    exit(1);
    }
    
  motherid=mid; 
  int i;
  for (i=0;i<64;i++) asd[i]=0;
  RICHASD *a;
  int k=0;
  for (ASDList.newiterator();(a=ASDList.iterate());) {
    if (a->MotherID()==motherid) { 
      k++;
      asd[index(a->DaughterID(),a->Connector())]=a;
      }
    }
  int n=0;
  for (i=0;i<64;i++) if (asd[i]!=0) n++;
  if (n!=k) {
    fprintf(stderr,"RICHFED::RICHFED: MotherID=%d\n",mid);
    fprintf(stderr," ASD to Connector Relation not unique: %d %d\n",k,n);
    exit(1);
    }
  }
  
// -------------------------------------------------------------------------

RICHDAQ::RICHDAQ(char *path) {
  char fname[256];
  strcpy(fname,path);
  strcat(fname,"m16wiremap.dat");
  ScriptFile m16(fname);
  M16ASD::m16base=new WiremapM16(m16);

  strcpy(fname,path);
  strcat(fname,"m4wiremap.dat");
  ScriptFile m4(fname);
  M4ASD::m4base= new WiremapM4(m4);
  
  strcpy(fname,path);
  strcat(fname,"ASD.dat");
  ScriptFile f(fname);
  char s[256],tag[256];
  while (f.getline(s)) {
    char *l=s;
    sscanf(l,"%s",tag);l+=f.itemlength(l,1);
    if (!strcmp(tag,"M16")) ASDList.append(new M16ASD(l));
    else if (!strcmp(tag,"M4")) ASDList.append(new M4ASD(l)); 
    else {
      fprintf(stderr,"RICHDAQ: Unknown Base type: %s\n",tag);
      exit(1);
      }
    }

  numchannel        =RICH_Detector->NumChannel();
  motherID          =new unsigned short[numchannel];
  daughter_connector=new unsigned char [numchannel];
  asdchannel_hit    =new unsigned char [numchannel];
  numhit            =new unsigned long [numchannel];
  if (!numhit) { // is enough to check, since it is last and largest array
    fprintf(stderr,"RICHDAQ: Out of memory: Memory allocation error.\n");
    exit(1);
    }
  ZeroNumHit();
  memset((char*)motherID          ,0,numchannel*sizeof(unsigned short));
  memset((char*)daughter_connector,0,numchannel*sizeof(unsigned char));
  memset((char*)asdchannel_hit    ,0,numchannel*sizeof(unsigned char));
   
  RICHDetector& pd=*RICH_Detector;
  int *mid=0;
  nummother=0;
  RICHASD *asd;
  for (ASDList.newiterator();(asd=ASDList.iterate());) {
    int i;
    for (i=0;i<nummother;i++) 
      if (mid[i]==asd->MotherID()) break;
    if (i==nummother) {
      nummother++;
      mid=(int*)realloc(mid,sizeof(int)*nummother);
      mid[nummother-1]=asd->MotherID();
      }
    int s=asd->SuperModule();
    for (i=0;i<asd->numchannel();i++) {
      int r=asd->Row(i);
      int c=asd->Col(i);
      int p=asd->PMTChannel(Channel(i));
      if (!pd.ChannelExist(s,r,c,p)) {
          fprintf(stderr,"RICHFED(int,List<RICHASD>):\n");
          fprintf(stderr,"  Initialisation error: \n");
          fprintf(stderr,"  RICHFED requires channel without pmt: %2d %2d %2d %2d\n",
                s,r,c,p);
          exit(1);
          }

      char flag=4;
      if (pd.ChannelIsHot()) flag=2;
      else if (pd.ChannelIsDead()) flag=0;
      else if (pd.ChannelWorks()) flag=1;
      asd->Flag(i)=flag;

      int ci=pd.ChannelIndex();
      SetMotherID  (ci,asd->MotherID());
      SetDaughterID(ci,asd->DaughterID());
      SetConnector (ci,asd->Connector());
      }
    }

  FED=new RICHFED*[nummother];
  int i;
  for (i=0;i<nummother;i++) {
    FED[i]=new RICHFED(mid[i],ASDList);
    }

  char *datafile=getenv("RICH_DATAFILE");
  if (!datafile) {
    printf("Set environement variable RICH_DATAFILE before starting the program\n");
    exit(1);
    }

  if (!(fp=fopen(datafile,"rb"))) {
    fprintf(stderr,"RICHDAQ: Cannot open data file: %s\n",datafile);
    exit(0);
    }
  unsigned short wc;
  if (fread(&wc,sizeof(wc),1,fp)!=1) {
    fprintf(stderr,"RICHDAQ: Error during reading data file: %s\n",datafile);
    exit(0);
    }
  fseek(fp,0,SEEK_SET);
  if (wc==0x0090) { }  // old formats
  else if (wc==0x9000) { fedbuffer.ByteSwap(); }
  else {
    unsigned long swapid;
    if (fread(&swapid,sizeof(unsigned long),1,fp)!=1) {
      fprintf(stderr,"RICHDAQ: Error during reading data file: %s\n",datafile);
      exit(0);
      }  
    if (swapid!=0x01020304) {
      byteswap=1;
      runheader.ByteSwap();
      fedbuffer.ByteSwap();
      }
    runheader.read(fp);
    }
  datastart=ftell(fp);
  MaxIndex=eventindex=0;
  }


int RICHDAQ::NextEvent() {
  int i;
  for (i=0;i<nummother;i++) {
    int ret;
    if ((ret=fedbuffer.read(fp))<=0) {
      if (ret==0) return 0;
      else {
        fprintf(stderr,"RICHDAQ::NextEvent: Error during read fed data\n");
        exit(1);
        }
      }
  
    if (i==0){
      eventno   =fedbuffer.NumTrigger();
      flteventno=fedbuffer.FLTEventNo();
      bx        =fedbuffer.BX();
      }
    else if (eventno!=fedbuffer.NumTrigger()) {
      fprintf(stderr,":Hits::NextEvent:  not enough FED's for event %d\n",
        eventno);
      exit(1);
      }
    
    if (eventindex==MaxIndex) MaxIndex++;
    eventindex++;

//fedbuffer.dump();
    RICHFED& rf=*RICH_DAQ->mother(fedbuffer.MotherID());
    FEDHits hit(&fedbuffer,rf);
    while (hit.Search()) {
      int i=hit.Bit();
      int s=rf.SuperModule(i);
      int r=rf.Row(i);
      int c=rf.Col(i);
      int p=rf.PMTChannel(i);
      if (RICH_Detector->ChannelExist(s,r,c,p)) {
        if (eventindex==MaxIndex){
           numhit[RICH_Detector->ChannelIndex()]++;
   	   } 
        SetHit(RICH_Detector->ChannelIndex(),1);
        }
      }    
    }
  return 1;
  }


int RICHDAQ::Wind(int numevent) {
  int k;
  if (eventindex+numevent>MaxIndex) {
    k=fedbuffer.wind(fp,MaxIndex-eventindex,datastart);
    eventindex+=k;
    for (;k<numevent;k++) { 
      if (!NextEvent()) break; 
      }
    }    
  else {
    k=fedbuffer.wind(fp,numevent,datastart);
    eventindex+=k;
    k+=NextEvent();
    }
  return k;
  }
  

// --------------------------------------------------------------------


RICHDAQ *RICH_DAQ=0;

WiremapM16* M16ASD::m16base=0;
WiremapM4*  M4ASD::m4base=0;


