#include "rich/PhotonDetection.h"
#include "rich/RICHFED.h"
#include "rich/vessel.h"

extern "C" {


void F77NAME(ricdeti)() {
  if (RICH_DAQ) return;

  char configdir[256];
  
  char* RICHconfigdir=getenv("RICH_CONFIGDIR");
  if (RICHconfigdir) {
    strcpy(configdir,RICHconfigdir);
    if (configdir[strlen(configdir)-1]!='/') strcat(configdir,"/");
    }
  else {
    char *hb=getenv("HB");
    char *hbvers=getenv("HBVERS");
    if (!hb||!hbvers) {
      fprintf(stderr,"RICINI:  $HB/$HBVERS is not defined. Abort.\n");
      exit(1);
      }
    strcpy(configdir,hb);
    strcat(configdir,"/");
    strcat(configdir,hbvers);
    strcat(configdir,"/geo/rich/");
    }
  printf(" === RICHDAQ configuration directory: %s\n",configdir);
  if (!RICH_Detector) RICH_Detector=new RICHDetector(configdir);
  RICH_DAQ     =new RICHDAQ(configdir);
  }


int F77NAME(nextevent)() {
  return RICH_DAQ->NextEvent();
  }


void F77NAME(newchannelloop)() {
  RICHDetector& pd=*RICH_Detector;
  pd.NewChannelLoop();
  }

int F77NAME(channelloop)(int *s,int *r,int *c,int *p) {
  RICHDetector& pd=*RICH_Detector;
  return pd.ChannelLoop(s,r,c,p);
  }

int F77NAME(pmnumchannel)() {
  RICHDetector& pd=*RICH_Detector;
  pd.pm().numchannel();
  }

int F77NAME(pmrow)() {
  RICHDetector& pd=*RICH_Detector;
  pd.pm().PMrow(pd.pmChannel());
  }

int F77NAME(pmcol)() {
  RICHDetector& pd=*RICH_Detector;
  pd.pm().PMcol(pd.pmChannel());
  }

float F77NAME(pmpadsize)() {
  RICHDetector& pd=*RICH_Detector;
  pd.pm().type().colsize();
  }


void F77NAME(smcoord)(float *x,float *y,float *z) {
  RICHDetector& pd=*RICH_Detector;
  Vec3 xx=pd.pos_smcoord();
  *x=xx.x();
  *y=xx.y();
  *z=xx.z();
  }

int F77NAME(eventno)()     { return RICH_DAQ->EventNo(); }
int F77NAME(numhit)()     { return RICH_DAQ->NumHit(RICH_Detector->ChannelIndex()); }
int F77NAME(eventhit)()   { return RICH_DAQ->Hit(RICH_Detector->ChannelIndex()); }
int F77NAME(isconnected)() {
  return (RICH_DAQ->Channel(RICH_Detector->ChannelIndex())!=-1);
  }
int F77NAME(channel)() { return RICH_DAQ->Channel(RICH_Detector->ChannelIndex()); }
int F77NAME(connector)()  { return RICH_DAQ->Connector(RICH_Detector->ChannelIndex()); }
int F77NAME(daughter)()   { return RICH_DAQ->DaughterID(RICH_Detector->ChannelIndex()); }
int F77NAME(motherid)()   { return RICH_DAQ->MotherID(RICH_Detector->ChannelIndex()); }
int F77NAME(wind)(int *step)    { return RICH_DAQ->Wind(*step); }

}

