#ifndef __RICHFED_H__
#define __RICHFED_H__
#include "rich/ricutil.h"
#include "rich/FED.h"

// --------------------------------------------------------------------
//#define INDEXCHECK

/// wire map of the M16-base
class WiremapM16 {
int pm[4*16],pmChannel[4*16];
int numchan;

inline int index(int asdid,int asdchannel) const {
  int ret=asdid*16+asdchannel;
#ifdef INDEXCHECK
  if (ret<0||ret>=numchan) {
    fprintf(stderr,"M16base::index out of range ! %d %d %d %d\n",
           asdid,asdchannel,ret,numchan);
    exit(1);
    }
#endif
  return ret;
  }

public:
/// contructor reading the wiring from as ASCII-File
  WiremapM16(ScriptFile& f);
/// returns the pmt (0...3) as function of the asd-ID(0...3) and the ASD-Channel
  inline int  pmt(int asdid,int asdchannel) const {
    return pm[index(asdid,asdchannel)];
    }
/// sets the pmt (0...3) as function of the asd-ID(0...3) and the ASD-Channel
  inline int& pmt(int asdid,int asdchannel) {
    return pm[index(asdid,asdchannel)];
    }
/// returns the pmt-channel (0...15) as function of the asd-ID(0...3) and the ASD-Channel
  inline int  pmtChannel(int asdid,int asdchannel) const {
    return pmChannel[index(asdid,asdchannel)];
    }
/// sets the pmt-channel (0...15) as function of the asd-ID(0...3) and the ASD-Channel
  inline int& pmtChannel(int asdid,int asdchannel) {
    return pmChannel[index(asdid,asdchannel)];
    }
  };

// --------------------------------------------------------------------
//#define INDEXCHECK
/// wire map of the M4-base
class WiremapM4 {
int pm[4*4],pmChannel[4*4];
int numchan;

inline int index(int asdchannel) const {
#ifdef INDEXCHECK
  if (asdchannel<0||asdchannel>=numchan) {
    fprintf(stderr,"M4base::index out of range ! %d %d\n",
           asdchannel,numchan);
    exit(1);
    }
#endif
  return asdchannel;
  }

public:
  WiremapM4(ScriptFile& f);
/// return the pmt (0...3) as function of the ASD-Channel(0..15)
  inline int  pmt(int asdchannel) const {
    return pm[index(asdchannel)];
    }
/// sets the pmt (0...3) as function of the ASD-Channel(0..15)
  inline int& pmt(int asdchannel) {
    return pm[index(asdchannel)];
    }
/// returns the pmt-channel (0...3) as function of the ASD-Channel(0..15)
  inline int  pmtChannel(int asdchannel) const {
    return pmChannel[index(asdchannel)];
    }
/// sets the pmt-channel (0...3) as function of the ASD-Channel(0..15)
  inline int& pmtChannel(int asdchannel) {
    return pmChannel[index(asdchannel)];
    }
  };


// ---------------------------------------------------------------------
/// mapping of the channels belonging to one ASD.
class RICHASD {
protected:
char       serialno[20];
int        motherID,daughterID,connector;
int        sm;
unsigned char       flag[16];
public:
/// return the number of channel of one ASD8-board
  inline int numchannel() const { return 16; }
/// returns ID of the FED-mother to which the ASD is connected to.
  inline int MotherID()            const { return motherID; }
/// returns ID of the FED-daughter to which the ASD is connected to.
  inline int DaughterID()          const { return daughterID;}
/// returns ID of the connector to which the ASD is connected to.
  inline int Connector()           const { return connector;}
/// return the SuperModule, where the ASD is plugged in.
  inline int SuperModule()         const { return sm; }
/// returns the row as function of the ASD-channel
  virtual int Row(int asdchannel)       const=0;
/// returns the column as function of the ASD-channel
  virtual int Col(int asdchannel)       const=0;
/// returns the pmt-channel as function of the ASD-channel
  virtual int PMTChannel(int asdchannel)const=0;
/// return if the channel is dead
  inline int IsDead(int asdchannel) const { return (flag[asdchannel]==0); }
/// return if the channel is properly working
  inline int ChannelWorks(int asdchannel) const { return (flag[asdchannel]==1); }
/// return if the channel is properly working
  inline int IsHot(int asdchannel) const { return (flag[asdchannel]==2); }
/// set the flag
  inline unsigned char& Flag(int i) { return flag[i]; }
/// return the flag as function of the ASD-channel
  inline unsigned char  Flag(int i) const { return flag[i]; }
/// print the contents of the class to an ASCII-File 
  virtual void print(FILE *fp=stdout)=0;
  };


// ---------------------------------------------------------------------
/// decoding of the hits from one ASD on a M16-base

class M16ASD : public RICHASD {
int asdid,r,c;

public:
static WiremapM16 *m16base;
/// constructor: See also page:
/**
The asdid(0...3) and the row and column of the lowerleft corner
(if the y-axis of the supermodule point upwards and the z-axis
is pointing to the observer) has to be given as an argument.
*/
  M16ASD(char *s);
/// return the row in the supermodule to which the ASD-channel is belonging to
  inline int Row(int asdchannel) const {
    return r+m16base->pmt(asdid,asdchannel)/2;
    }
/// return the column in the supermodule to which the ASD-channel is belonging to
  inline int Col(int asdchannel) const  {
    return c+m16base->pmt(asdid,asdchannel)%2;
    }
///  return the pmt-channel to which the ASD-channel is belonging to.
  inline int PMTChannel(int asdchannel) const {
    return m16base->pmtChannel(asdid,asdchannel);
    }
/// print the contents of the class to an ASCII-File.
  void print(FILE *fp=stdout);
  };  


// ---------------------------------------------------------------------
/// decoding of the hits from one ASD on a M4-base

class M4ASD : public RICHASD {
int r,c;
public:
static WiremapM4 *m4base;
/// constructor: See also page:
/**
The row and column of the lowerleft corner
(if the y-axis of the supermodule point upwards and the z-axis
is pointing to the observer) has to be given as an argument.
*/
  M4ASD(char *s);
/// return the row in the supermodule to which the ASD-channel is belonging to
  inline int Row(int asdchannel) const {
    return r+m4base->pmt(asdchannel)/2;
    }
/// return the column in the supermodule to which the ASD-channel is belonging to
  inline int Col(int asdchannel) const {
    return c+m4base->pmt(asdchannel)%2;
    }
///  return the pmt-channel to which the ASD-channel is belonging to.
  inline int PMTChannel(int asdchannel) const {
    return m4base->pmtChannel(asdchannel);
    }
/// print the contents of the class to an ASCII-File.
  void print(FILE *fp=stdout);
  };


// ---------------------------------------------------------------------
/// description of one RICH-FED-system
/**
  In the class RICHFED 64 RICHASD are collected, which are needed
  to interprete the data of one FED--mother. Also theinformation 
  about which ASD respectively channel works is stored here.
 */
class RICHFED : public ChannelInfo {
const RICHASD *asd[64];
int motherid;
public:
/// constructor: See also page:
/**
The mother-id and the list of all ASD has to be supplied. The
constructor searches through the list and collect all ASD's which
are connected to the given mother-id. To decide, which channel work
the efficiency variation factor as given in pm.dat are used. If the
efficiency <=0, the channel is dead, if the efficiency is > 1, then
the channel is hot, else the channel is assumed to work correctly.
*/
RICHFED(int mid,List<RICHASD> ASDList);

/// returns the supermodule-id as function of the position in the bit-map 
inline int SuperModule(int i) const {
  int ret=-1;
  if (asd[i/16]) ret=asd[i/16]->SuperModule(); 
  return ret;
  }

/// returns the row as function of the position in the bit-map 
inline int Row(int i) const { 
  int ret=-1;
  if (asd[i/16]) ret=asd[i/16]->Row(Channel(i)); 
  return ret;
  }

/// returns the column as function of the position in the bit-map 
inline int Col(int i)        const { 
  int ret=-1;
  if (asd[i/16]) ret=asd[i/16]->Col(Channel(i)); 
  return ret;
  }

/// returns the PMT-channel as function of the position in the bit-map 
inline int PMTChannel(int i) const { 
  int ret=-1;
  if (asd[i/16]) ret=asd[i/16]->PMTChannel(Channel(i)); 
  return ret;
  }
/// returns 1 if the channel is dead
inline int IsDead(int i) const      {
  return (asd[i/16]&&asd[i/16]->IsDead(Channel(i)));
  }
/// returns 1 if the channel is hot
inline int IsHot(int i) const {
  return (asd[i/16]&&asd[i/16]->IsHot(Channel(i)));
  }
/// return 1 if channel is properly working.
inline int ChannelWorks(int i) const  {
  return (asd[i/16]&&asd[i/16]->ChannelWorks(Channel(i))); 
  }
/// returns 1 if a ASD is connected to this connector.
inline int ASDWorks(int connector) const { 
  return (asd[connector]!=0); 
  }
/// return the MotherID
inline int MotherID() { return motherid; };

/// print the contents of the class to an ASCII-File
void print(FILE *fp=stdout);
};


// ---------------------------------------------------------------------
//  global RICH variables
/// the root for the RICH-DAQ software
/**
 The class RICHDAQ collects all FED-systems.
 */
class RICHDAQ {
List<RICHASD> ASDList;
RICHFED** FED;

FILE *fp;
unsigned long datastart;
int byteswap;
RunHeader runheader;
OfflineFEDBuffer fedbuffer;

unsigned long  *numhit;
unsigned short *motherID;
unsigned char  *daughter_connector;
unsigned char  *asdchannel_hit;
int numchannel,eventno,imoth,nummother,MaxIndex,eventindex;
unsigned short flteventno,bx;

inline void SetMotherID(int ChannelIndex,unsigned short m) { 
  motherID[ChannelIndex]=m; 
  }

inline void SetDaughterID(int ChannelIndex,int d) { 
  daughter_connector[ChannelIndex]&=((d<<4)|0x0F); 
  }

inline void SetConnector(int ChannelIndex,int  c) { 
  daughter_connector[ChannelIndex]&=(0xF0|c); 
  }

inline void SetASDChannel(int ChannelIndex,int  a) { 
  asdchannel_hit[ChannelIndex]&=((a<<4)&0x0F); 
  }

inline void SetHit(int ChannelIndex,int bit) {
  asdchannel_hit[ChannelIndex]=(0xF0|bit);  
  }


public:

/// constructor: argument is the path, where the configuration file can be found
RICHDAQ(char *path="./");

/// returns the RICHFED which belong to a given mother-id (zero if not found)
inline RICHFED* mother(int motherid) {
  int i;
  for (i=0;i<nummother;i++)
    if (FED[i]->MotherID()==motherid) break;
  if (i==nummother) {
    fprintf(stderr,"mother %d not found. Inconsitent use. Abort.\n",motherid);
    exit(1);
    }
  return FED[i];
  }

/// return the number of FED-mothers
inline int NumMother() const { return nummother; }

/// initialise a loop over all FED-systems
inline void NewMotherIterator() { imoth=0; }

/// looping over all FED-systems
inline RICHFED* NextMother() { return FED[imoth++]; }

/// return the RunHeader
  inline const RunHeader& Header()  const { return runheader; }

/// return the event number
  inline int EventNo()          const { return eventindex; }
/// return the event number
  inline int FLTEventNo()          const { return flteventno; }
/// return the BX number
  inline int BX()          const { return bx; }

/// return the number of triggers
  inline int NumTrigger()          const { return fedbuffer.NumTrigger(); }

/// return MotherID as function of the RICHDetector::ChannelIndex
  inline int MotherID(int ci)   const { return motherID[ci]; }

/// return DaughterID as function of the RICHDetector::ChannelIndex
  inline int DaughterID(int ci) const { return (daughter_connector[ci]>>4);}

/// return Connector as function of the RICHDetector::ChannelIndex
  inline int Connector(int ci)  const { return (daughter_connector[ci]&0x0F);}

/// return ASD-channel as function of the RICHDetector::ChannelIndex
  inline int Channel(int ci)    const { return (asdchannel_hit[ci]>>4);}

/// return the state of the channel
  inline int Hit(int ci)        const { return (asdchannel_hit[ci]&0x0F);}

/// return added number of hits as function of the RICHDetector::ChannelIndex
  inline int NumHit(int ci)     const { return numhit[ci]; }

/// set the added number of hits to zero.
  inline void ZeroNumHit() { memset((char*)numhit,0,numchannel*sizeof(*numhit)); }

/// read next event
  int NextEvent();

/// go forward (n>0) / backward (n<0) by a certain number of events. The number of evevents skipped is returned.
  int Wind(int n);

/// dump event
  inline void dump(FILE* fp=stdout) { fedbuffer.dump(fp); }
  };


extern RICHDAQ *RICH_DAQ;


#endif
