

      SUBROUTINE RICREC

************************************************************************
*
* Created: October 1997 by Ignacio Arinyo
*
* Description: Routine to find the log likelihoods for mass hypothesis:
*             e,mu,pi,k,prot using the RICH.
*             Based on reconstruction algorithm from Thorsten Oest.
*
* Called from: usinit (eventually from rcevnt)
*
* External references: It calls nphoton.F, trackpar.F, lnpadd.F
*
* Argument in/out description:  Output is provisionally written 
*             to table RTRA, to entries ELE, MUO, PI, K, PRO. A new table 
*             will be made specific for RICH output. 
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "auto/EV.fi"
#include "auto/CON.fi"
#include "auto/GEO.fi"
#include "auto/RIC.fi"

#include "rire/ricpar.fi"       

C  VARIABLE DECLARATIONS
C PARAMETERS
      REAL PCUT,MSEL
      PARAMETER(PCUT=3.,MSEL=3.)
      INTEGER NCAMAX
      PARAMETER(NCAMAX=1000)

c COUNTERS
      INTEGER I,J,k,IRTRA,NCA,NCAND,IHYP,IHIT,ICA,ITER

C GEOMETRY
     
      REAL NREFR,EPHOT
      REAL X0,Y0,Z0,P
      REAL R0(3),PR(3),ru(3),rd(3),n(3)
      REAL DX,DY,DZ,LENGTH,lengthu,lengthd
      REAL RHO,NMED,RES,HITRES,MRES,A,MASS,MD

c likelihood
      REAL NGAMMA,LOGPG,PROB,FS,FB,B,PADD,DA,LNOLD,LNL(2)
      REAL LNP,D1LNP,D2LNP,MASSC(NCAMAX),MRESC(NCAMAX),DMASS
      REAL LH(5)
      INTEGER NPHOT,NHIT

      LOGICAL ok

C DATAs

      REAL EMEAN,SIGE
      DATA EMEAN  /3.4/, SIGE   /0.552/

      REAL MHYP(5)
      DATA MHYP / 0.000511, .1056584, .1395699, .493677, .938272 /


#include "arte/Arte.ff"
#include "auto/EV.ff"
#include "auto/CON.ff"
#include "auto/GEO.ff"
#include "auto/RIC.ff"

      NREFR(EPHOT) = 1.0013178 + 1.72422E-7 * EPHOT +
     +                           5.49512E-6 * EPHOT**2


***********************************************************



C TAKE THE RICH PARAMETERS FROM ARTE TABLES

      DO I=1,NGESL()
         IF(IGESL_CMP(I).EQ.5) THEN
            ZRICHMIN=FGESL_Z1(I)
            ZRICHMAX=FGESL_Z2(I)
         ENDIF
      ENDDO

      LRAD=ZRICHMAX-ZRICHMIN

      RADIUS=FCSMI_R(1) !RADIUS FROM TABLE CSMI (ONLY ONE ENTRY) 

      IF (PHOTON.EQ.0) THEN
         PHOTON=35.             !VALUE OF EXPECTED PHOTONS FOR BETA=1 
                                !TRACKS TO BE OPTIMIZED.
      ENDIF

C LOOP OVER RECONSTRUCTED TRACKS 

      IF(NRTRA().EQ.0) THEN
         C78='RICH reconstruction aborted: no reconstructed tracks'
         call me78wa
         GOTO 900
      ENDIF

      if(NRTRA().GT.5000) THEN
         C78='RICH reconstruction aborted: NRTRA() > 5000. '
         call me78wa

         GOTO 900
      ENDIF
         
 
      DO IRTRA=1,NRTRA()


         ok=.true.
         call trackpar(irtra,r0,pr,ok)

         if(.not.ok) goto 850


         IF (R0(3).LT.ZRICHMIN-100.) GOTO 850

         p=sqrt(pr(1)**2+pr(2)**2+pr(3)**2)

c         IF(P.LE.PCUT) GOTO 850


         do i=1,3
            ru(I)=r0(I)
            rd(I)=r0(I)
            n(I)=pr(I)/p
         enddo

 
         call traceupper(ru,n,lengthu)
         call tracelower(rd,n,lengthd)


 
c COUNT CANDIDATE NUMBER

         DO IHYP=1,5
            ncand=0
            DO I=1,NCAMAX
               MASSC(I)=0
               MRESC(I)=0
            ENDDO

            DO IHIT=1,NHITB()
       
               IF (IGEDE_CMP(IHITB_GEDE(IHIT)).NE.5) goto 100
               X0=FHITB_X(IHIT)
               Y0=FHITB_Y(IHIT)
               Z0=FHITB_Z(IHIT)
          
               IF (Y0.GT.0.0) THEN
                  DX  = X0-ru(1)
                  DY  = Y0-ru(2)
                  DZ  = Z0-ru(3)
                  LENGTH = LENGThu
               ELSE
                  DX  = X0-rd(1)
                  DY  = Y0-rd(2)
                  DZ  = Z0-rd(3)
                  LENGTH = LENGThd
               ENDIF
               
               RHO = SQRT(DX**2 + DY**2 +DZ**2)

               NMED   = NREFR(EMEAN)
               RES   = (1.1E-5 * EMEAN * SIGE)**2 + (1.E-5)**2

               HITRES = FHITB_R(IHIT) * RHO/MAX(ABS(DX),ABS(DY))
              
               MRES  = 2. * P**2 * SQRT(RES +
     +              (HITRES/LENGTH)**2 *(NMED**2-1.))

               a = 1. + (RHO/LENGTH)**2
               MASS   = P**2 * (NMED**2-A)/A

               MD  = ( (MHYP(IHYP))**2-MASS )/MRES

               IF (ABS(MD).LE.MSEL.AND.NCAND.LT.NCAMAX) THEN

                  NCAND=NCAND+1
                  MASSC(NCAND) = MASS
                  MRESC(NCAND) = MRES
              ENDIF
               
 100           CONTINUE
            ENDDO



C ESTIMATE NUMBER OF PHOTONS FOR MASS HYPOTHESIS
            CALL NPHOTON(R0,PR,MHYP(IHYP),Nphot)
            ngamma=float(nphot)


            IF (NGAMMA.GT.0.0) THEN
               IF (NCAND.LE.15) THEN
                  PADD   = 1.0
                  LOGPG  = PADD
                  DO J=1,NCAND
                     PADD   = PADD * NGAMMA / J
                     LOGPG  = LOGPG + PADD
                  ENDDO
                  LOGPG = LOG(LOGPG) - NGAMMA
               ELSE
                  PADD    = (NGAMMA-NCAND)**2/NGAMMA
                  IF (NGAMMA.GT.NCAND) THEN
                     LOGPG   = 0.5*PROB(PADD,1)
                  ELSE
                     LOGPG   = 1. - 0.5*PROB(PADD,1)

                     
                  ENDIF
                  LOGPG    = LOG(LOGPG)
               ENDIF
            ELSE
               LOGPG     = 0.0
            ENDIF


            IF (NCAND.GE.1) THEN

               FS  = MAX(0.0,MIN(1.0, NGAMMA/NCAND))
               FB  = 1. - FS
               
               DO J=1,2
                  A = 0.0
                  B = 0.5/MSEL
                  ITER = 0
                  IF (FB.LE.0.0) GOTO 30 
                 
 20               CONTINUE                  

                  LNP   = 0.0
                  D1LNP = 0.0
                  D2LNP = 0.0

                  DO I=1,NCAND

                     DMASS  = MHYP(IHYP)**2-MASSC(I) 
                     MRES   = MRESC(I)

                     CALL LNPADD(DMASS,MRES,FS,FB,A,B,LNP,D1LNP,D2LNP)
                  ENDDO
               
                  IF (D2LNP.EQ.0.0) THEN
                     DA = - 1./D1LNP
                  ENDIF

c maximitze likelihood for A
              
                  DA = - D1LNP/D2LNP
               
                  A = A + DA
               
                  IF (A*MSEL  .LT. -B) THEN
                     A = -B/MSEL
                  ELSEIF (-A*MSEL .LT. -B) THEN
                     A =  B/MSEL
                  ELSEIF (ITER.LE.20.AND.(LNP-LNOLD.GE.0.1
     +                    .OR.ITER.EQ.0)) THEN
                     LNOLD = LNP
                     ITER = ITER + 1
                     GOTO 20
                  ENDIF
               

C     final values
              
 30               CONTINUE

                  LNP   = 0.0
                  D1LNP = 0.0
                  D2LNP = 0.0
                  DO I=1,NCAND

                     DMASS  = MHYP(IHYP)**2 - MASSC(I)
                     MRES   = MRESC(I)

                     CALL LNPADD(DMASS,MRES,FS,FB,A,B,LNP,D1LNP,D2LNP)
                  ENDDO
                              
                  LNL(J) = LNP
                  
                  FS = 0.0
                  FB = 1.0
                  
               ENDDO
               
               LH(IHYP) = LNL(1) - LOGPG - LNL(2)
               
            ELSE
               LH(IHYP) = -LOGPG
            ENDIF
            
         ENDDO


C    WRITE IN THE ARTE TABLE RTRA  (FOR THE MOMENT)

         Q(QRTRA_E(IRTRA))=LH(1)
         Q(QRTRA_MU(IRTRA))=LH(2)
         Q(QRTRA_PI(IRTRA))=LH(3)
         Q(QRTRA_K(IRTRA))=LH(4)
         Q(QRTRA_P(IRTRA))=LH(5)


 850     CONTINUE


      ENDDO


 900  continue
      END






