#ifndef __PHILA_HH__
#define __PHILA_HH__

#include <stdio.h>
#include "rich/ricutil.h"

#define PI_HALF 1.570796



//-----------------------------------------------------------------//
class SinglePhiLambda {

private:
    double row0,col0,lambda0,dldr,dldc,phi0,dpdr,dpdc,d2pdrdc;

public:
    inline double Row0()  const {return row0;}
    inline double& Row0()       {return row0;}

    inline double Col0()  const {return col0;}
    inline double& Col0()       {return col0;}

    inline double  Lambda0() const {return lambda0;}
    inline double& Lambda0()       {return lambda0;}

    inline double  Dldr() const {return dldr;}
    inline double& Dldr()       {return dldr;}

    inline double  Dldc() const {return dldc;}
    inline double& Dldc()       {return dldc;}

    inline double  Phi0() const {return phi0;}
    inline double& Phi0()       {return phi0;}

    inline double  Dpdr() const {return dpdr;}
    inline double& Dpdr()       {return dpdr;}

    inline double  Dpdc() const {return dpdc;}
    inline double& Dpdc()       {return dpdc;}

    inline double  D2pdrdc() const {return d2pdrdc;}
    inline double& D2pdrdc()       {return d2pdrdc;}

    inline double Phi(double row,double col) {
      return phi0+dpdr*(row-row0)+dpdc*(col-col0)
	+d2pdrdc*(row-row0)*(col-col0);
    }

    inline double Lambda(double row,double col) {
      double x=dldr*(row-row0)+dldc*(col-col0),x3=x*x*x;
      return lambda0+x-x3/3;
    }

    inline SinglePhiLambda() {;};
    SinglePhiLambda(ScriptFile& f);
};



//-----------------------------------------------------------------//

class PhiLambda {

    SinglePhiLambda **spl;
    int numsm;

    public:

    inline       int        NumSuperModule() const { return numsm; }

// coefficients for ith supermodule 
    inline const SinglePhiLambda& SPL(int i=0) const {return *spl[i];}
    inline       SinglePhiLambda& SPL(int i=0)       {return *spl[i];}  

// Vector direction to phi, lambda
    inline void Vector2PhiLambda(Vec3 vec,double *phi,double *lambda) {
      Vec3 v=Vec3(normalize(vec));
      *lambda=asin(v.y()); // radians
      *phi=atan(v.x()/sqrt(v.x()*v.x()+v.z()*v.z())); // radians	      
    }

// Supermodule hit coordinate to phi, lambda
    inline void Hit2PhiLambda(Vec3 hit,int sm,int row,int col,
			  double *phi,double *lambda) {
      double effrow=(double)row+hit.y()/36.-.5;
      double effcol=(double)col+hit.x()/36.-.5;
      *phi=SPL(sm).Phi(effrow,effcol);  // radians
      *lambda=SPL(sm).Lambda(effrow,effcol); // radians
    }

// DAQID to phi, lambda
    inline void Daqid2PhiLambda(int sm,int row,int col,int ch,int nch,
				double *phi,double *lambda) {
      double effrow,effcol;
      if (nch==4) {
        effrow=row+1./4.-ch/2/2.;       // magnification = -2
        effcol=col+1./4.-ch%2/2.;       // magnification = -2	
	//cout<<"effcol="<<effcol<<" effrow="<<effrow<<endl;
      }
      else if (nch==16) {
        effrow=row+3./8.-ch/4/4.;       // magnification = -2
        effcol=col+3./8.-ch%4/4.;       // magnification = -2
	//cout<<"effcol="<<effcol<<"   effrow="<<effrow<<endl;
      }
      else {
	fprintf(stderr,"\nNumber of channels different than 4 or 16");
	exit(0);
      }
      *phi=SPL(sm).Phi(effrow,effcol);  // radians
      *lambda=SPL(sm).Lambda(effrow,effcol); // radians	
    }   

    PhiLambda(char* path=0);
};

//-----------------------------------------------------------------//

#endif


