*CMZ :  1.00/01 11/07/96  16.22.48  by   Ivan Belyaev
*CMZ :  1.04/10 15/11/95  08.35.16  by  joerg f. rieling
*-- Author :    joerg f. rieling   13/11/95

c     -----------------------------------------------------------

      subroutine sklmsmo ( p_nkp1, C_nkp1, p_kk , C_kk ,
     &                      zfrom , zto   , sin_k, cos_k, m_k,
     &                      p_nk  , C_nk  , r_nk , dch_k )
C*
C* EXTERNAL reference: DSINV from CERNLIB
C*

c     -- performs a Kalman smoother step.
c     -- Input parameters:
c     --   p_nkp1 = p at meas. k+1 including all measurements n
c     --   C_nkp1 = C "   "     "      "      "       "       "
c     --   p_kk   = filtered p vector at k
c     --   C_kk   = filtered C matrix at C
c     --   zfrom  = z-position of the former  meas.
c     --   zto    = z-position of the current meas.; see dz-def!
c     --   sin_k  = sine   of stereo angle of current meas.
c     --   sin_k  = cosine of stereo angle of current meas.
c     --   m_k    = meas. value k
c     -- Output parameters:
c     --   p_nk   = p at meas. k including all measurements n
c     --   C_nk   = C "   "    "      "     "       "       "
c     --   r_nk   = residual at k     "     "       "       "
c     --   dch_k  = deltaChi2  at k   "     "       "       "

c     -----------------------------------------------------------

#include "arte/Arte.fi"

c     -- formal parameters
      real p_nkp1(4), C_nkp1(4,4), p_kk(4), C_kk(4,4)
      real zfrom    , zto        , sin_k  , cos_k    , m_k
      real p_nk  (4), C_nk  (4,4), r_nk   , dch_k

c     -- local variables
      integer i, j, k, ifail
      real C_kkp1(4,4)

      DOUBLE PRECISION C_kkp1_inv(4,4)

      real A_k   (4,4)
      real mhelp (4,4), vhelp (4)
      real p_kkp1(4)  , RR_nk
      real dz

      REAL AAA,RK

      INTEGER  LENOCC 
      EXTERNAL LENOCC

c     -- constants

#include "susi/zsiconst.fi"
#include "arte/Arte.ff"

c     -- data statements
      data ifail / 0 /
      data mhelp / 16*0 /

c     -- (re)calc extrapolated C, p and gain matrix A

      dz = zfrom - zto ! < 0 when smoothing in pos z - direction

      call sklmpre(p_kk, C_kk, dz, p_kkp1, C_kkp1)

      do i=1, 4
         do j=1, 4
            C_kkp1_inv(i,j) = C_kkp1(i,j)
         end do
      end do
      ifail = 0 

      call Dsinv(4, C_kkp1_inv, 4, ifail)

      IF (ifail.ne.0) THEN
        C78='SUSi==>SKLMSMO:: inversion problems!Skip!'
        CALL MEMESC(128780174,1000,ILARGEST,C78(1:LENOCC(C78)))
        RETURN 
      ENDIF

      do i=1, 4 ! mhelp = F^T * C_kkp1^-1
         mhelp(i,1) = C_kkp1_inv(i,1)
         mhelp(i,2) = C_kkp1_inv(i,2)
         mhelp(i,3) = C_kkp1_inv(i,3) + dz*C_kkp1_inv(i,1)
         mhelp(i,4) = C_kkp1_inv(i,4) + dz*C_kkp1_inv(i,2)
      end do ! i
      do i=1, 4 ! A_k = C_kk * mhelp
         do j=1, 4
            A_k(j,i) = 0.
            do k=1, 4
               A_k(j,i) = A_k(j,i) + C_kk(k,i)*mhelp(j,k)
            end do
         end do
      end do

c     -- calc new C matrix
      do i=1, 4 ! mhelp = ( C^n_k+1 - C^k_k ) * A^T_k
         do j=1, 4
            mhelp (j,i) = 0.
            do k=1, 4
               mhelp(j,i) = mhelp(j,i) +
     &                      (C_nkp1(k,i)-C_kkp1(k,i)) * A_k(k,j)
            end do
         end do
      end do
      do i=1, 4 ! C^n_k = C_kk + A_k*mhelp
         do j=i, 4
            C_nk(j,i) = 0.
            do k=1, 4
               C_nk(j,i) = C_nk(j,i) + A_k(k,i)*mhelp(j,k)
            end do
            C_nk(j,i) = C_nk(j,i) + C_kk(j,i)
         end do
      end do
      C_nk(1,2) = C_nk(2,1)
      C_nk(1,3) = C_nk(3,1)
      C_nk(2,3) = C_nk(3,2)
      C_nk(1,4) = C_nk(4,1)
      C_nk(2,4) = C_nk(4,2)
      C_nk(3,4) = C_nk(4,3)

c     -- calc new p vector
      do i=1, 4 ! vhelp = p^n_k+1 - p^k_k+1
         vhelp(i) = p_nkp1(i) - p_kkp1(i)
      end do
      do i=1, 4 ! p^n_k = p^k_k + A * vhelp
         p_nk(i) = p_kk(i) +
     &             A_k(1,1)*vhelp(1) + A_k(2,1)*vhelp(2) +
     &             A_k(3,1)*vhelp(3) + A_k(4,1)*vhelp(4)
      end do

c     -- calc new dChi2
      r_nk  = m_k - cos_k*p_nk(1) + sin_k*p_nk(2)

c      RR_nk = SI_SIG*SI_SIG - ( C_nk(1,1)*cos_k*cos_k -
c     &                        2*C_nk(2,1)*cos_k*sin_k +
c     &                          C_nk(2,2)*sin_k*sin_k )
c
c      dch_k = r_nk*r_nk / RR_nk
             
      AAA =          C_kk(1,1)*cos_K*cos_K  
     &       - 2.0 * C_kk(1,2)*cos_K*sin_K 
     &       +       C_kk(2,2)*sin_K*sin_K 
      IF( AAA .LE. 0.0 )THEN 
       C78='SKLMSMO, c_kk is not positive defined!'
       CALL MEMESC(921478,1,ILARGEST,C78(1:LENOCC(C78)))
       AAA = ABS( AAA ) 
      ENDIF  
      RK    = SI_SIG**4/(SI_SIG**2+AAA)
      RR_NK = RK -   (      (C_nk(1,1)-C_kk(1,1))*cos_K*cos_K
     &               -2.0*(C_nk(1,2)-C_kk(1,2))*cos_K*sin_K
     &               +1.0*(C_nk(2,2)-C_kk(2,2))*sin_K*sin_K)

      IF( RR_NK .LE. 0.0 )THEN 
       C78='SKLMSMO, RR_NK is not positive defined!'
       CALL MEMESC(921479,1,ILARGEST,C78(1:LENOCC(C78)))
       RR_NK = ABS( RR_NK) 
      ENDIF  

      dch_K = r_nk*r_nk/RR_nk
      return
      end



