*CMZ :          07/11/96  16.34.53  by  T. Oest
*-- Author :    T. Oest   29/10/96

      SUBROUTINE V0FITE(INIT, COVAR, TRK1, TRK2, NZ1, ZMUSC1, ANG1,
     +                  NZ2, ZMUSC2, ANG2, PAR, VPAR, WPAR,
     +                  DCHI2, CHI2V0, IRET)

************************************************************************
*
* Description: returns new vertex/track parameters or the
*              covariance matrix
*
*              this methods uses eigenvalues
*
* Called from:
*
* External references:
*
* Argument in/out description
* INIT     in     = .TRUE.    do initialization
* COVAR    in     = .FALSE.   calculate new parameters
*                 = .TRUE.    calculate covariance matrix
* TRK1     in     first track
* TRK2     in     second track
* NZ1,2    in     number of multiple scattering planes for track 1,2
* ZMUSC1,2 in     zposition of multiple scattering planes for track 1,2
* ANG1,2   in     (multiple scattering angle)**2 for 1 GeV, tx=ty=0
* PAR      out    vertex, track parameters at START
*                 (ZSTART,x,y,tx1,ty1,1/p1,tx2,ty2,1/p2)
* VPAR     out    covariance matrix
* WPAR     out    (VPAR)**-1
* DCHI2    out    increase in CHI2  (COVAR=.FALSE.)
* CHI2V0   out    CHI2 of fit (COVAR=.TRUE.)
* IRET     out    =  0 OK
*                 = -1 chi2 increasing
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "auto/EV.fi"

      LOGICAL          COVAR, INIT
      INTEGER          TRK1, TRK2, IRET, TRK(2), NZ1, NZ2
      DOUBLE PRECISION PAR(9), VPAR(9,9), WPAR(9,9), DCHI2, CHI2V0
      DOUBLE PRECISION ZMUSC1(*), ZMUSC2(*), ANG1(*), ANG2(*)


      INTEGER          I, J, K, L, M, IERROR, INDEX(5,5), IC1, IC2
      LOGICAL          COVMS, SCATT, REDO
      DOUBLE PRECISION VTRK(2,5,5), WTRK(2,5,5)
      DOUBLE PRECISION A(2,5,3), B(2,5,3), W(2,5,5)
      DOUBLE PRECISION E(2,3,3), EINV(2,3,3), DD(2,5), DQ(2,3)
      DOUBLE PRECISION D0(3,3), D(2,3,3)
      DOUBLE PRECISION DX(3)
      DOUBLE PRECISION MA(6,3), MB(6,3), MVMS(6,6), MS(2,5,5)
      DOUBLE PRECISION CHIOLD, PAROLD(9), FAC
      DOUBLE PRECISION PIN(6), POUT(6), ZOUT, DZ, PREC(4)
      DOUBLE PRECISION MW5(5,5), RW5(5), RW3(3), MW3(3,3)
      DOUBLE PRECISION WA(2,5,3), WB(2,5,3), BE(2,5,3), DSUM


      REAL*4           MAT(9,9), EVAL(9), EVEC(9,9), WORK(9)
      DOUBLE PRECISION NVEC(9), DPAR(9)

      SAVE  EVAL, EVEC, D, DD, W, VTRK, WTRK

#include "arte/Arte.ff"
#include "auto/EV.ff"

      DATA INDEX /  1,  2,  4,  7, 11,
     +              2,  3,  5,  8, 12,
     +              4,  5,  6,  9, 13,
     +              7,  8,  9, 10, 14,
     +             11, 12, 13, 14, 15 /

      DATA SCATT /.TRUE./

C-----------------------------------------------------------------
C-----------------------------------------------------------------
C  initialization
C-----------------------------------------------------------------
      IRET   = 0
      TRK(1) = TRK1
      TRK(2) = TRK2

      
      IF (PRTRX_HITB(TRK1).EQ.0) THEN
       IC1 = 0
      ELSE
       IC1 = 1
      ENDIF
      IF (PRTRX_HITB(TRK2).EQ.0) THEN
       IC2 = 0
      ELSE
       IC2 = 1
      ENDIF


      REDO = .FALSE.
C-----------------------------------------------------------------
C  covariance matrix
C-----------------------------------------------------------------
c      IF (INIT) THEN

       DO I=1,2

         DO J=1,5
         DO K=J,5
          MW5(J,K) = DBLE(FRTRA_CVF(INDEX(J,K), TRK(I)))
          MW5(K,J) = MW5(J,K)
          VTRK(I,J,K) = MW5(J,K)
          VTRK(I,K,J) = MW5(K,J)
         ENDDO
        ENDDO

         CALL DINV(5, MW5, 5, RW5, IERROR)
         IF (IERROR.NE.0) THEN
          IRET = -2
          GOTO 9000
         ENDIF

         DO J=1,5
         DO K=1,5
          WTRK(I,J,K) = MW5(J,K)
         ENDDO
         ENDDO

       ENDDO
c       GOTO 9000

c      ENDIF
C-----------------------------------------------------------------
C  new parameter or covariance matrix ?
C-----------------------------------------------------------------
      IF (COVAR) GOTO 1000
C-----------------------------------------------------------------
C  swim track parameter to measured tracks
C         --> DD, A, B
C-----------------------------------------------------------------

10    CONTINUE


      PIN(1) = PAR(1)
      PIN(2) = PAR(2)
      PIN(3) = PAR(3)

      DO I=1,2

       PIN(4) = PAR(1+I*3)
       PIN(5) = PAR(2+I*3)
       PIN(6) = PAR(3+I*3)
       ZOUT   = DBLE(FRTRA_ZF(TRK(I)))
       DZ     = 10. * ABS(PIN(1) - ZOUT)
       PREC(1)= SQRT(FRTRA_CVF( 1,TRK(I))) / DZ
       PREC(2)= SQRT(FRTRA_CVF( 3,TRK(I))) / DZ
       PREC(3)= SQRT(FRTRA_CVF( 6,TRK(I))) / DZ
       PREC(4)= SQRT(FRTRA_CVF(10,TRK(I))) / DZ

       IF (I.EQ.1) THEN
        CALL V0FEXT(IC1, SCATT, NZ1, ZMUSC1, ANG1, PIN, ZOUT, PREC, 
     +              POUT, MA, MB, COVMS, MVMS, IERROR)
       ELSE
        CALL V0FEXT(IC2, SCATT, NZ2, ZMUSC2, ANG2, PIN, ZOUT, PREC, 
     +              POUT, MA, MB, COVMS, MVMS, IERROR)
       ENDIF
       IF (IERROR.NE.0) THEN
        IRET = -3
        GOTO 9000
       ENDIF

       DO J=1,5
        DO K=1,3
         A(I,J,K)  = MA(J+1,K)
         B(I,J,K)  = MB(J+1,K)
        ENDDO
       ENDDO
       DO J=1,5
       DO K=1,5
         MS(I,J,K) = MVMS(J+1,K+1)
       ENDDO
       ENDDO

       DD(I,1) = FRTRA_XF (TRK(I)) - POUT(2)
       DD(I,2) = FRTRA_YF (TRK(I)) - POUT(3)
       DD(I,3) = FRTRA_TXF(TRK(I)) - POUT(4)
       DD(I,4) = FRTRA_TYF(TRK(I)) - POUT(5)
       DD(I,5) = FRTRA_PF (TRK(I)) - POUT(6)


      ENDDO

C-----------------------------------------------------------------
C  fill covariance matrices
C-----------------------------------------------------------------
        DO J=1,3
        DO K=1,3
         D0(J,K) = 0.0
        ENDDO
        ENDDO

C ---   W1, W2

        DO I=1,2

         IF (COVMS) THEN
          DO J=1,5
          DO K=J,5
           MW5(J,K) = VTRK(I,J,K) + MS(I,J,K)
           MW5(K,J) = MW5(J,K)
          ENDDO
          ENDDO

          CALL DINV(5, MW5, 5, RW5, IERROR)
          IF (IERROR.NE.0) THEN
           IRET = -4
           GOTO 9000
          ENDIF

          DO J=1,5
          DO K=1,5
           W(I,J,K) = MW5(J,K)
          ENDDO
          ENDDO

         ELSE
          DO J=1,5
          DO K=1,5
           W(I,J,K) = WTRK(I,J,K)
          ENDDO
          ENDDO
         ENDIF


C ---   WA, WB
        DO J=1,5
        DO K=1,3
         WA(I,J,K) = 0.0
         WB(I,J,K) = 0.0
         DO L=1,5
          WA(I,J,K) = WA(I,J,K) + W(I,J,L) * A(I,L,K)
          WB(I,J,K) = WB(I,J,K) + W(I,J,L) * B(I,L,K)
         ENDDO
        ENDDO
        ENDDO

C --->  E, D0

        DO J=1,3
        DO K=1,3
         D(I,J,K) = 0.0
         MW3(J,K) = 0.0
         DO L=1,5
          D0(J,K)  = D0(J,K)  + A(I,L,J) * WA(I,L,K)
          D(I,J,K) = D(I,J,K) + A(I,L,J) * WB(I,L,K)
          MW3(J,K) = MW3(J,K) + B(I,L,J) * WB(I,L,K)
         ENDDO
         E(I,J,K) = MW3(J,K)
        ENDDO
        ENDDO



        ENDDO


C-----------------------------------------------------------------
C eigenvector
C-----------------------------------------------------------------
      DO I=1,3
      DO J=1,3
       MAT(I  ,J  ) = SNGL(D0(I,J))
       MAT(I  ,J+3) = SNGL(D(1,I,J))
       MAT(I  ,J+6) = SNGL(D(2,I,J))
       MAT(I+3,J+3) = SNGL(E(1,I,J))
       MAT(I+6,J+6) = SNGL(E(2,I,J))
       MAT(I+3,J+6) = 0.0
      ENDDO
      ENDDO

      DO I=1,9
      DO J=I+1,9
       MAT(J,I) = MAT(I,J)
      ENDDO
      ENDDO

      CALL EISRS1(9,9,MAT,EVAL,EVEC,IERROR,WORK)
      IF (IERROR.NE.0) THEN
       IRET = -1
       GOTO 9000
      ENDIF

      DO I=1,3
       NVEC(I)   = 0.0
       NVEC(I+3) = 0.0
       NVEC(I+6) = 0.0
       DO J=1,5
        NVEC(I) = NVEC(I) + WA(1,J,I) * DD(1,J) + WA(2,J,I) * DD(2,J)
        NVEC(I+3) = NVEC(I+3) + WB(1,J,I) * DD(1,J)
        NVEC(I+6) = NVEC(I+6) + WB(2,J,I) * DD(2,J)
       ENDDO
      ENDDO


      DO I=1,9
       DPAR(I) = 0.0
      ENDDO
 
      DCHI2 = 0.0

      DO I=1,9
       IF (EVAL(I).GT.0.01) THEN
        FAC = 0.0
        DO J=1,9
         FAC = FAC + EVEC(J,I) * NVEC(J)
        ENDDO
        DO J=1,9
         DPAR(J) = DPAR(J) + FAC/EVAL(I) * EVEC(J,I)
        ENDDO

        DCHI2 = DCHI2 + FAC**2 / EVAL(I)

       ENDIF
      ENDDO

C-----------------------------------------------------------------
C new parameters and chi2 decrease
C-----------------------------------------------------------------
        DO I=1,9
         PAR(I)   = PAR(I)   + DPAR(I)
        ENDDO


      CHI2V0 = 0.0
      DO I=1,5
      DO J=1,5
       CHI2V0 = CHI2V0 + DD(1,I)*W(1,I,J)*DD(1,J)
     +                 + DD(2,I)*W(2,I,J)*DD(2,J)
      ENDDO
      ENDDO

      IF (.NOT. COVAR) GOTO 9000
C-----------------------------------------------------------------
C  covariance matrix
C-----------------------------------------------------------------
 1000 CONTINUE


      DO I=1,9
      DO J=I,9
       VPAR(I,J) = 0.0D0
       DO K=1,9
        VPAR(I,J) = VPAR(I,J) + EVEC(I,K)*EVEC(J,K)/MAX(EVAL(K),0.001)
       ENDDO
       VPAR(J,I) = VPAR(I,J)
      ENDDO
      ENDDO

      DO I=1,3
      DO J=1,3

       WPAR(I  ,J  ) = D0(I,J)
       WPAR(I+3,J  ) = D(1,J,I)
       WPAR(I+6,J  ) = D(2,J,I)
       WPAR(I  ,J+3) = D(1,I,J)
       WPAR(I  ,J+6) = D(2,I,J)
       WPAR(I+3,J+3) = E(1,I,J)
       WPAR(I+6,J+6) = E(2,I,J)
       WPAR(I+6,J+3) = 0.0D0
       WPAR(I+3,J+6) = 0.0D0
      ENDDO
      ENDDO

      CHI2V0 = 0.0
      DO I=1,5
      DO J=1,5
       CHI2V0 = CHI2V0 + DD(1,I)*W(1,I,J)*DD(1,J)
     +                 + DD(2,I)*W(2,I,J)*DD(2,J)
      ENDDO
      ENDDO


C-----------------------------------------------------------------
C  the end
C-----------------------------------------------------------------


 9000 END

************************************************************************
* End of V0FITI
************************************************************************
