*-- Author :    T. Oest   24/10/96

      SUBROUTINE V0HITW(IWALL, DP1, DP2, XOUT, YOUT, ZOUT, MUSC, IRET)

************************************************************************
*
* Description: point where the track hits a wall 
*              tracks are supposed to be straight tracks
*
* Called from:
*
* External references:
*
* Argument in/out description
* IWALL    in     wall number
* DP1      in     first  point (z,x,y,tx,ty,rho)
* DP2      in     second point (z,x,y,tx,ty,rho)
* XOUT     out    x position where track hits wall
* YOUT     out    y position where track hits wall
* ZOUT     out    z position where track hits wall
* MUSC     out    multiple scattering angle for a 1 GeV track
* IRET     out    =  1  trajectory hits pipe
*                 =  0  trajectory doesn't hit wall
*                 = -1  error
*
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "auto/GWAL.fi"

      INTEGER          IWALL, IRET
      DOUBLE PRECISION DP1(6), DP2(6), XOUT, YOUT, ZOUT, MUSC


      INTEGER          I
      REAL*4           P1(6), P2(6)
      REAL*4           X, Y, Z, ZWALL, XROT, YROT, ZROT, FAC
      REAL*4           ROT(3,3), RW(3)

#include "arte/Arte.ff"
#include "auto/GWAL.ff"

*-----------------------------------------------------------------------
* initialization
*-----------------------------------------------------------------------
      IRET = 0
      IF (IWALL.LE.0 .OR. IWALL.GT.NGWAL()) THEN
       IRET = -1
       GOTO 9000
      ENDIF

      DO I=1,6
       P1(I) = SNGL(DP1(I))
       P2(I) = SNGL(DP2(I))
      ENDDO

      ZWALL = 0.5*(FGWAL_ZMIN(IWALL)+FGWAL_ZMAX(IWALL))

      IF (  P1(1).EQ.P2(1) .OR.
     +     (P1(1).GT.ZWALL .AND. P2(1).GT.ZWALL) .OR.
     +     (P1(1).LT.ZWALL .AND. P2(1).LT.ZWALL)) THEN
       IRET = 0
       GOTO 9000
      ENDIF
*-----------------------------------------------------------------------
* track point for z=zwall
*-----------------------------------------------------------------------
      FAC = (ZWALL-P1(1))/(P2(1)-P1(1))
      X = P1(2) + FAC * (P2(2)-P1(2))
      Y = P1(3) + FAC * (P2(3)-P1(3))
*-----------------------------------------------------------------------
* rotate the point
*-----------------------------------------------------------------------
      XROT = X * FGWAL_ROT(1,IWALL) - Y * FGWAL_ROT(2,IWALL)
      YROT = X * FGWAL_ROT(2,IWALL) + Y * FGWAL_ROT(1,IWALL)

      IF (XROT.GE.FGWAL_XMIN(IWALL) .AND. XROT.LE.FGWAL_XMAX(IWALL).AND.
     +    YROT.GE.FGWAL_YMIN(IWALL) .AND. YROT.LE.FGWAL_YMAX(IWALL))THEN
       IRET = 1
       XOUT = DBLE(X)
       YOUT = DBLE(Y)
       ZOUT = DBLE(ZWALL)
       MUSC = DBLE( FGWAL_MS(IWALL) * 
     +        SQRT(((P2(1)-P1(1))**2+(P2(2)-P1(2))**2+(P2(3)-P1(3))**2)/
     +              (P2(1)-P1(1))**2))
      ELSE
       IRET = 0
      ENDIF
*-----------------------------------------------------------------------
* print information
*-----------------------------------------------------------------------
c      print*,' ********** v0sidW test ******************', iret
c      print*,' X,Y,Z ',X,Y,ZWALL
*-----------------------------------------------------------------------
* the end
*-----------------------------------------------------------------------
 9000 END
